import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
 
public class InstallColab implements ActionListener {

    JFrame     frame;
    JTextField dirText, hostText;
    JButton    dirButton, okButton;
    JTextArea  infoText;
    
    public InstallColab() {

        frame = new JFrame("Colab: ");
	Container container = frame.getContentPane();
	container.setLayout(new BorderLayout());

	JPanel panel = new JPanel (new BorderLayout());
	JLabel label = new JLabel ("Instalacao do CoLab", SwingConstants.CENTER);
	label.setForeground (Color.black);
	label.setBorder (BorderFactory.createEmptyBorder(5, 0, 10, 0));
	panel.add (label, BorderLayout.NORTH);

	JPanel column = new JPanel (new GridLayout (0,1));
	label = new JLabel ("Diretorio onde sera instalado o CoLab: ");
	label.setBorder (BorderFactory.createEmptyBorder(5, 0, 5, 0));
	column.add (label);
	label = new JLabel ("Nome do servidor: ");
	label.setBorder (BorderFactory.createEmptyBorder(5, 0, 5, 0));
	column.add (label);
	
	panel.add (column, BorderLayout.WEST);

	column = new JPanel (new GridLayout (0,1));
	JPanel line = new JPanel(new BorderLayout());

	dirText = new JTextField("");
	line.add (dirText, BorderLayout.CENTER);
	dirButton = new JButton ("Buscar");
	dirButton.setActionCommand ("search");
	dirButton.addActionListener(this);
	line.add (dirButton, BorderLayout.EAST);

	column.add (line);

	hostText = new JTextField("");
	column.add (hostText, BorderLayout.CENTER);

	panel.add (column, BorderLayout.CENTER);
	
	okButton = new JButton ("OK");
	okButton.setActionCommand ("ok");
	okButton.addActionListener(this);
	panel.add (okButton, BorderLayout.SOUTH);

	container.add (panel, BorderLayout.NORTH);

	
	infoText = new JTextArea(15,70);
	JScrollPane pane = new JScrollPane (infoText);
	pane.setBorder (BorderFactory.createEtchedBorder(Color.white, Color.black));
	container.add (pane, BorderLayout.CENTER);

	frame.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		System.exit(0);
	    }
        });
	
	frame.pack();
	frame.setVisible(true);
    }

    public void actionPerformed(ActionEvent evt) {

	System.out.println ("");

	if (evt.getActionCommand().equals("search")) {
	
	    JFileChooser fileChooser = new JFileChooser();
	    //fileChooser.setApproveButtonText( "Open");
	    fileChooser.setFileSelectionMode (JFileChooser.DIRECTORIES_ONLY);

	    int returnVal = fileChooser.showOpenDialog(frame);

	    if (returnVal == JFileChooser.APPROVE_OPTION) {

		File file = fileChooser.getSelectedFile();
		dirText.setText (file.getAbsolutePath());
		
	    }
	} else if (evt.getActionCommand().equals("ok")) {
	    File file= new File (dirText.getText());

	    this.create (file);
	    File persistentDir = new File (file, "persistent");
	    this.create (persistentDir);
	    this.create (new File (persistentDir, "home"));
	    File forumDir = new File (persistentDir, "forum");
	    this.create (forumDir);
	    this.create (new File (forumDir, "msgs"));
	    this.create (new File (forumDir, "users"));
	    File mailDir = new File (persistentDir, "mail");
	    this.create (mailDir);
	    this.create (new File (mailDir, "allmails"));
	    this.create (new File (mailDir, "mailbox"));

	    this.createPolicyFile (file, persistentDir);
	    this.createTestScript (file, persistentDir);
	    this.createScript (file, persistentDir);
	    this.createDownScript (file);
	    this.createClientScript (file);
	}
    }

    public void createTestScript (File root, File persistentDir) {

	infoText.append ("Gravando script de teste do servidor: \n\n");
	File policyFile = new File (root, "server2.bat");
	try {
	    
	    PrintStream out = new PrintStream (new FileOutputStream (policyFile));

	    String host       = " -Djava.rmi.server.hostname=" + hostText.getText() + " ";
	    String security   = " -Djava.security.policy=java.policy ";
	    String persistent = " -Dpersistence.directory=" + persistentDir.getAbsolutePath();
	    String classpath  = " -cp CoLab.jar ";
	    String program    = " colab.server.ColabServer " +  hostText.getText();

	    if (! persistent.endsWith (File.separator))
		persistent = persistent + File.separator;
		
	    this.save (out, "java " + host + security + persistent + classpath + program);

	    out.close();

	    infoText.append ("\tok\n\n");

	} catch (Exception e) {

	    infoText.append (e.getMessage());

	}
    }

    public void createScript (File root, File persistentDir) {

	infoText.append ("Gravando script principal: \n\n");
	File policyFile = new File (root, "server.bat");
	try {
	    
	    PrintStream out = new PrintStream (new FileOutputStream (policyFile));

	    String host       = " -Djava.rmi.server.hostname=" + hostText.getText() + " ";
	    String security   = " -Djava.security.policy=java.policy ";
	    String persistent = " -Dpersistence.directory=" + persistentDir.getAbsolutePath();
	    String classpath  = " -cp CoLab.jar ";
	    String program    = " colab.server.ColabServer " +  hostText.getText();

	    if (! persistent.endsWith (File.separator))
		persistent = persistent + File.separator;
		
	    this.save (out, "javaw " + host + security + persistent + classpath + program);

	    out.close();

	    infoText.append ("\tok\n\n");

	} catch (Exception e) {

	    infoText.append (e.getMessage());

	}
    }

    public void createDownScript (File root) {

	infoText.append ("Gravando script de derrubada do servidor: \n\n");
	File policyFile = new File (root, "downserv.bat");
	try {
	    
	    PrintStream out = new PrintStream (new FileOutputStream (policyFile));

	    String classpath  = " -cp CoLab.jar ";
	    String program    = " colab.server.ColabServerDown " +  hostText.getText();

	    this.save (out, "java " + classpath + program);

	    out.close();

	    infoText.append ("\tok\n\n");

	} catch (Exception e) {

	    infoText.append (e.getMessage());

	}
    }

    public void createClientScript (File root) {

	infoText.append ("Gravando script do cliente: \n\n");
	File policyFile = new File (root, "client.bat");
	try {
	    
	    PrintStream out = new PrintStream (new FileOutputStream (policyFile));

	    String classpath  = " -cp CoLab.jar ";
	    String program    = " colab.client.gui.Colab " + hostText.getText();

	    this.save (out, "java " + classpath + program);

	    out.close();

	    infoText.append ("\tok\n\n");

	} catch (Exception e) {

	    infoText.append (e.getMessage());

	}
    }


    public void createPolicyFile (File root, File persistentDir) {

	infoText.append ("Gravando policy.file: \n\n");
	File policyFile = new File (root, "java.policy");
	try {
	    
	    PrintStream out = new PrintStream (new FileOutputStream (policyFile));
	    
	    String persist = persistentDir.getAbsolutePath();
	    if (! persist.endsWith (File.separator))
		persist = persist + File.separator;
	    for (int i = 0; i < persist.length(); i++){
		if (persist.substring (i, i+1).equals("\\")) {
		    // need double separator in the string to be interpreted as one \ -> \\
		    persist = persist.substring (0, i+1) + persist.substring (i);
		    i++;
		}
	    }
	    
	    this.save (out, "grant {");
	    this.save (out, "    permission java.util.PropertyPermission \"persistence.directory\", \"read\";");
	    this.save (out, "    permission java.util.PropertyPermission \"user.dir\", \"read\";");
	    this.save (out, "    permission java.net.SocketPermission \"*:1024-65535\", \"connect,accept\";");
	    this.save (out, "    permission java.net.SocketPermission \"*:80\", \"connect\";");
	    this.save (out, "    permission java.io.FilePermission");
	    this.save (out, "                  \"" + persist + "-\", \"read,write\";");
	    this.save (out, "};");

	    out.close();
	    
	    infoText.append ("\tok\n\n");

	} catch (Exception e) {

	    infoText.append (e.getMessage());

	}
    }

    public void save (PrintStream out, String text) {

	infoText.append (text + "\n");
	out.println (text);

    }

    public void create (File file) {
	
	if (! file.exists()) {
	    
	    infoText.append ("Criando diretorio "+file.getAbsolutePath() + "\n");
	    file.mkdirs();
	    infoText.append ("\tok\n\n");

	}

    }

    public static void main (String args[]) {

	InstallColab install = new InstallColab();

    }
}
