package colab;

import java.rmi.*;
import java.rmi.server.*;
import java.util.*;
import colab.util.*;
import colab.server.*;
import colab.util.group.*;
import colab.util.chat.*;
import java.io.*;
import colab.server.chat.Room;
import colab.server.chat.ChatUser;

public class ChatServer extends UnicastRemoteObject
                          implements iChatServer
{

    public static final String ALL_USERS_STRING = "TODOS";
    private Hashtable usersAddr;
    private Map rooms;

    private static String CLASS_ROOM_NAME = "Toda Turma";

    ThreadGroup roomThreadGroup;
    boolean ServerGoingDown = false;

    iGroupServer groupServer;

    /**
     * @link aggregation
     * @clientCardinality 1
     * @supplierCardinality 1..* 
     */
    private colab.Room lnkRoom;

    public ChatServer(String host, iGroupServer _groupServer) throws RemoteException, IOException {
        super();

        groupServer = _groupServer;
        usersAddr = new Hashtable();
        rooms = Collections.synchronizedMap(new HashMap());
        roomThreadGroup= new ThreadGroup("Chat Rooms");

	this.createRoom (this.CLASS_ROOM_NAME);

        String name = "//" + host + ":" + RegistryPort.getPort() + "/" + iChatServer.NAME;
        Naming.rebind(name, (iChatServer) this);
    }


    public void exit() throws RemoteException {
        // avisar todas as salas (Room) de chat da saida
        Room[] arrayRoom;

        ServerGoingDown = true;

        synchronized(rooms) {
            arrayRoom = new Room[rooms.size()];
            Collection col = rooms.values();
            arrayRoom = (Room[]) col.toArray(arrayRoom);

            for (int i = 0; i < arrayRoom.length; i++)
                arrayRoom[i].exit();
        }
    }  // exit

    public void subscribe (String _login, iChatClient _client) throws RemoteException, ServerIsGoingDownException {
        //Inclui este cliente na lista de clientes

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

	User user = groupServer.getUserInformation (_login);
	if (user != null) {
	    String group = user.getGroup().getName();

	    //Verify (and create) if the room of this group exist.
	    if (! rooms.containsKey (group))
		this.createRoom(group);

	    String type =User.getNameType (user.getType());
	    usersAddr.put (_login, new ChatUser (_login, group, type, _client));
	}

        System.out.println ("Inscrevi cliente: " + _login);
    }


    private void createRoom (String _roomName) {

	try {
	    Room   room = new Room (_roomName, this);
	    new    Thread (roomThreadGroup, room, _roomName).start();
	    rooms.put (_roomName, room);
	} catch (Exception e){
	    ErrorLog.logError ("colab.server.chat.ChatServer.createRoom()", "Nao foi possivel criar sala de chat: " + _roomName, e);
	    System.out.println ("server.chat.ChatServer.createRoom()"+e.getMessage());
                    }

    }


    public void subscribeTemp (String _name, String _group, iChatClient _client) throws RemoteException, ServerIsGoingDownException {
        //Inclui este cliente na lista de clientes

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        //System.out.println ("Inscrevi cliente: " + _name );
        ChatUser client = new ChatUser (_name, _group, "tipo", _client);

        usersAddr.put (_name, client);
    }


    public String[] getRooms() throws RemoteException, ServerIsGoingDownException {
        // return the names of the rooms

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        String roomList[] = new String[rooms.size()];
        roomList = (String[]) rooms.keySet().toArray(roomList);

        return roomList;
    }


    public String[] getRooms(String _login) throws RemoteException, ServerIsGoingDownException {
        // return the names of the rooms
	if (ServerGoingDown)
            throw new ServerIsGoingDownException();

	User user = groupServer.getUserInformation (_login);
	String group = user.getGroup().getName();

	String roomList[] = new String[2];
	roomList[0] = user.getGroup().getName();
	roomList[1] = CLASS_ROOM_NAME;

        return roomList;
    }


    public iRoom getRoom (String _room) throws RemoteException, ServerIsGoingDownException{
        //System.out.println ("Sala solicitada: " + _room);

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        iRoom r = (iRoom) rooms.get(_room);

        return r;
    }


    protected ChatUser getUser (String _name) {
        return (ChatUser) usersAddr.get (_name);
    }

}
