package colab.client;

import java.io.*;
import java.rmi.*;
import java.rmi.server.*;
import javax.swing.filechooser.*;
import javax.swing.text.StyledDocument;
import colab.util.*;
import colab.util.file.*;
import colab.client.*;
import colab.util.tool.iToolModel;

public class ColabFileSystem extends FileSystemView {

    private String loginName;

    private iFileServer       fileServer;
    private iRemoteFileSystem remoteFileSystem;

    public ColabFileSystem(boolean _load) throws IOException {

	//System.out.println ("Criando ColabFileSystem (proxy)");

	ColabProperties properties = PropertiesOwner.getProperties();

	this.loginName = properties.getLoginName();
	String host = properties.getHost();

	try {

	    String name = "//" + host + ":" + RegistryPort.getPort() + "/" + iFileServer.NAME;
	    fileServer = (iFileServer) Naming.lookup(name);

	    remoteFileSystem = fileServer.getFileSystem (loginName, _load);

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.contructor(): " + e.getMessage());
	    //e.printStackTrace();
	    throw new IOException ("Nao foi possivel contato com o servidor de arquivos.");

	}

    }


    public File createFileObject(File dir, String filename) {

	//System.out.println ("createFileObject(File dir, filename)");

	try {
	    File f = remoteFileSystem.createFileObject (dir, filename);
	    return f;

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.createFileObject(File,String): " + e.getMessage());
	    //e.printStackTrace();

	}

	return null;
    }

    public File createFileObject(String path) {

	//System.out.println ("createFileObject (" +path+")");
	if (path.indexOf(":") != -1) {
	    //System.out.println ("\tretornando null");
	    File[] files = this.getRoots();
	    return files[0];
	}

	try {

	    File file = remoteFileSystem.createFileObject (path);
	    //if (file != null)
	    //System.out.println ("\t" + file.toString());
	    return file;

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.createFileObject(String): " + e.getMessage());
	    //e.printStackTrace();

	}

	return null;
    }

    public File createNewFolder(File containingDir) {

	//System.out.println ("createNewFolder");

	try {

	    File f = remoteFileSystem.createNewFolder (containingDir);
	    return f;

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.createNewFolder(): " + e.getMessage());
	    //e.printStackTrace();

	}

	return null;

    }

    public File[] getFiles(File dir, boolean useFileHiding) {

	//System.out.println ("getFiles (" + dir.getAbsolutePath() + ", " + useFileHiding+")");

	try {

	    File[] files = remoteFileSystem.getFiles (dir, useFileHiding);
	    return files;

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.getFiles(): " + e.getMessage());
	    //e.printStackTrace();

	}

	return null;

    }

    public static FileSystemView getFileSystemView() {

	//System.out.println ("===================");
	//System.out.println ("getFileSystemView()");
	//System.out.println ("==================");

	return null;

    }

    public File getHomeDirectory() {

	//System.out.println ("getHomeDirectory");

	try {

	    File file = remoteFileSystem.getHomeDirectory ();
	    //if (file != null)
	    //System.out.println ("\t" + file.toString());
	    return file;

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.getHomeDirectory(): " + e.getMessage());
	    //e.printStackTrace();

	}

	return null;

    }

    public File getParentDirectory(File dir) {

	//System.out.println ("getParentDirectory (" + dir.getAbsolutePath() +")");

	try {

	    File file = remoteFileSystem.getParentDirectory (dir);
	    return file;

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.getParentDirectory(): " + e.getMessage());
	    //e.printStackTrace();

	}

	return null;
    }

    public File[] getRoots() {

	//System.out.println ("getRoots:");

	try {

	    File[] files = remoteFileSystem.getRoots ();
	    //for (int i = 0; i < files.length; i++)
	    //System.out.println ("\t" + files[i].toString());
	    return files;

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.getRoots(): " + e.getMessage());
	    //e.printStackTrace();

	}

	return null;

    }


    public boolean isHiddenFile(File f) {

	//System.out.println ("isHiddenFiles");

	try {

	    boolean isHidden = remoteFileSystem.isHiddenFile (f);
	    return isHidden;

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.isHiddenFile(): " + e.getMessage());
	    //e.printStackTrace();

	}

	return false;
    }

    public boolean isRoot(File f) {

	//System.out.println ("isRoot (" + f.getAbsolutePath()+")");

	if (f.getAbsolutePath() != "/") {
	    //System.out.println ("\tretornando false");
	    return false;
	}

	try {

	    boolean isRoot = remoteFileSystem.isRoot (f);
	    //System.out.println ("\t" + isRoot);
	    return isRoot;

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.isRoot(): " + e.getMessage());
	    e.printStackTrace();

	}

	return false;

    }


    public void saveModel (File _file, iToolModel _model) throws IOException {

	try {

	    remoteFileSystem.saveModel (_file, _model);

	} catch (Exception e) {
	    System.out.println ("colab.client.ColabFileSystem.saveModel(): " + e.getMessage());
	    throw new IOException ("Nao foi possivel salvar o arquivo.\n" + e.getMessage());
	}

    }

    public void saveStyledDocument (File _file, StyledDocument _document) throws IOException {

	//System.out.println ("FileSystem.saveStyledDocument: " + _file.getAbsolutePath());

	try {

	    remoteFileSystem.saveStyledDocument (_file, _document);

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.saveStyledDocument(): " + e.getMessage());
	    e.printStackTrace();
	    throw new IOException ("Nao foi possivel salvar o arquivo.\n" + e.getMessage());

	}

    }


    public void publishModel (iToolModel _model) throws IOException {

	try {

	    remoteFileSystem.publishModel (_model);

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.publishModel(): " + e.getMessage());
	    throw new IOException ("Nao foi possivel salvar o arquivo.\n" + e.getMessage());

	}

    }


    public iToolModel openModel (File _file) throws IOException {

	try {

	    iToolModel model = remoteFileSystem.openModel (_file);
	    return model;

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.openModel (): " + e.getMessage());
	    throw new IOException ("Nao foi possivel ler o arquivo.\n" + e.getMessage());

	}

    }

    public StyledDocument openStyledDocument (File _file) throws IOException {

	//System.out.println ("FileSystem.open: " );

	try {

	    StyledDocument model = remoteFileSystem.openStyledDocument (_file);
	    return model;

	} catch (Exception e) {

	    System.out.println ("colab.client.ColabFileSystem.openStyledDocument (): " + e.getMessage());
	    //e.printStackTrace();
	    throw new IOException ("Nao foi possivel ler o arquivo.\n" + e.getMessage());

	}

    }
}
