package colab.client.group;

import colab.util.group.*;
import colab.util.*;
import javax.swing.table.AbstractTableModel;
import java.rmi.*;
import java.rmi.server.*;
import java.util.*;
import colab.util.chat.*;
import colab.client.*;
import colab.client.gui.*;

public class UsersDataModel extends AbstractTableModel implements iGroupClient, iChatListener {

    // By extending AbstractTableModel, instead of
    // implementing TableModel yourself,
    // AbstractTableModel takes care of
    // TableModelListener list management

    String loginName, serverName;
    iGroupServer groupServer;

    Vector users;
    Map usersMap;
    String header[] = {"Nome", "Tipo", "Grupo", "Sala de Chat"};
    private int numColumns = header.length;


    public UsersDataModel () throws Exception {

        System.out.println ("\tUsersDataModel");

	ColabProperties properties = PropertiesOwner.getProperties();
	this.loginName  = properties.getLoginName();
	this.serverName = properties.getHost();

	//make this object exportable
	UnicastRemoteObject.exportObject ((iGroupClient) this);
        System.out.print (" ...");

	// reference to group server
	String name = "//" + this.serverName + ":" + RegistryPort.getPort() + "/" + iGroupServer.NAME;
	groupServer = (iGroupServer) Naming.lookup(name);

        System.out.print (" ...");
	// Listen chat panel to inform to group server what room the user is
	ModulesOwner.getChatPanel().addChatListener (this);

	UserInformation[] usersInf = groupServer.subscribe(this.loginName, this);

	users    = new Vector();
	usersMap = Collections.synchronizedMap(new HashMap());

	for (int i = 0; i < usersInf.length; i++) {
	    this.insertUser (usersInf[i]);
	}

        System.out.println (" ok");
    }


    private void insertUser (UserInformation _user) {

	int index = this.getPosition (this.getKey(_user));

	users.add (index, this.getKey(_user));
	usersMap.put (this.getKey(_user), _user);
    }


    private int getPosition (String _key) {

	if (users.size() == 0)
	    return 0;

	int index = 0;

	while ( index < users.size() ) {

	    if ( _key.compareTo ( (String) users.elementAt (index)) < 0)

		return index;

	    index++;

	}

	return index;
    }


    private String getKey (UserInformation _user) {

	return _user.getGroup() + _user.getLogin();

    }

    private int removeUser (UserInformation _user) {

	int i = users.indexOf (this.getKey(_user));

	if (-1 != i) {
	    users.remove (i);
	    usersMap.remove (this.getKey(_user));
	}

	return i;

    }


    public void chatChanged (ChatEvent _e) {

	if (_e.getEventType() == ChatEvent.GETIN) {

	    try {

		groupServer.userGetInARoom (this.loginName, _e.getRoom());

	    } catch (Exception e) {

		System.out.println ("Need to make error log. UsersDataModel.chatChanged():\n" + e.getMessage());
		e.printStackTrace();

	    }

	} else if (_e.getEventType() == ChatEvent.GETOUT) {

	    try {

		groupServer.userGetOutARoom (this.loginName, _e.getRoom());

	    } catch (Exception e) {

		System.out.println ("Need to make error log. UsersDataModel.chatChanged():\n" + e.getMessage());
		e.printStackTrace();

	    }
	}
    }


    public void receiveEvent (GroupEvent _event) throws RemoteException {

	if (iGroupServer.USERIN == _event.getType()) {

	    if (! usersMap.containsKey (this.getKey(_event.getUser())))
		this.insertUser (_event.getUser());

	} else if (iGroupServer.USEROUT == _event.getType()) {

	    int i = this.removeUser (_event.getUser());
	    fireTableRowsDeleted (i-1, i-1);

	} else if (iGroupServer.USERMOVE == _event.getType()) {

	    UserInformation user = (UserInformation) usersMap.get (this.getKey(_event.getUser()));
	    if (null != user)
		user.setChatRoom (_event.getUser().getChatRoom());

	}

	fireTableDataChanged();

    }


    public int getColumnCount() {

	return numColumns;

    }


    public int getRowCount() {

	return users.size();

    }


    public Class getColumnClass (int _col) {

	if (_col >= 0 && _col < numColumns)
	    return String.class;

	return null;

    }


    public Object getValueAt (int _row, int _col) {
	if (_col == 0)
	    return this.getUserbyRow(_row).getName();
	else if (_col == 1)
	    return this.getUserbyRow(_row).getType();
	else if (_col == 2)
	    return this.getUserbyRow(_row).getGroup();
	else if (_col == 3)
	    return this.getUserbyRow(_row).getChatRoom();

	return "";
    }


    private UserInformation getUserbyRow (int _i) {

	return (UserInformation) usersMap.get ( (String) users.elementAt(_i));

    }


    public String getColumnName (int columnIndex) {
	return header[columnIndex];
    }
}
