/**
* Title:        ChatPanel<p>
* Description:  <p>
* @author Paulo Henrique Fisch de Brito
* @version 1.0
*/
package colab.client.gui;

import java.net.*;
import java.io.*;
import java.rmi.*;
import java.rmi.server.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;
import java.util.*;
import colab.util.chat.*;
import colab.client.*;
import colab.client.tool.*;
import colab.util.*;
import colab.util.tool.*;
//***********b Application***************
import colab.util.tool.ploter.*;
import colab.client.tool.ploter.*;
//***********e Application***************

public class ChatPanel extends FocusPanel implements iChatClient, ActionListener{

    private static int MAIN_PANE = 1;
    private static int ROOM_PANE = 2;

    //Data about chat server
    iChatServer server;
    iRoom room;
    String login, serverName, roomName;

    // Listeners
    Vector listeners;

    boolean roomPaneMaked;
    boolean mainPaneMaked;
    int actualPane = MAIN_PANE;

    // Data about chat main window
    String roomListNames[] = null;
    JList roomJList = new JList();

    // Data about room
    JSplitPane splitPane;
    SimpleAttributeSet attrNormal, attrColor;
    DefaultStyledDocument msgDoc;
    JTextPane receivedMsgArea;
    JScrollPane rcvMsgScroll;
    JList userList = new JList();
    JScrollPane userScroll;
    Vector users = new Vector();
    JPanel msgPane;
    JPanel centerPane;
    //JPanel roomNamePane;
    JTextField msgField;
    JCheckBox privateCheck;
    Color       myColor = Color.black;

    //***********b Application***************
    ToolPanel collaborativeToolPanel;
    //***********e Application***************

    public ChatPanel ()  throws IOException {

	this.setFocus (false);
	ColabProperties properties = PropertiesOwner.getProperties();
	this.login  = properties.getLoginName();
	this.serverName = properties.getHost();

	String name = "//" + this.serverName + ":" + RegistryPort.getPort() + "/" + iChatServer.NAME;

	try{
	    //make this object exportable
	    UnicastRemoteObject.exportObject (this);

	    // "Pega" referencia para o servidor
	    this.server = (iChatServer) Naming.lookup(name);

	    // Faz o subscribe no servidor
	    server.subscribe (this.login, this);
	} catch (NotBoundException e) {

	    throw new IOException ("Nao foi possivel achar o servidor do chat.");

	} catch (MalformedURLException e) {

	    throw new IOException ("Problemas com o endereco do chat.\nUrl mal formada: "+name);

	} catch (RemoteException e) {

	    throw new IOException ("Problemas na comunicacao com o servidor de chat: "+ e.getMessage());

	} catch (ServerIsGoingDownException e) {

	    throw new IOException ("O servidor esta saindo do ar. Saia do CoLab, espere uns minutos e entre novamente.");

	}

	this.setLayout (new BorderLayout (4, 4));

        roomJList.setToolTipText("Escolha uma das salas de chat disponveis");
	roomJList.setBorder (BorderFactory.createBevelBorder(BevelBorder.RAISED));

	actualPane = MAIN_PANE;
	roomPaneMaked = false;

	makeMainPane();
    }


    private void switchPane () {

	this.removeAll();
	if (actualPane == MAIN_PANE) {
	    setFocus(true);
	    actualPane = ROOM_PANE;
	    makeRoomPane();
	    fireEvent (new ChatEvent (ChatEvent.GETIN, roomName));
	} else if (actualPane == ROOM_PANE) {
	    setFocus(false);
	    actualPane = MAIN_PANE;
	    makeMainPane();
	    fireEvent (new ChatEvent (ChatEvent.GETOUT, roomName));
	}
	this.validate();
	this.repaint();
    }


    // Pane with the user outside a room
    private void makeMainPane() {

        JLabel roomChoiceLabel = new JLabel ("Selecione uma sala de chat e clique no botao [Entrar]: ");
	roomChoiceLabel.setBorder( BorderFactory.createEmptyBorder (20, 30, 20, 0) );

        // room list:
        try {

            this.roomListNames = this.server.getRooms(this.login);

	} catch (RemoteException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "Problemas de comunicacao com o servidor.");

        } catch (ServerIsGoingDownException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "O servidor esta saindo do ar devido a problemas.\nVoce deve sair do programa, esperar uns minutos e entrar de novo.");

        }

	roomJList.setListData (this.roomListNames);

	JScrollPane roomsScrollPane = new JScrollPane (roomJList);
	roomsScrollPane.setPreferredSize (new Dimension(150, 219));
	roomsScrollPane.setBorder(BorderFactory.createEmptyBorder(0, 40, 60, 0));
	JButton roomEnterButton = new JButton ("Entrar");
	roomEnterButton.setBackground(new Color(190,190,190));
	roomEnterButton.setToolTipText ("Entrar na sala de chat selecionada.");
	roomEnterButton.addActionListener(new ActionListener() {
	    public void actionPerformed (ActionEvent e) {
		if (!roomJList.isSelectionEmpty()) {
		    roomName = roomListNames[roomJList.getSelectedIndex()];
		    enter();
		}
	    }
	});
	// Pane for the button
	JPanel p = new JPanel (new BorderLayout());
	JPanel p2 = new JPanel (new FlowLayout (FlowLayout.CENTER, 8, 200));
	p2.add (roomEnterButton);
	p.add (p2, BorderLayout.WEST);
	//Panel for the RoomList
	JPanel p3 = new JPanel (new BorderLayout(8,8));
	p3.add (roomsScrollPane, BorderLayout.WEST);
	p3.add (p, BorderLayout.CENTER);

	this.add (roomChoiceLabel, BorderLayout.NORTH);
	//add (pList, BorderLayout.WEST);
	this.add (p3, BorderLayout.CENTER);
    }


   //Panel with the user in a room
    private void makeRoomPane() {

	if (!roomPaneMaked) {

	    //CENTER
	    centerPane = new JPanel (new BorderLayout());

	    //Area with received messages
	    attrNormal = new SimpleAttributeSet();
	    attrColor = new SimpleAttributeSet();
	    StyleConstants.setForeground (attrColor, Color.blue);

	    msgDoc = new DefaultStyledDocument();
	    receivedMsgArea = new JTextPane(msgDoc); //JTextArea();
	    receivedMsgArea.setEditable(false);
	    //receivedMsgArea.setLineWrap(true);
	    //receivedMsgArea.setWrapStyleWord(true);
	    rcvMsgScroll = new JScrollPane(receivedMsgArea, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER) {
		public Dimension getPreferredSize () {
		    Dimension d = super.getPreferredSize();
		    //System.out.println ("RcvScroll: "+d.toString());
		    return d;
		}
	    };
	    rcvMsgScroll.setPreferredSize (new Dimension (100, 150));
	    JPanel jp1 = new JPanel (new BorderLayout ());
	    userScroll = new JScrollPane(userList);
	    userScroll.setPreferredSize (new Dimension (90, 150));
	    userList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    JButton outButton = new JButton ("Sair da Sala");
	    outButton.setActionCommand("roomOut");
	    outButton.addActionListener(this);
	    outButton.setToolTipText ("Sair da sala e voltar para a tela com a lista de salas de chat.");
	    jp1.add (outButton, BorderLayout.NORTH);
	    jp1.add (userScroll, BorderLayout.CENTER);

	    centerPane.add (rcvMsgScroll, BorderLayout.CENTER);
	    centerPane.add (jp1, BorderLayout.EAST);

	    //Pane with msg text field, send button and privateCheckBox
	    msgPane = new JPanel();
	    msgPane.setLayout (new BorderLayout());
            JButton sendButton = new JButton("Enviar");
	    sendButton.addActionListener(this);
	    sendButton.setActionCommand("send");
            sendButton.setToolTipText ("Enviar a mensagem para os participantes da sala.");
	    msgField = new JTextField();
	    msgField.addActionListener(this);
	    msgField.setActionCommand("send");
	    privateCheck = new JCheckBox("mensagem particular");

	    msgPane.add (sendButton, BorderLayout.EAST);
	    msgPane.add (msgField, BorderLayout.CENTER);
	    msgPane.add (privateCheck, BorderLayout.SOUTH);

	    JPanel tempPanel = new JPanel(new BorderLayout());
	    tempPanel.add (centerPane, BorderLayout.CENTER);
	    tempPanel.add (msgPane, BorderLayout.SOUTH);

	    //***********b Application***************
	    splitPane = new JSplitPane (JSplitPane.HORIZONTAL_SPLIT, getColToolPanel(), tempPanel);
	    //***********e Application***************

	    roomPaneMaked = true;
	} else {
	    //***********b Application***************
	    splitPane.setLeftComponent(getColToolPanel());
	    //***********e Application***************
	}

	//roomLabel.setText("Sala: " + this.roomName);
	receivedMsgArea.setText("");
	rcvMsgScroll.setBorder (BorderFactory.createTitledBorder(LineBorder.createGrayLineBorder(), "Sala: "+this.roomName,TitledBorder.RIGHT, TitledBorder.TOP));

	//this.add (roomNamePane, BorderLayout.NORTH);
	this.add (splitPane, BorderLayout.CENTER);
	splitPane.setDividerLocation (380);
    }


    public Dimension getPreferredSize() {
	Dimension d = super.getPreferredSize();
	//System.out.println (d+" (preferida)");
	return d;
    }


    public Dimension getMinimumSize() {
	Dimension d = super.getMinimumSize();
	//System.out.println (d+" (minima)");
	return d;
    }


    public void receive (iMessage _msg) throws RemoteException {
	// recebe uma mensagem
	String destiny = _msg.getDestiny();
	boolean isEqual;
	SimpleAttributeSet attr = attrNormal;
	if (isEqual = destiny.equals(this.login)) {
	    attr = this.attrColor;
	}
  	try {
	    String priv = "";
	    if (((ChatMsg)_msg).getPrivate())
		priv = "particular ";
	    msgDoc.insertString (msgDoc.getLength(), "Mensagem " + priv + "de " + _msg.getSender() + " para " +  _msg.getDestiny(), attr);
	    msgDoc.insertString (msgDoc.getLength(), "\n" + _msg.getBody() + "\n\n", attr);
  	} catch (BadLocationException e) {
  	    //BadLocationException: do nothing.
  	}
    }

    public void receiveEvent (int _type, String _event) throws RemoteException {
	//recebe um evento
	//System.out.println ("Evento recebido:");
	if (_type == iChatServer.USER_IN) {
	    // New user enter the room
	    users.add (_event);
	    userList.setListData(users);
	    try {
            msgDoc.insertString(msgDoc.getLength(), _event+" entrou na sala.\n\n", attrNormal);
        } catch (BadLocationException e) {
            //BadLocationException: do nothing.
        }

	} else if (_type == iChatServer.USER_OUT) {
	    // New user enter the room
	    users.remove (_event);
	    userList.setListData(users);
	    try {
	    msgDoc.insertString(msgDoc.getLength(), _event+" saiu da sala.\n\n", attrNormal);
	} catch (BadLocationException e) {
	    //BadLocationException: do nothing.
	}
	}
    }

    public void exit() {

        try {

	    System.out.println ("Vou sair");
            if (actualPane == ROOM_PANE) {
                this.room.out (this.login);
		System.out.println ("\t ok");
	    }


        } catch (RemoteException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "Problemas na comunicacao com o servidor de chat.\n Tente de novo, se o problema persistir saia do programa e entre de novo");

        } catch (ServerIsGoingDownException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "O servidor esta saindo do ar. Saia do programa por uns minutos e tente de novo.");

	}

	if (actualPane == ROOM_PANE) {
	    //***********b Application***************
	    if (null != collaborativeToolPanel)
		collaborativeToolPanel.exit();
	    //***********e Application***************
	}
    }


    private void enter() {
        //System.out.println ("Enter in the room "+roomName+".");

//  	do {
//              myColor = JColorChooser.showDialog(ChatPanel.this, "Escolha a cor das suas funcoes", Color.black);
//  	} while (myColor.getRed() > 200 && myColor.getBlue() < 103 && myColor.getGreen() < 103);

        try {
            this.room = this.server.getRoom (roomName);
            room.in (this.login, this);

            users.removeAllElements(); // melhor colocar quando for sair da sala
            String usersStr[] = room.getList();
            for (int i = 0; i < usersStr.length; i++)
            users.add (usersStr[i]);
            userList.setListData(users);
            userList.setSelectedIndex(0);

            this.switchPane();

        } catch (RemoteException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "Problemas na comunicacao com o servidor de chat.\n Tente de novo, se o problema persistir saia do programa e entre de novo");
	    // Log Error

        } catch (ServerIsGoingDownException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "O servidor esta saindo do ar devido a problemas.\nVoce deve sair do programa, esperar uns minutos e entrar de novo");

	}
    }

    public void actionPerformed(ActionEvent evt) {

        if (evt.getActionCommand().equals("send")) {
            String destiny = (String) userList.getSelectedValue();

            if (destiny != null) {
                String text = msgField.getText();
                ChatMsg msg = new ChatMsg (this.login, destiny, text, privateCheck.getSelectedObjects() != null);
                try {

                    this.room.send (msg);

		} catch (ServerIsGoingDownException e) {

		    StandardDialogs.errorDialog (null, "ERRO", "O servidor esta saindo do ar devido a problemas.\nVoce deve sair do programa, esperar uns minutos e entrar de novo");

                } catch (RemoteException e) {

		    StandardDialogs.errorDialog (null, "ERRO", "Problemas na comunicacao com o servidor de chat.\n Tente de novo, se o problema persistir saia desta sala e entre de novo");

                };

                msgField.setText("");
            }
        } else

            if (evt.getActionCommand().equals("roomOut"))
                try {

		    this.room.out (this.login);

		    //***********b Application***************
		    if (null != collaborativeToolPanel)
			collaborativeToolPanel.exit();
		    //***********e Application***************

		    switchPane();

                } catch (RemoteException e) {

		    StandardDialogs.errorDialog (null, "ERRO", "Problemas na comunicacao com o servidor.\n Tente de novo, se o problema persistir saia desta sala e entre de novo.");

                } catch (ServerIsGoingDownException e) {

		    StandardDialogs.errorDialog (null, "ERRO", "O servidor esta saindo do ar devido a problemas.\nTente novamente, se o problema persistir voce deve sair do programa e entrar de novo");

                }
    }

    //***********b Application***************
    private ToolPanel getColToolPanel() {

	try {

	    iToolModel toolModel = this.room.getRemoteToolModel ();

	    //collaborativeToolPanel = new SharedPloterPanel ((iPloterModel) toolModel);
	    collaborativeToolPanel = ModulesOwner.getToolFactory().createCollaborativeToolPanel(toolModel);

	} catch (RemoteException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "Problemas na comunicacao com o servidor da ferramenta colaborativa.\n Saia desta sala e entre de novo");

	} catch (ServerIsGoingDownException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "O servidor esta saindo do ar devido a problemas.\nTente novamente, se o problema persistir voce deve sair do programa e entrar de novo");

	} catch (Exception e) {

	    e.printStackTrace();
	    //System.out.println (e.toString());
	    StandardDialogs.errorDialog (null, "ERRO", "Nao sei qual e'");
	}


	return collaborativeToolPanel;
    }
    //***********e Application***************

    public void addChatListener (iChatListener _listener) {

	//System.out.println ("Adicionei chat listener");

	if(null == this.listeners)
	    this.listeners = new Vector();

	this.listeners.add (_listener);

    }

    public void removeChatListener (iChatListener _listener) {

	if (null != this.listeners)
	    this.listeners.remove (_listener);

    }

    protected void fireEvent (ChatEvent _e) {

	if (null != this.listeners)

	    for (int i = 0; i < this.listeners.size(); i++)

		( (iChatListener)this.listeners.elementAt(i)).chatChanged (_e);


    }

    public boolean isInARoom() {

	if (actualPane == ROOM_PANE)
	    return true;

	return false;

    };

    public ToolPanel getToolPanel() {

	if (actualPane == ROOM_PANE) {
	    //***********b Application***************
	    return collaborativeToolPanel;
	    //***********e Application***************
	}

	return null;
    }
}
