package colab.client.gui;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.event.MouseInputAdapter;
import java.rmi.*;
import ice.iblite.Browser;
import colab.client.tool.*;
import colab.client.tool.ploter.*;
import colab.client.*;
import colab.util.group.*;
import colab.util.*;

public class Colab implements ActionListener {

    // Constants
    private static final int PLOTER  = 0;

    private String serverName;
    private String loginName;

    private ChatPanel   chatPanel;
    private JFrame      chatFrame;

    private GroupPanel  groupPanel;

    private MailPanel   mailPanel;
    private JFrame      mailFrame;

    private ForumPanel  forumPanel;
    private JFrame      forumFrame;
    private JFrame      notepadFrame;

    //------ b ---------------------
    //private PloterPanel ploterPanel;
    //------ - ---------------------
    private ToolPanel   toolPanel;
    private JFrame      toolFrame;
    //------ - ---------------------

    private Browser     browser;

    private Container   container;
    private JSplitPane  splitPane;
    private JFrame      frame;

    private User user;
    private iGroupServer groupServer;

    public Colab (String _serverName) throws Exception {

	this.serverName = _serverName;

	// Define the standard Look and Feel
	try {
	    UIManager.setLookAndFeel(
		UIManager.getCrossPlatformLookAndFeelClassName());
	} catch (Exception e) { }

	//Create the main container
	this.frame = new JFrame("Colab: ");
	container = frame.getContentPane();
	container.setLayout(new BorderLayout());

	//Finish setting up the frame, and show it.
	frame.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		exit();
	    }
	});

	frame.setSize (600, 580);
	frame.setVisible(true);

	this.makeLogin();

	// Create object (singleton) with global variables
	PropertiesOwner.getProperties (loginName, serverName);

	// MenuBar
	JMenuBar menuBar = new JMenuBar();
	this.createMenuBar(menuBar);
	frame.setJMenuBar(menuBar);

	// ToolBar
	JToolBar toolBar = new JToolBar();
	createToolBar(toolBar);

	System.out.println ("Colab 1");
	splitPane = new JSplitPane (JSplitPane.VERTICAL_SPLIT, this.getBrowser(), ModulesOwner.getGroupPanel());
	System.out.println ("Colab 2");

	JPanel northPanel = new JPanel (new BorderLayout());
	northPanel.add (toolBar, BorderLayout.NORTH);
	toolBar = new JToolBar();
	this.createLinkBar (toolBar);
	northPanel.add (toolBar, BorderLayout.SOUTH);

	container.add (splitPane, BorderLayout.CENTER);
	container.add (northPanel, BorderLayout.NORTH);

	frame.validate();
	splitPane.setDividerLocation (0.7);
	frame.repaint();
    }


    private void makeLogin() {
	//Ask for login and password
	String login = "", password = "";
	boolean ok = false;
	do {

	    //Ask for login and password
	    JPanel col1 = new JPanel(new GridLayout(0,1));
	    col1.add(new JLabel("Login"));
	    col1.add(new JLabel("Senha"));
	    JPanel col2 = new JPanel(new GridLayout(0,1));
	    JTextField txLogin = new JTextField (25);
	    col2.add(txLogin);
	    JPasswordField txPaswd = new JPasswordField (25);
	    col2.add(txPaswd);
	    JPanel jp = new JPanel (new BorderLayout());
	    jp.add (col1, BorderLayout.WEST);
	    jp.add (col2, BorderLayout.EAST);
	    String str =  new String("Digite seu login e sua senha para entrar no Colab:");
	    Object[] lines = {str, jp};
	    int resp = JOptionPane.showConfirmDialog (frame, lines, "Login", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE);
	    if (resp == JOptionPane.OK_OPTION) {
		login = txLogin.getText();
		password = new String(txPaswd.getPassword());
		ok = true;
	    } else {
		ok = false;
	    }

	    if (ok)
		try{
		    groupServer = getGroupServer (serverName);

		    user = groupServer.login (login, password);

		    if (user == null)
			System.out.println ("user == null");

		    loginName = login;
		    frame.setTitle ("Colab: "+loginName);
		} catch (LoginFailException e) {
		    System.out.println ("Fazer log do erro, e falar que a senha e' invalida");
		    ok = false;
		} catch (Exception e) {
		    System.out.println ("Need to make error log. Colab.constructor():\n" + e.getMessage());
		    ok = false;
		}
	    else System.exit(0);
	} while ( !ok );
    }


    private void createMenuBar(JMenuBar _menuBar) {

	_menuBar.setAlignmentX(JComponent.LEFT_ALIGNMENT);
	_menuBar.setAlignmentY(JComponent.TOP_ALIGNMENT);

	//-------------------------------------
	// Arquivo
	JMenu fileMenu = (JMenu) _menuBar.add(new JMenu("Arquivo"));
	fileMenu.setMnemonic('a');

	JMenuItem exitItem = (JMenuItem) fileMenu.add(new JMenuItem("Sair"));
	exitItem.addActionListener (new ActionListener() {
	    public void actionPerformed (ActionEvent e) {
		exit(); //
	    }
	});
	exitItem.setMnemonic('S');

//      JMenu windowMenu = (JMenu) _menuBar.add(new JMenu("Janelas"));
//      windowMenu.setMnemonic('j');

//      JMenuItem browserItem = (JMenuItem) windowMenu.add(new JMenuItem("Browser"));
//      browserItem.setHorizontalTextPosition(JButton.RIGHT);
//      browserItem.setMnemonic('B');

//      JMenuItem ploterItem = (JMenuItem) windowMenu.add(new JMenuItem("Ploter"));
//      ploterItem.setHorizontalTextPosition(JButton.RIGHT);
//      ploterItem.setMnemonic('P');

//      JMenuItem chatItem = (JMenuItem) windowMenu.add(new JMenuItem("Chat"));
//      chatItem.setHorizontalTextPosition(JButton.RIGHT);
//      chatItem.setMnemonic('C');

//      JMenuItem mailItem = (JMenuItem) windowMenu.add(new JMenuItem("Mail"));
//      mailItem.setHorizontalTextPosition(JButton.RIGHT);
//      mailItem.setMnemonic('M');
    }

    private void createToolBar(JToolBar _toolBar) {

	_toolBar.setFloatable(false);

	_toolBar.setBorder(BorderFactory.createEmptyBorder (6, 6, 6, 6));

	JButton button = new JButton("Ploter");
	button.setActionCommand ("ploter");
	button.addActionListener(this);
	button.setToolTipText("Mostra a janela do ploter");
	_toolBar.add(button);

	_toolBar.addSeparator();

	button = new JButton("Chat");
	button.setActionCommand ("chat");
	button.addActionListener(this);
	button.setToolTipText("Mostra a janela de chat");
	_toolBar.add(button);

	_toolBar.addSeparator();

	button = new JButton("Mail");
	button.setActionCommand ("mail");
	button.addActionListener(this);
	button.setToolTipText("Mostra a janela de mail");
	_toolBar.add(button);

	_toolBar.addSeparator();

	button = new JButton("Forum");
	button.setActionCommand ("forum");
	button.addActionListener(this);
	button.setToolTipText("Mostra a janela de forum");
	_toolBar.add(button);

	_toolBar.addSeparator();

	button = new JButton("Editor de Relatorios");
	button.setActionCommand ("report");
	button.addActionListener(this);
	button.setToolTipText("Mostra o editor de relatorios");
	_toolBar.add(button);

    }

    private void createLinkBar(JToolBar _toolBar) {

	_toolBar.setFloatable(false);

	_toolBar.setBorder(BorderFactory.createEmptyBorder (6, 6, 6, 6));
	_toolBar.setBackground (new Color (250, 250, 250));

	JButton button = new JButton("<-");//new ImageIcon ("images/leftarrow3.gif"));
	button.setActionCommand ("back");
	button.addActionListener(this);
	button.setToolTipText("Volta para a pagina anterior na lista de navegacao");
	Color buttonColor = new Color (235, 235, 235);
	button.setBackground (buttonColor);
	_toolBar.add(button);

	_toolBar.addSeparator(new Dimension (4,4));

	button = new JButton("->");//new ImageIcon ("images/rightarrow3.gif"));
	button.setActionCommand ("forward");
	button.addActionListener(this);
	button.setToolTipText("Vai para a pagina seguinte na lista de navegacao");
	button.setBackground (buttonColor);
	_toolBar.add(button);

	_toolBar.addSeparator(new Dimension (12,12));

	button = new JButton("Apresentacao");
	button.setActionCommand ("link_apresentacao");
	button.addActionListener(this);
	button.setToolTipText("Pagina de apresentacao do CoLab");
	button.setBackground (buttonColor);
	_toolBar.add(button);

	Dimension dim = new Dimension (6,6);
	_toolBar.addSeparator(dim);

	button = new JButton("Cronograma de Atividades");
	button.setActionCommand ("link_cronograma");
	button.addActionListener(this);
	button.setToolTipText("Pagina com o cronograma de atividades do curso");
	button.setBackground (buttonColor);
	_toolBar.add(button);

	_toolBar.addSeparator(dim);

	button = new JButton("Tutorial sobre o CoLab");
	button.setActionCommand ("link_tutorial");
	button.addActionListener(this);
	button.setToolTipText("Pagina inicial do tutorial do CoLab");
	button.setBackground (buttonColor);
	_toolBar.add(button);

	_toolBar.addSeparator(dim);

	button = new JButton("Funcoes");
	button.setActionCommand ("link_funcoes");
	button.addActionListener(this);
	button.setToolTipText("Pagina com explicacoes sobre funcoes de 1 variavel");
	button.setBackground (buttonColor);
	_toolBar.add(button);

    }

    public void actionPerformed(ActionEvent evt) {

	try {

	    if (evt.getActionCommand().equals("ploter")) {

		JFrame toolFrame = getToolFrame();
		toolFrame.setVisible(true);

	    } else if (evt.getActionCommand().equals("mail")) {

		getMailFrame().setVisible(true);

	    } else if (evt.getActionCommand().equals("chat")) {

		try {

		    getChatFrame().setVisible(true);

		} catch (IOException e) {

		    StandardDialogs.errorDialog (this.frame, "ERRO", "Problemas para abrir o chat.\n" +e.getMessage());

		}

	    } else if (evt.getActionCommand().equals("forum")) {

		getForumFrame().setVisible(true);

	    } else if (evt.getActionCommand().equals("report")) {

		ModulesOwner.getNotepadFrame().setVisible(true);


	    } else if (evt.getActionCommand().equals("back")) {

		getBrowser().goBack();

	    } else if (evt.getActionCommand().equals("forward")) {

		getBrowser().goForward();

	    } else if (evt.getActionCommand().equals("link_apresentacao")) {

		browser.gotoLocation ("http://www.dca.fee.unicamp.br/~fisch/colab/apresentacao.html");

	    } else if (evt.getActionCommand().equals("link_cronograma")) {

		browser.gotoLocation ("http://www.dca.fee.unicamp.br/~fisch/colab/cronograma.html");

	    } else if (evt.getActionCommand().equals("link_tutorial")) {

		browser.gotoLocation ("http://www.dca.fee.unicamp.br/~fisch/colab/tutorial.html");

	    } else if (evt.getActionCommand().equals("link_funcoes")) {

		browser.gotoLocation ("http://www.dca.fee.unicamp.br/~fisch/colab/funcoes.html");

	    }

	} catch (Exception e) {

	    System.out.println ("colab.client.gui.Colab.actionPerformed: "+e.getMessage() + "\n\n");
	    e.printStackTrace();

	}
    }

    private iGroupServer getGroupServer(String _host) {
	if (groupServer == null) {

	    try{
	      // Reference to GroupServer
		groupServer = (iGroupServer)Naming.lookup("//"+_host+":" + RegistryPort.getPort() +"/GroupServer");
	    } catch (Exception e) {
		System.out.println ("Need to make error log. colab.getGroupServer():\n" + e.getMessage());
		e.printStackTrace();
		return null;
	    }
	}
	return groupServer;
    }

    private void exit() {
	if (ModulesOwner.isThereChatPanel())
	    try {
		ModulesOwner.getChatPanel().exit();
	    } catch (IOException e) {
		//do nothing: exit
	    }

	if (ModulesOwner.isThereMailPanel())
	    ModulesOwner.getMailPanel().exit();

	if (ModulesOwner.isThereForumPanel())
	    ModulesOwner.getForumPanel().exit();

	try {

	    if (groupServer != null)
		groupServer.logout (loginName);

	} catch (Exception e) {

	    System.err.println ("colab.client.gui.Colab.exit(): "+e.getMessage());
	    e.printStackTrace();

	}

	System.exit(0);
    }

    public static void main(String[] args) {
      Colab col;

      try {
	  if (args.length == 0)
	      col = new Colab("");
	  else
	      col = new Colab(args[0]);
      } catch (Exception e) {
	  System.err.println ("colab.client.gui.Colab.main(): "+e.getMessage());
      };
    }

    private Browser getBrowser() {

	if (null == browser) {

	    browser = ModulesOwner.getMainBrowserPanel();//new Browser();
	    browser.gotoLocation ("http://www.dca.fee.unicamp.br/~fisch/colab/apresentacao.html");
	    browser.setPreferredSize (new Dimension(480, 420));

	}

	return browser;
    }


    private JFrame getChatFrame() throws IOException {
	if (null == chatFrame) {

	    chatFrame = new JFrame ("CHAT");
	    chatFrame.setSize(800,570);
	    Container chatContainer = chatFrame.getContentPane();
	    chatContainer.setLayout(new BorderLayout());
	    chatContainer.add (ModulesOwner.getChatPanel(), BorderLayout.CENTER);

	}

	return chatFrame;
    }


    private JFrame getMailFrame() {
	if (null == mailFrame) {

	    mailFrame = new JFrame ("MAIL");
	    mailFrame.setSize(480,520);
	    Container mailContainer = mailFrame.getContentPane();
	    mailContainer.setLayout(new BorderLayout());
	    mailContainer.add (ModulesOwner.getMailPanel(), BorderLayout.CENTER);

	}

	return mailFrame;
    }

    private JFrame getForumFrame() {

	if (null == forumFrame) {
	    forumFrame = new JFrame ("FORUM");
	    forumFrame.setSize(480,480);
	    Container forumContainer = forumFrame.getContentPane();
	    forumContainer.setLayout(new BorderLayout());
	    forumContainer.add (ModulesOwner.getForumPanel(), BorderLayout.CENTER);

	}

	return forumFrame;
    }


//      private JFrame getNotepadFrame() {

//      if (null == notepadFrame) {
//          notepadFrame = new JFrame ("Editor de Relatorios");
//          notepadFrame.setSize(480,480);
//          Container notepadContainer = notepadFrame.getContentPane();
//          notepadContainer.setLayout(new BorderLayout());
//          notepadContainer.add (new Notepad(), BorderLayout.CENTER);

//      }

//      return notepadFrame;
//      }


    private JFrame getToolFrame() {

	if (null == toolFrame) {

	    toolFrame = new JFrame ("FERRAMENTA");
	    toolFrame.setSize (500, 640);
	    Container toolContainer = toolFrame.getContentPane();
	    toolContainer.setLayout(new BorderLayout());
	    toolContainer.add (ModulesOwner.getToolPanel(null), BorderLayout.CENTER);

	}

	return toolFrame;
    }
}
