package colab.client.gui;

import java.rmi.*;
import java.rmi.server.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.tree.*;
import java.util.*;
import colab.util.forum.*;
import colab.client.*;
import colab.util.*;
import colab.client.gui.forum.*;
import java.io.*;

public class ForumPanel extends FocusPanel implements ActionListener, ListSelectionListener, Runnable {

    private final static int READPANE = 1;
    private final static int WRITEPANE = 2;
    private int actualPane = READPANE;

    //Data about forum server
    String loginName, serverName;
    iForumServer forumServer;
    String[] topics;

    // interface
    JToolBar toolBar;

    // msgList pane
    JTreeTable msgTreeTable;
    TreeTableModelAdapter treeTableModelAdapter;
    ForumTreeTableModel forumTreeTableModel;
    JScrollPane msgListScroll;

    //
    // msg pane
    JPanel msgPane;
    ForumMsg forumMsg;
    JScrollPane msgScroll;
    JPanel readPane;
    JPanel writePane;
    JLabel senderLabel, topicLabel, dateLabel, subjectLabel;
    JComboBox topicsComboBox;
    JCheckBox anonymityBox;
    JTextArea msgTextArea;
    JTextField subjectTextField = new JTextField(30);

    //Buttons
    JButton writeButton, replyButton, nextButton;
    JButton relOrderButton, subjOrderButton;
    JButton sendButton, cancelButton;


    public ForumPanel () { //throws Exception {

	    ColabProperties properties = PropertiesOwner.getProperties();
	    this.loginName  = properties.getLoginName();
	    this.serverName = properties.getHost();

	    try{
	        // reference to forum server
	        String name = "//" + this.serverName + ":" + RegistryPort.getPort() + "/" + iForumServer.NAME;
	        forumServer = (iForumServer) Naming.lookup(name);

	        //System.out.println ("1");

	        forumServer.login (loginName);

	        this.setLayout (new BorderLayout (3, 3));

	        //System.out.println ("2");

	        // ToolBar
	        toolBar = new JToolBar();
	        toolBar.setFloatable(false);
	        this.createToolBarButtons();

	        // msgListPane
	        //System.out.println ("2.5");
	        forumTreeTableModel = new ForumTreeTableModel(forumServer, loginName);
	        msgTreeTable = new JTreeTable(forumTreeTableModel);
	        JTree tree = msgTreeTable.getTree();
	        tree.putClientProperty("JTree.lineStyle", "Angled");
	        tree.setRootVisible (false);
	        tree.setShowsRootHandles (true);
	        treeTableModelAdapter = (TreeTableModelAdapter) msgTreeTable.getModel();
	        //System.out.println ("3");
	        msgTreeTable.getSelectionModel().addListSelectionListener(this);
	        msgTreeTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	        msgListScroll = new JScrollPane (msgTreeTable);

	        //System.out.println ("4");
	        // topics
	        topics = forumServer.getTopics();

	        new Thread(this).start();

	        //System.out.println ("5");
	        // msgPane
	        msgTextArea = new JTextArea();
	        msgTextArea.setLineWrap (true);
            msgTextArea.setWrapStyleWord (true);
	        JScrollPane msgTextScroll = new JScrollPane (msgTextArea, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

	        //System.out.println ("6");
	        msgPane = new JPanel(new BorderLayout(5,5));
	        //System.out.println ("7");
	        msgPane.add (getReadPane(), BorderLayout.NORTH);
	        //System.out.println ("8");
	        msgPane.add (msgTextScroll, BorderLayout.CENTER);

	        JSplitPane splitPane = new JSplitPane (JSplitPane.VERTICAL_SPLIT,msgListScroll, msgPane);

	        this.add (toolBar, BorderLayout.NORTH);
	        this.add (splitPane, BorderLayout.CENTER);

		this.validate();
		splitPane.setDividerLocation (100);
		this.repaint();

	    } catch (Exception e) {
	        System.out.println ("Need to make error log. ForumPanel.constructor():\n"+ e.getMessage());
	        e.printStackTrace();
	    }
    }


    private void switchPane() {

	if (actualPane == READPANE) {

	    msgPane.remove (getReadPane());
	    msgPane.add (getWritePane(), BorderLayout.NORTH);
	    actualPane = WRITEPANE;

	} else { // actualPane == WRITEPANE

	    msgPane.remove (getWritePane());
	    msgPane.add (getReadPane(), BorderLayout.NORTH);
	    actualPane = READPANE;
	    this.setReadable();

	}

	this.revalidate();
	this.repaint();

    }


    private JPanel getReadPane() {
        //System.out.println ("a");
	    if (readPane == null) {
	        //System.out.println ("b");
            readPane = new JPanel(new GridLayout(1,0));
            JPanel panel = new JPanel (new BorderLayout());
	        JPanel colPanel = new JPanel(new GridLayout(0,1));
  		    JLabel label = new JLabel ("De: ");
		    colPanel.add (label);
            label = new JLabel ("Topico: ");
		    colPanel.add (label);

	        //System.out.println ("c");
	        panel.add (colPanel, BorderLayout.WEST);
	        colPanel = new JPanel (new GridLayout(0,1));
		    senderLabel = new JLabel("");
		    colPanel.add (senderLabel);
		    topicLabel = new JLabel ("");
		    colPanel.add (topicLabel);

	        panel.add (colPanel, BorderLayout.CENTER);

	        //System.out.println ("d");
	        readPane.add (panel);
	        panel =  new JPanel (new BorderLayout());
		    colPanel = new JPanel (new GridLayout(0,1));
  		    label = new JLabel ("Data: ");
		    colPanel.add (label);
		    label = new JLabel ("Assunto: ");
		    colPanel.add (label);

	        //System.out.println ("e");
	        panel.add (colPanel, BorderLayout.WEST);
		    colPanel = new JPanel (new GridLayout(0,1));
  		    dateLabel = new JLabel ("");
		    colPanel.add (dateLabel);
		    subjectLabel = new JLabel ("");
		    colPanel.add (subjectLabel);
	        panel.add (colPanel, BorderLayout.CENTER);
	        readPane.add (panel);
	    }

        //System.out.println ("f");
	    toolBar.removeAll();
	    toolBar.add (writeButton);
	    toolBar.add (replyButton);
	    toolBar.add (nextButton);
	    //toolBar.add (relOrderButton);
	    //toolBar.add (subjOrderButton);

        //System.out.println ("g");
	    return readPane;
    }

    private JPanel getWritePane() {
	if (writePane == null) {
            writePane = new JPanel(new BorderLayout());

	    JPanel colPanel = new JPanel(new GridLayout(0,1));
              JLabel label = new JLabel ("Topico: ");
	      colPanel.add (label);
	      label = new JLabel ("Assunto: ");
	      colPanel.add (label);

	    writePane.add (colPanel, BorderLayout.WEST);

	    colPanel = new JPanel (new GridLayout(0,1));
	      JPanel line = new JPanel (new BorderLayout());
	      topicsComboBox = new JComboBox(topics);
	      line.add (topicsComboBox, BorderLayout.CENTER);
              anonymityBox = new JCheckBox ("Enviar mensagem anonima");
	      line.add (anonymityBox, BorderLayout.EAST);
	      colPanel.add (line);
	      colPanel.add (subjectTextField);

	    writePane.add (colPanel, BorderLayout.CENTER);
	}

	toolBar.removeAll();
	toolBar.add (sendButton);
	toolBar.add (cancelButton);

	return writePane;
    }

    public void run () {
        while (true) {

            if (forumTreeTableModel.verifyMsgs()) {

            }

            try {
                Thread.sleep(15000);
            } catch (InterruptedException e){
            }
        }
    }

    public void exit() {

    }

    private void createToolBarButtons() {

        writeButton = new JButton("Escrever");
        writeButton.setActionCommand ("write");
        writeButton.addActionListener(this);
        writeButton.setToolTipText("Escrever uma nova mensagem.");

        replyButton = new JButton("Responder");
        replyButton.setActionCommand ("reply");
        replyButton.addActionListener(this);
        replyButton.setToolTipText("Responder a mensagem selecionada.");

        nextButton = new JButton("Proxima");
        nextButton.setActionCommand ("next");
        nextButton.addActionListener(this);
        nextButton.setToolTipText("Ir para a proxima mensagem");

        sendButton = new JButton("Enviar");
        sendButton.setActionCommand ("send");
        sendButton.addActionListener(this);
        sendButton.setToolTipText("Enviar a mensagem.");

        cancelButton = new JButton("Cancelar");
        cancelButton.setActionCommand ("cancel");
        cancelButton.addActionListener(this);
        cancelButton.setToolTipText("Cancelar esta mensagem");
    };

    private iForumMsgNode getSelected() {
	int selection = msgTreeTable.getSelectedRow();
  	if (selection != -1)
  	    return (iForumMsgNode) ((DefaultMutableTreeNode)treeTableModelAdapter.nodeForRow(selection)).getUserObject();
	return null;
    }

    public void actionPerformed(ActionEvent evt) {

        try {
            if (evt.getActionCommand().equals("write")) {

		forumMsg = new ForumMsg (loginName, "", "", "");

		subjectTextField.setText("");
		msgTextArea.setText("");

		this.switchPane();

	    } else if (evt.getActionCommand().equals("reply")) {

		iForumMsgNode msg = getSelected();
		if (msg != null) {
		    forumMsg = new ForumMsg (loginName, "", "", "", msg.getId());

		    this.switchPane();
		}
            } else if (evt.getActionCommand().equals("next")) {
		//Verify selection

	    } else if (evt.getActionCommand().equals("relOrder")) {

	    } else if (evt.getActionCommand().equals("subjOrder")) {
		//

            } else if (evt.getActionCommand().equals("send")) {
                //send forum
		forumMsg.setTopic ((String)topicsComboBox.getSelectedItem());
		forumMsg.setSubject (subjectTextField.getText());
		forumMsg.setBody (msgTextArea.getText());
                //System.out.println ("Selecao: " + anonymityBox.getModel().isSelected());
		forumMsg.setAnonymity (anonymityBox.getModel().isSelected());
		msgTextArea.setText("");

		forumServer.send (forumMsg);
                //System.out.println ("verificar erro e avisar ao usuario");

		this.switchPane();

            } else if (evt.getActionCommand().equals("cancel")) {
                //cancel message
		this.switchPane();

            }

        } catch (Exception e) {
            System.out.println ("colab.client.gui.Colab.actionPerformed: "+e.getMessage() + "\n\n");
            e.printStackTrace();
        }
    }

    public void valueChanged (ListSelectionEvent e) {
	if (actualPane == READPANE) {
	    this.setReadable();
	}
    }

    private void setReadable () {
	int selection = msgTreeTable.getSelectedRow();
	if (selection != -1) {
	    Object obj = treeTableModelAdapter.nodeForRow (selection);
	    iForumMsgNode msg = (iForumMsgNode) ((DefaultMutableTreeNode)treeTableModelAdapter.nodeForRow(selection)).getUserObject();

	    if (msg != null) {
		msgTextArea.setText  (msg.getBody());
		senderLabel.setText  (msg.getSender());
		topicLabel.setText   (msg.getTopic());
		dateLabel.setText    (msg.getDate().toString());
		subjectLabel.setText (msg.getSubject());
	    }
	}
    }
}
