/**
* Title:        MailPanel<p>
* Description:  <p>
* @author Paulo Henrique Fisch de Brito
* @version 1.0
*/
package colab.client.gui;

import java.rmi.*;
import java.rmi.server.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;
import java.util.*;
import colab.util.mail.*;
import colab.client.*;
import colab.util.*;
import javax.swing.table.AbstractTableModel;
import java.io.*;

public class MailPanel extends FocusPanel implements ActionListener, Runnable {

    private static int MAIN_PANE = 1;
    private static int MSG_PANE = 2;

    //Data about mail server
    String loginName, serverName;

    // interface
    boolean msgPaneMaked;
    boolean mainPaneMaked;
    int actualPane = MAIN_PANE;
    JPanel utilPane;

    // main pane
    JTable msgTable;
    MailDataModel mailModel;
    JScrollPane msgScroll;

    // msg pane
    boolean write = true;
    MailMsg mailMsg;
    JScrollPane msgTextScroll;
    JTextArea msgTextArea = new JTextArea();
    JPanel northPane;
    JTextField toTextField = new JTextField(30);
    JTextField subjectTextField = new JTextField(30);

    //Buttons
    JButton readButton, writeButton, replyButton, delButton;
    JButton forwardButton, msgButton, sendButton;

    //Remote objects
    iMailBox mailBox;
    iMailServer mailServer;

    public MailPanel () { //throws Exception {

	//this.setFocus(true);
	ColabProperties properties = PropertiesOwner.getProperties();
	this.loginName  = properties.getLoginName();
	this.serverName = properties.getHost();

	try{
	    // reference to mail server
	    String name = "//" + this.serverName + ":" + RegistryPort.getPort() + "/" + iMailServer.NAME;
	    mailServer = (iMailServer) Naming.lookup(name);

	    // reference to MailBox
	    mailServer.login (this.loginName);
	    mailBox = mailServer.getMailBox(this.loginName);
	    System.out.println ("MailBox: " + mailBox.getUser().getName());

	    // msgTable
	    mailModel = new MailDataModel(mailBox);
	    msgTable = new JTable(mailModel);
	    msgTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    msgScroll = new JScrollPane (msgTable);
	    //msgTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

	    new Thread(this).start();
	} catch (Exception e) {
	    System.out.println ("Need to make error log. MailPanel.constructor():\n"
			+ e.getMessage());
	    e.printStackTrace();
	}

	this.setLayout (new BorderLayout (7, 7));
	//this.setBorder (BorderFactory.createEtchedBorder());

	JToolBar toolBar = new JToolBar();
	toolBar.setFloatable(false);
	this.createToolBar(toolBar);
	this.add (toolBar, BorderLayout.NORTH);

	utilPane = new JPanel (new BorderLayout());
	utilPane.setBorder (BorderFactory.createLineBorder(Color.black, 3));
	this.add (utilPane, BorderLayout.CENTER);

	actualPane = MAIN_PANE;
	msgPaneMaked = false;

	makeMainPane();
    }

    public void run () {
        while (true) {

            if (mailModel.verifyMsgs()) {
                System.out.println (" Mail novo ");
		msgScroll.getViewport().setView (msgTable);
                //msgScroll.getViewport().validate();
                //msgScroll.getViewport().repaint();
            }

            try {
                Thread.sleep(10000);
            } catch (InterruptedException e){
            }

        }
    }

    private void switchPane () {

	this.utilPane.removeAll();
	if (actualPane == MAIN_PANE) {
	    actualPane = MSG_PANE;
	    makeMsgPane();
	} else if (actualPane == MSG_PANE) {
	    actualPane = MAIN_PANE;
	    makeMainPane();
	}
	this.revalidate();
	this.repaint();
    }


    // Pane with the user outside a room
    private void makeMainPane() {

        readButton.setEnabled (true);
        writeButton.setEnabled (true);
        replyButton.setEnabled (true);
        delButton.setEnabled (true);
        forwardButton.setEnabled (true);
        msgButton.setEnabled (false);
        sendButton.setEnabled (false);

        this.utilPane.add (msgScroll, BorderLayout.CENTER);

    }


   //Panel with the user in a room
    private void makeMsgPane() {

        if (!msgPaneMaked) {
            northPane = new JPanel(new BorderLayout(5,5));
            JPanel panel = new JPanel (new GridLayout (0,1));
            JLabel label = new JLabel ("Para:");
            panel.add(label);
            label = new JLabel ("Assunto:");
            panel.add(label);
            northPane.add (panel, BorderLayout.WEST);
            panel = new JPanel (new GridLayout (0,1));
            panel.add (toTextField);
            panel.add (subjectTextField);
            northPane.add (panel, BorderLayout.CENTER);

            msgTextArea.setRows (6);
            msgTextArea.setLineWrap (true);
            msgTextArea.setWrapStyleWord (true);
            msgTextScroll = new JScrollPane (msgTextArea, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

            msgPaneMaked = true;
        }

	toTextField.setText (mailMsg.getDestiny());
	subjectTextField.setText (mailMsg.getSubject());
	msgTextArea.setText (mailMsg.getBody());

        readButton.setEnabled (false);
        writeButton.setEnabled (false);
        replyButton.setEnabled (false);
        delButton.setEnabled (false);
        forwardButton.setEnabled (false);
        msgButton.setEnabled (true);

	if (write) {
	    sendButton.setEnabled (true);
	    toTextField.setEditable (true);
	    subjectTextField.setEditable (true);
	    msgTextArea.setEditable (true);
	} else {
	    sendButton.setEnabled (false);
	    toTextField.setEditable (false);
	    subjectTextField.setEditable (false);
	    msgTextArea.setEditable (false);
	}

        this.utilPane.add (northPane, BorderLayout.NORTH);
        this.utilPane.add (msgTextScroll, BorderLayout.CENTER);
    }

    public void exit() {

    }

    private void createToolBar(JToolBar _toolBar) {

        _toolBar.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));
        readButton = new JButton("Ler");
        readButton.setActionCommand ("read");
        readButton.addActionListener(this);
        readButton.setToolTipText("Ler a mensagem selecionada.");
        _toolBar.add(readButton);

        writeButton = new JButton("Escrever");
        writeButton.setActionCommand ("write");
        writeButton.addActionListener(this);
        writeButton.setToolTipText("Escrever uma nova mensagem de email");
        _toolBar.add(writeButton);

        replyButton = new JButton("Responder");
        replyButton.setActionCommand ("reply");
        replyButton.addActionListener(this);
        replyButton.setToolTipText("Responder a mensagem selecionada.");
        _toolBar.add(replyButton);

        delButton = new JButton("Apagar");
        delButton.setActionCommand ("del");
        delButton.addActionListener(this);
        delButton.setToolTipText("Apagar a mensagem selecionada.");
        _toolBar.add(delButton);

        forwardButton = new JButton("Encaminhar");
        forwardButton.setActionCommand ("forward");
        forwardButton.addActionListener(this);
        forwardButton.setToolTipText("Encaminhar a mensagem `a outra pessoa");
        _toolBar.add(forwardButton);

        msgButton = new JButton("Mensagens");
        msgButton.setActionCommand ("msg");
        msgButton.addActionListener(this);
        msgButton.setToolTipText("Mostrar a lista de mensagens recebidas");
        _toolBar.add(msgButton);

        sendButton = new JButton("Enviar");
        sendButton.setActionCommand ("send");
        sendButton.addActionListener(this);
        sendButton.setToolTipText("Enviar a mensagem.");
        _toolBar.add(sendButton);
    };

    private iMailMsg getSelected() {
	int selection = msgTable.getSelectedRow();
	if (selection != -1)
	    return mailModel.getMsg(selection);
	else return null;
    }

    public void actionPerformed(ActionEvent evt) {

        try {
            if (evt.getActionCommand().equals("read")) {
                if (actualPane == MAIN_PANE) {
		    write = false;
		    iMailMsg msg = getSelected();
		    //System.out.println ("Classe da mensagem: " + msg.getClass().getName());
		    if (msg != null) {
			System.out.println ("Setar como lida");
			msg.setReaded(true);
			System.out.println ("Mensagem lida = " + msg.getReaded());
			mailMsg = new MailMsg(msg.getSender(), msg.getDestiny(), msg.getSubject(), msg.getBody());
			//toTextField.setText (msg.getSender());
			//subjectTextField.setText (msg.getSubject());
			//msgTextArea.setText (msg.getBody());
			this.switchPane();
		    }
                }
            } else if (evt.getActionCommand().equals("write")) {
                if (actualPane == MAIN_PANE) {
		    write = true;
		    mailMsg = new MailMsg (loginName, "", "", "");
                    this.switchPane();
                }
            } else if (evt.getActionCommand().equals("reply")) {
                if (actualPane == MAIN_PANE) {
		    write = true;
		    iMailMsg msg = getSelected();
		    if (msg != null) {
			StringReader oldBody = new StringReader (msg.getBody());
			StringWriter newBody = new StringWriter();

			newBody.write ("\n----- Mensagem enviada por " + msg.getSender() + " -----\n>\n>");
			int c = oldBody.read();
			while (c != -1) {
			    newBody.write(c);
			    if (c == '\n')
				newBody.write ("> ");
			    c = oldBody.read();
			}
			mailMsg = new MailMsg (loginName, msg.getSender(),"Resposta: "+msg.getSubject(), newBody.toString(), msg.getId());
			this.switchPane();
		    }
		}
            } else if (evt.getActionCommand().equals("del")) {
                if (actualPane == MAIN_PANE) {
                    //Verify selection
                }
            } else if (evt.getActionCommand().equals("forward")) {
                if (actualPane == MAIN_PANE) {
                    //this.toTextField = table
		    write = true;
		    iMailMsg msg = getSelected();
		    if (msg != null) {
			mailMsg = new MailMsg (loginName, "", "Encaminhada: "+msg.getSubject(), "\n----- Mensagem enviada por " + msg.getSender() + " -----\n\n"+msg.getBody());
			this.switchPane();
		    }
                }
            } else if (evt.getActionCommand().equals("msg")) {
                if (actualPane == MSG_PANE) {
                    this.switchPane();
                }
            } else if (evt.getActionCommand().equals("send")) {
                //send mail
                if (actualPane == MSG_PANE) {
                    mailMsg.setDestiny(toTextField.getText());
		    mailMsg.setSubject(subjectTextField.getText());
		    mailMsg.setBody(msgTextArea.getText());
                    mailServer.send (mailMsg);
                    System.out.println ("verificar erro e avisar ao usuario");
                    //this.toTextField = table
                    this.switchPane();
                }
            }

        } catch (Exception e) {
            System.out.println ("colab.client.gui.MailPanel.actionPerformed: "+e.getMessage() + "\n\n");
            e.printStackTrace();
        }
        }

}


class MailDataModel extends AbstractTableModel {

    // By extending AbstractTableModel, instead of
    // implementing TableModel yourself,
    // AbstractTableModel takes care of
    // TableModelListener list management

    iMailMsg[] mailMsgs;
    iMailBox  mailBox;


    String columns[] = {"Nova", "Assunto", "De", "Data", "Anexo"};
    private int numColumns = columns.length;

    private int numRows = 0;

    public MailDataModel (iMailBox _mailBox) {
	mailBox = _mailBox;

	this.verifyMsgs();
    }

    public boolean verifyMsgs () {
        boolean newMsgs = false;

	//System.out.println ("Verificando se tem mensagens novas");

        try{
            int size = mailBox.getSize();
            if (size != numRows) {
                mailMsgs = mailBox.getList();

		fireTableDataChanged();

                newMsgs = true;
                numRows = size;
            } // if (size != numRows)
        } catch (ServerIsGoingDownException e) {
            System.out.println ("O servidor esta saindo do ar. Make Error log, MailPanel.MailDataModel.verifyMsgs(): "+e.getMessage());
            e.printStackTrace();
        } catch (RemoteException e) {
            System.out.println ("Make Error log, MailPanel.MailDataModel.verifyMsgs(): "+e.getMessage());
            e.printStackTrace();
        }

	return newMsgs;
    }

    public iMailMsg getMsg(int _i){
	if (_i >= 0 && _i < mailMsgs.length)
	    return mailMsgs[_i];
	else return null;
    }

    public int getColumnCount() {
	return numColumns;
    }

    public int getRowCount() {
	return numRows;
    }

    public Class getColumnClass (int _col) {
	if (_col == 0)
	    return Boolean.class;
	else if (_col == 3)
	    return Date.class;
	else
	    return String.class;
    }

    public Object getValueAt (int _row, int _col) {
	if (_col == 0)
	    return new Boolean (!mailMsgs[_row].getReaded());
	else if (_col == 1)
	    return mailMsgs[_row].getSubject();
	else if (_col == 2)
	    return mailMsgs[_row].getSender();
	else if (_col == 3)
	    return mailMsgs[_row].getDate();
	else if (_col == 4)
	    return "-";

	return "";
    }

    public String getColumnName (int columnIndex) {
	return columns[columnIndex];
    }

}
