/*
 * @(#)Notepad.java	1.15 99/04/23
 *
 * Copyright (c) 1997-1999 by Sun Microsystems, Inc. All Rights Reserved.
 * 
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 * 
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 * 
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package colab.client.gui;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.print.*;
import java.beans.*;
import java.io.*;
import java.net.URL;
import java.util.*;

import javax.swing.text.*;
import javax.swing.undo.*;
import javax.swing.event.*;
import javax.swing.*;

import com.sun.image.codec.jpeg.*;
/**
 * Sample application using the simple text editor component that
 * supports only one font.
 *
 * @author  Timothy Prinzing
 * @version 1.15 04/23/99
 */
public class Notepad extends JPanel implements DocumentListener, Printable {

    private boolean isSaved = true;   

    private File    htmlFileDirectory;

    private static ResourceBundle resources;

    private int     imageNumber = 0; 
    private String  imageDirectoryName = "images";
    private File    imageDirectory;

    static {
        try {
            resources = ResourceBundle.getBundle("resources.Notepad", 
                                                 Locale.getDefault());
        } catch (MissingResourceException mre) {
            System.err.println("resources/Notepad.properties not found");
            //System.exit(1);
        }
    }

    public Notepad() {
	super(true);

	// Force SwingSet to come up in the Cross Platform L&F
	try {
	    UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	    // If you want the System L&F instead, comment out the above line and
	    // uncomment the following:
	    // UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	} catch (Exception exc) {
	    System.err.println("Error loading L&F: " + exc);
	}

        String vers = System.getProperty("java.version");

	setBorder(BorderFactory.createEtchedBorder());
	setLayout(new BorderLayout());

	// create the embedded JTextComponent
	editor = createEditor();
	editor.setFont(new Font("monospaced", Font.PLAIN, 12));
	// Add this as a listener for undoable edits.
	editor.getDocument().addUndoableEditListener(undoHandler);

	// install the command table
	commands = new Hashtable();
	Action[] actions = getActions();
	for (int i = 0; i < actions.length; i++) {
	    Action a = actions[i];
	    //commands.put(a.getText(Action.NAME), a);
	    commands.put(a.getValue(Action.NAME), a);
	}
	
	JScrollPane scroller = new JScrollPane();
	JViewport port = scroller.getViewport();
	port.add(editor);
	try {
	    String vpFlag = resources.getString("ViewportBackingStore");
	    Boolean bs = new Boolean(vpFlag);
	    port.setBackingStoreEnabled(bs.booleanValue());
	} catch (MissingResourceException mre) {
	    // just use the viewport default
	}

	menuItems = new Hashtable();
	menubar = createMenubar();
	add("North", menubar);
	JPanel panel = new JPanel();
	panel.setLayout(new BorderLayout());	
	//panel.add("North",createToolbar());
	panel.add("Center", scroller);
	add("Center", panel);
	add("South", createStatusbar());
    }

    public static void main(String[] args) {
        try {
        String vers = System.getProperty("java.version");
        if (vers.compareTo("1.1.2") < 0) {
            System.out.println("!!!WARNING: Swing must be run with a " +
                               "1.1.2 or higher version VM!!!");
        }
        JFrame frame = new JFrame();
        frame.setTitle(resources.getString("Title"));
	frame.setBackground(Color.lightGray);
	frame.getContentPane().setLayout(new BorderLayout());
	frame.getContentPane().add("Center", new Notepad());
	frame.addWindowListener(new AppCloser());
	frame.pack();
	frame.setSize(500, 600);
        frame.show();
        } catch (Throwable t) {
            System.out.println("uncaught exception: " + t);
            t.printStackTrace();
        }
    }

    // Implements the print method
    public int print (Graphics graphics, PageFormat pageFormat, int pageIndex) {

	if (pageIndex > 0)
	    return Printable.NO_SUCH_PAGE;

	Graphics2D g2d = (Graphics2D) graphics;
	g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
	// Turn off double buffering
	JComponent comp = getEditor();
	RepaintManager currentManager = RepaintManager.currentManager(comp);
	currentManager.setDoubleBufferingEnabled(false);
	//printing
	comp.paint(g2d);
	// Turn double buffering back on
	currentManager.setDoubleBufferingEnabled(true);
	return(PAGE_EXISTS);
    }
    
    /**
     * Fetch the list of actions supported by this
     * editor.  It is implemented to return the list
     * of actions supported by the embedded JTextComponent
     * augmented with the actions defined locally.
     */
    public Action[] getActions() {
	return TextAction.augmentList(editor.getActions(), defaultActions);
    }

    /**
     * Create an editor to represent the given document.  
     */
    protected JTextComponent createEditor() {
	return new JTextPane();
    }

    /** 
     * Fetch the editor contained in this panel
     */
    protected JTextComponent getEditor() {
	return editor;
    }

    public void insertComponents (JComponent[] _components) {

	for (int i = 0; i < _components.length; i++) {
	    _components[i].setMaximumSize (new Dimension(300, 300));
	    ((JTextPane)this.getEditor()).insertComponent (_components[i]);
	}

    }

    public void insertComponents (Object[] _components, int w, int h) {

	JTextPane pane = (JTextPane)this.getEditor();

	for (int i = 0; i < _components.length; i++) {

	    try {

		if (Icon.class.isInstance(_components[i])) {

		    //((ImageIcon)_components[i]).setMaximumSize (new Dimension(w, h));
		    pane.insertIcon ((Icon)_components[i]);

		} else {
		
		    StyledDocument sDoc= pane.getStyledDocument();
		    sDoc.insertString (sDoc.getLength(), _components[i].toString(), null);
		}

	    } catch (Exception e) {

		System.out.println ("Erro em Notepad.insertComponents(): "+ e.getMessage());
		e.printStackTrace();

	    }
	}
    }

    //Methods of interface DocumentListener
    public void changedUpdate (DocumentEvent e) {
	this.isSaved = false;
    }

    public void insertUpdate (DocumentEvent e) {
	this.isSaved = false;
    }

    public void removeUpdate (DocumentEvent e) {
	this.isSaved = false;
    }
    
    /**
     * To shutdown when run as an application.  This is a
     * fairly lame implementation.   A more self-respecting
     * implementation would at least check to see if a save
     * was needed.
     */
    protected static final class AppCloser extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
	    //System.exit(0);
	}
    }

    /**
     * Find the hosting frame, for the file-chooser dialog.
     */
    protected JFrame getFrame() {
	for (Container p = getParent(); p != null; p = p.getParent()) {
	    if (p instanceof JFrame) {
		return (JFrame) p;
	    }
	}
	return null;
    }

    /**
     * This is the hook through which all menu items are
     * created.  It registers the result with the menuitem
     * hashtable so that it can be fetched with getMenuItem().
     * @see #getMenuItem
     */
    protected JMenuItem createMenuItem(String cmd) {
	JMenuItem mi = new JMenuItem(getResourceString(cmd + labelSuffix));
        URL url = getResource(cmd + imageSuffix);
	if (url != null) {
	    mi.setHorizontalTextPosition(JButton.RIGHT);
	    mi.setIcon(new ImageIcon(url));
	}
	String astr = getResourceString(cmd + actionSuffix);
	if (astr == null) {
	    astr = cmd;
	}
	mi.setActionCommand(astr);
	Action a = getAction(astr);
	if (a != null) {
	    mi.addActionListener(a);
	    a.addPropertyChangeListener(createActionChangeListener(mi));
	    mi.setEnabled(a.isEnabled());
	} else {
	    mi.setEnabled(false);
	}
	menuItems.put(cmd, mi);
	return mi;
    }

    /**
     * Fetch the menu item that was created for the given
     * command.
     * @param cmd  Name of the action.
     * @returns item created for the given command or null
     *  if one wasn't created.
     */
    protected JMenuItem getMenuItem(String cmd) {
	return (JMenuItem) menuItems.get(cmd);
    }

    protected Action getAction(String cmd) {
	return (Action) commands.get(cmd);
    }

    protected String getResourceString(String nm) {
	String str;
	try {
	    str = resources.getString(nm);
	} catch (MissingResourceException mre) {
	    str = null;
	}
	return str;
    }

    protected URL getResource(String key) {
	String name = getResourceString(key);
	if (name != null) {
	    URL url = this.getClass().getResource(name);
	    return url;
	}
	return null;
    }

    protected Container getToolbar() {
	return toolbar;
    }

    protected JMenuBar getMenubar() {
	return menubar;
    }

    /**
     * Create a status bar
     */
    protected Component createStatusbar() {
	// need to do something reasonable here
	status = new StatusBar();
	return status;
    }

    /**
     * Create the toolbar.  By default this reads the 
     * resource file for the definition of the toolbar.
     */
    private Component createToolbar() {
	toolbar = new JToolBar();
	String[] toolKeys = tokenize(getResourceString("toolbar"));
	for (int i = 0; i < toolKeys.length; i++) {
	    if (toolKeys[i].equals("-")) {
		toolbar.add(Box.createHorizontalStrut(5));
	    } else {
		toolbar.add(createTool(toolKeys[i]));
	    }
	}
	toolbar.add(Box.createHorizontalGlue());
	return toolbar;
    }

    /**
     * Hook through which every toolbar item is created.
     */
    protected Component createTool(String key) {
	return createToolbarButton(key);
    }

    /**
     * Create a button to go inside of the toolbar.  By default this
     * will load an image resource.  The image filename is relative to
     * the classpath (including the '.' directory if its a part of the
     * classpath), and may either be in a JAR file or a separate file.
     * 
     * @param key The key in the resource file to serve as the basis
     *  of lookups.
     */
    protected JButton createToolbarButton(String key) {
	URL url = getResource(key + imageSuffix);
        JButton b = new JButton(new ImageIcon(url)) {
            public float getAlignmentY() { return 0.5f; }
	};
        b.setRequestFocusEnabled(false);
        b.setMargin(new Insets(1,1,1,1));

	String astr = getResourceString(key + actionSuffix);
	if (astr == null) {
	    astr = key;
	}
	Action a = getAction(astr);
	if (a != null) {
	    b.setActionCommand(astr);
	    b.addActionListener(a);
	} else {
	    b.setEnabled(false);
	}

	String tip = getResourceString(key + tipSuffix);
	if (tip != null) {
	    b.setToolTipText(tip);
	}
 
        return b;
    }

    /**
     * Take the given string and chop it up into a series
     * of strings on whitespace boundries.  This is useful
     * for trying to get an array of strings out of the
     * resource file.
     */
    protected String[] tokenize(String input) {
	Vector v = new Vector();
	StringTokenizer t = new StringTokenizer(input);
	String cmd[];

	while (t.hasMoreTokens())
	    v.addElement(t.nextToken());
	cmd = new String[v.size()];
	for (int i = 0; i < cmd.length; i++)
	    cmd[i] = (String) v.elementAt(i);

	return cmd;
    }

    /**
     * Create the menubar for the app.  By default this pulls the
     * definition of the menu from the associated resource file. 
     */
    protected JMenuBar createMenubar() {
	JMenuItem mi;
	JMenuBar mb = new JMenuBar();

	String[] menuKeys = tokenize(getResourceString("menubar"));
	for (int i = 0; i < menuKeys.length; i++) {
	    JMenu m = createMenu(menuKeys[i]);
	    if (m != null) {
		mb.add(m);
	    }
	}
	return mb;
    }
    

    /**
     * Create a menu for the app.  By default this pulls the
     * definition of the menu from the associated resource file.
     */
    protected JMenu createMenu(String key) {
	String[] itemKeys = tokenize(getResourceString(key));
	JMenu menu = new JMenu(getResourceString(key + "Label"));
	for (int i = 0; i < itemKeys.length; i++) {
	    if (itemKeys[i].equals("-")) {
		menu.addSeparator();
	    } else {
		JMenuItem mi = createMenuItem(itemKeys[i]);
		menu.add(mi);
	    }
	}
	return menu;
    }

    // Yarked from JMenu, ideally this would be public.
    protected PropertyChangeListener createActionChangeListener(JMenuItem b) {
	return new ActionChangedListener(b);
    }

    // Yarked from JMenu, ideally this would be public.
    private class ActionChangedListener implements PropertyChangeListener {
        JMenuItem menuItem;
        
        ActionChangedListener(JMenuItem mi) {
            super();
            this.menuItem = mi;
        }
        public void propertyChange(PropertyChangeEvent e) {
            String propertyName = e.getPropertyName();
            if (e.getPropertyName().equals(Action.NAME)) {
                String text = (String) e.getNewValue();
                menuItem.setText(text);
            } else if (propertyName.equals("enabled")) {
                Boolean enabledState = (Boolean) e.getNewValue();
                menuItem.setEnabled(enabledState.booleanValue());
            }
        }
    }

    private JTextComponent editor;
    private Hashtable commands;
    private Hashtable menuItems;
    private JMenuBar menubar;
    private JToolBar toolbar;
    private JComponent status;
    private JFrame elementTreeFrame;
    protected ElementTreePanel elementTreePanel;

    protected FileDialog fileDialog;

    /**
     * Listener for the edits on the current document.
     */
    protected UndoableEditListener undoHandler = new UndoHandler();

    /** UndoManager that we add edits to. */
    protected UndoManager undo = new UndoManager();

    /**
     * Suffix applied to the key used in resource file
     * lookups for an image.
     */
    public static final String imageSuffix = "Image";

    /**
     * Suffix applied to the key used in resource file
     * lookups for a label.
     */
    public static final String labelSuffix = "Label";

    /**
     * Suffix applied to the key used in resource file
     * lookups for an action.
     */
    public static final String actionSuffix = "Action";

    /**
     * Suffix applied to the key used in resource file
     * lookups for tooltip text.
     */
    public static final String tipSuffix = "Tooltip";

    public static final String openAction  = "open";
    public static final String newAction   = "new";
    public static final String saveAction  = "save";
    public static final String printAction = "print";
    public static final String exitAction  = "exit";
    public static final String showElementTreeAction = "showElementTree";

    class UndoHandler implements UndoableEditListener {

	/**
	 * Messaged when the Document has created an edit, the edit is
	 * added to <code>undo</code>, an instance of UndoManager.
	 */
        public void undoableEditHappened(UndoableEditEvent e) {
	    undo.addEdit(e.getEdit());
	    undoAction.update();
	    redoAction.update();
	}
    }

    /**
     * FIXME - I'm not very useful yet
     */
    class StatusBar extends JComponent {

        public StatusBar() {
	    super();
	    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	}

        public void paint(Graphics g) {
	    super.paint(g);
	}

    }

    // --- action implementations -----------------------------------

    private UndoAction undoAction = new UndoAction();
    private RedoAction redoAction = new RedoAction();

    /**
     * Actions defined by the Notepad class
     */
    private Action[] defaultActions = {
	new NewAction(),
	new OpenAction(),
	new SaveAction(),
	new PrintAction(),
	new ExitAction(),
	new ShowElementTreeAction(),
        undoAction,
        redoAction
    };

    class UndoAction extends AbstractAction {
	public UndoAction() {
	    super("Undo");
	    setEnabled(false);
	}

	public void actionPerformed(ActionEvent e) {
	    try {
		undo.undo();
	    } catch (CannotUndoException ex) {
		System.out.println("Unable to undo: " + ex);
		ex.printStackTrace();
	    }
	    update();
	    redoAction.update();
	}

	protected void update() {
	    if(undo.canUndo()) {
		setEnabled(true);
		putValue(Action.NAME, undo.getUndoPresentationName());
	    }
	    else {
		setEnabled(false);
		putValue(Action.NAME, "Undo");
	    }
	}
    }

    class RedoAction extends AbstractAction {
	public RedoAction() {
	    super("Redo");
	    setEnabled(false);
	}

	public void actionPerformed(ActionEvent e) {
	    try {
		undo.redo();
	    } catch (CannotRedoException ex) {
		System.out.println("Unable to redo: " + ex);
		ex.printStackTrace();
	    }
	    update();
	    undoAction.update();
	}

	protected void update() {
	    if(undo.canRedo()) {
		setEnabled(true);
		putValue(Action.NAME, undo.getRedoPresentationName());
	    }
	    else {
		setEnabled(false);
		putValue(Action.NAME, "Redo");
	    }
	}
    }

    class OpenAction extends NewAction {

	OpenAction() {
	    super(openAction);
	}

        public void actionPerformed(ActionEvent e) {
	    JFrame frame = getFrame();

	    if (verifySave()) {
	    
		StyledDocument styledDoc = PersistenceDialog.openStyledDocument (frame);

		if (styledDoc != null) {
		    // initialize the statusbar
		    status.removeAll();
	    
		    ((JTextPane)getEditor()).setStyledDocument (styledDoc);

		    // we are done... get rid of progressbar
		    styledDoc.addUndoableEditListener(undoHandler);
		    status.removeAll();
		    status.revalidate();

		    if (elementTreePanel != null) {
			SwingUtilities.invokeLater(new Runnable() {
			    public void run() {
				elementTreePanel.setEditor(getEditor());
			    }
			});
		    }
		}
	    }				       
	}
    }

    class PrintAction extends NewAction {

	PrintAction() {
	    super(printAction);
	}

        public void actionPerformed(ActionEvent e) {

	    PrinterJob printJob = PrinterJob.getPrinterJob();
	    printJob.setPrintable (Notepad.this);

	    if (printJob.printDialog())
		try {
		    printJob.print ();
		    System.out.println ("Printing OK.");
		} catch (PrinterException pe) {
		    System.out.println ("Error printing: " + pe.getMessage());
		}
	}
    }

    class SaveAction extends NewAction {

	SaveAction() {
	    super(saveAction);
	}

        public void actionPerformed(ActionEvent e) {

	    saveDocument();
	    //StyledDocument styledDoc = ((JTextPane)getEditor()).getStyledDocument();
	    //boolean saved = PersistenceDialog.saveStyledDocument (frame, styledDoc);
	    //if (false == saved)
	    //System.out.println ("Nao salvou");
	    
	}
    }

    
    private boolean verifySave() {

	if (isSaved) {
	    
	    int resp = StandardDialogs.questionYesNoCancelDialog (null, "O relatorio nao foi salvo", "Voce nao salvou o seu atual relatorio e ira perder as ultimas modificacoes.\nGostaria de salvar antes de continuar?");

	    if (resp == StandardDialogs.CANCEL)
		return false;

	    if (resp == StandardDialogs.YES)
		this.saveDocument();

	}

	return true;
    }

    private boolean saveDocument() {

	StyledDocument styledDoc = ((JTextPane)getEditor()).getStyledDocument();
	boolean saved = PersistenceDialog.saveStyledDocument (getFrame(), styledDoc);
	this.isSaved = true;

	return saved;
    }
	
    class NewAction extends AbstractAction {

	NewAction() {
	    super(newAction);
	}

	NewAction(String nm) {
	    super(nm);
	}

        public void actionPerformed(ActionEvent e) {
	    if (verifySave()) {
		Document oldDoc = getEditor().getDocument();
		if(oldDoc != null)
		    oldDoc.removeUndoableEditListener(undoHandler);
		getEditor().setDocument(new DefaultStyledDocument());
		getEditor().getDocument().addUndoableEditListener(undoHandler);
		revalidate();
	    }
	}
    }

    /**
     * Really lame implementation of an exit command
     */
    class ExitAction extends AbstractAction {

	ExitAction() {
	    super(exitAction);
	}

        public void actionPerformed(ActionEvent e) {
	    //Notepad.this.frame.setVisible(false);
	    //System.exit(0);
	}
    }

    /**
     * Action that brings up a JFrame with a JTree showing the structure
     * of the document.
     */
    class ShowElementTreeAction extends AbstractAction {

	ShowElementTreeAction() {
	    super(showElementTreeAction);
	}

	ShowElementTreeAction(String nm) {
	    super(nm);
	}

	public String runElementTree (Element el) {

	    String strH = "";

	    AttributeSet as = el.getAttributes().copyAttributes();/* 1 */

	    if(as != null) {

		Enumeration        names = as.getAttributeNames(); /* 2 */
		while(names.hasMoreElements()) {

		    Object        nextName = names.nextElement();
		    
		    if(nextName != StyleConstants.ResolveAttribute) {

			Object o = as.getAttribute(nextName);

			if (o instanceof ImageIcon) { /* 3 */

			    String        asString = "Imagem";
			    BufferedImage bImage;

			    if (((ImageIcon)o).getImage() instanceof BufferedImage) {

				bImage = (BufferedImage) ((ImageIcon)o).getImage();

			    } else { 

				// need conversion to BufferedImage
  				bImage = Image2BufferedImage (((ImageIcon)o).getImage()); /* 4 */

			    }

			    try {

				//Saving image as JPEG /* 5 */
				FileOutputStream fos = new FileOutputStream(new File (imageDirectory, asString+imageNumber+".jpg"));
				BufferedOutputStream bos = new BufferedOutputStream(fos);
				JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(bos);
				encoder.encode(bImage);
				bos.close();

				//Generating HTML code /* 6 */
				strH = strH.concat("<center>\n<img SRC=\"");
				strH = strH.concat("images/" + (asString+imageNumber) + ".jpg");
				strH = strH.concat("\">\n</center>\n<br>\n");
				imageNumber++;

			    } catch (Exception e) {

				System.out.println ("Colocar um dialogo");
				e.printStackTrace();

			    }

			} 
		    }
		}
	    } 

	    //extracting text to html code /* 7 */
	    if (el.isLeaf()) { /* 7 */

		strH = strH.concat("</p> \n <p>");

		try {

		    strH = strH.concat(el.getDocument().getText(el.getStartOffset(), el.getEndOffset()-el.getStartOffset()));

		} catch (Exception e) {}
		    
	    }

	    //Recursive traverse of the element tree /* 8 */
	    int n = el.getElementCount();
	    if (n > 0)
		for (int i = 0; i<n; i++)
		    strH = strH.concat(runElementTree (el.getElement(i))); /* 9 */

	    // return htmlCode
	    return strH;
	}

	//convert an Image to a BufferedImage
	public BufferedImage Image2BufferedImage(Image img) {

	    BufferedImage bi = new BufferedImage(img.getWidth(null), img.getHeight(null), BufferedImage.TYPE_INT_RGB);
	    Graphics2D g2d = bi.createGraphics();
	    g2d.drawImage(img, null, null);
	
	    return bi;
	}
	
        public void actionPerformed(ActionEvent e) {

	    if(elementTreeFrame == null) {

		JFileChooser chooser = new JFileChooser();
		String ar = "html";
		CustomFileFilter filter = new CustomFileFilter(ar, "HTML files");
		chooser.setFileFilter(filter);
		int returnVal = chooser.showSaveDialog(getFrame());

		if(returnVal == JFileChooser.APPROVE_OPTION) {
		    // Here I have to save the string to HTML file.....
		    
		    //Creating direcory to put images.
		    imageDirectory = chooser.getSelectedFile();
		    imageDirectory = imageDirectory.getParentFile();
		    imageDirectory = new File (imageDirectory, imageDirectoryName);
		    if ( !imageDirectory.exists())
			imageDirectory.mkdir();
		    
		    imageNumber = 1;

		    String htmlString = "<!doctype html public \"-//w3c//dtd"+
			"html 4.0 transitional//en\">\n<html>\n"+
			"<head>\n<title>Report of Colab</title>\n"+
			"</head>\n<body bgcolor=\"#FFFFFF\">\n";

		    Element[] roots = getEditor().getDocument().getRootElements();
		    htmlString = htmlString.concat(runElementTree (roots[0]));
		    htmlString = htmlString.concat("<br>&nbsp;\n<p>Here comes the Authors Name and Date!!!!!\n</body>\n</html>");

		    try {
			FileOutputStream outFile = new
			    FileOutputStream(chooser.getSelectedFile());
			outFile.write(htmlString.getBytes());
			outFile.close();
		    } catch (IOException ioe) {}
		}

		System.out.println ("1");
		// Create a frame containing an instance of 
		// ElementTreePanel.

//  		try {
//  		    String    title = resources.getString
//  			("ElementTreeFrameTitle");
//  		    elementTreeFrame = new JFrame(title);
//  		} catch (MissingResourceException mre) {
//  		    elementTreeFrame = new JFrame();
//  		}

//  		elementTreeFrame.addWindowListener(new WindowAdapter() {
//  		    public void windowClosing(WindowEvent weeee) {
//  			elementTreeFrame.setVisible(false);
//  		    }
//  		});
//  		Container fContentPane = elementTreeFrame.getContentPane();

//  		fContentPane.setLayout(new BorderLayout());
//  		elementTreePanel = new ElementTreePanel(getEditor());
//  		fContentPane.add(elementTreePanel);
//  		elementTreeFrame.pack();
	    }

//  //  	    elementTreeFrame.show();
	}
    }

    /**
     * Thread to load a file into the text storage model
     */
    class FileLoader extends Thread {

	FileLoader(File f, Document doc) {
	    setPriority(4);
	    this.f = f;
	    this.doc = doc;
	}

        public void run() {
	    try {
		// initialize the statusbar
		status.removeAll();
		JProgressBar progress = new JProgressBar();
		progress.setMinimum(0);
		progress.setMaximum((int) f.length());
		status.add(progress);
		status.revalidate();

		// try to start reading
		Reader in = new FileReader(f);
		char[] buff = new char[4096];
		int nch;
		while ((nch = in.read(buff, 0, buff.length)) != -1) {
		    doc.insertString(doc.getLength(), new String(buff, 0, nch), null);
		    progress.setValue(progress.getValue() + nch);
		}

		// we are done... get rid of progressbar
		doc.addUndoableEditListener(undoHandler);
		status.removeAll();
		status.revalidate();
	    }
	    catch (IOException e) {
		System.err.println(e.toString());
	    }
	    catch (BadLocationException e) {
		System.err.println(e.getMessage());
	    }
	    if (elementTreePanel != null) {
		SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			elementTreePanel.setEditor(getEditor());
		    }
		});
	    }
	}

	Document doc;
	File f;
    }

}
