package colab.client.gui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.text.StyledDocument;
import colab.client.*;
import colab.util.tool.ploter.*;
import colab.util.tool.iToolModel;
import colab.client.*;

public class PersistenceDialog  {

    public PersistenceDialog() {
        super();
    }


    static public PloterModel openPloterModel (Component _parent) {

	try {

	    ColabFileSystem fileSystem = new ColabFileSystem (true);
	    JFileChooser fileChooser = new JFileChooser (fileSystem);
	    fileChooser.setApproveButtonText ("Abrir");
	    fileChooser.setApproveButtonToolTipText("Abre o arquivo selecionado.");
	    fileChooser.setDialogTitle("Abrir arquivo");
	    fileChooser.setFileFilter(new CustomFileFilter(".func", "Modelos de funcoes"));
	    int returnVal = fileChooser.showOpenDialog(_parent);

	    if (returnVal == JFileChooser.APPROVE_OPTION) {

		File file = fileChooser.getSelectedFile();
		//System.out.println("Opening: " + file.getAbsolutePath() + "." );
		PloterModel model = (PloterModel) fileSystem.openModel (file);

		return model;

	    } else {

		//System.out.println("Open command cancelled by user.");

	    }

	} catch (IOException e) {

	    System.out.println ("PersistenceDialog.open(): "+e.getMessage());

	}

	return null;
    }


    static public boolean savePloterModel (Component _parent, PloterModel _model) {

	try {

	    ColabFileSystem fileSystem = new ColabFileSystem (false);
	    JFileChooser fileChooser = new JFileChooser (fileSystem);
	    fileChooser.setApproveButtonText ("Salvar");
            fileChooser.setApproveButtonToolTipText("Salva o arquivo com o nome escolhido.");
	    fileChooser.setDialogTitle("Salvar arquivo");
	    fileChooser.setFileFilter(new CustomFileFilter(".func", "Modelos de funcoes"));

	    boolean end = false;

	    do {
		int returnVal = fileChooser.showSaveDialog(_parent);

		if (returnVal == JFileChooser.APPROVE_OPTION) {

		    File file = fileChooser.getSelectedFile();

		    if (! file.getParent().equals(File.separator)) {
			fileSystem.saveModel (file, (iToolModel) _model);
			return true;
                    } else {
                        StandardDialogs.errorDialog (null, "Erro", "Voc deve entrar no seu diretrio ou no diretrio do grupo para salvar o aquivo.", "file:\\c:\\java\\docs\\api\\index.html");
                    }

		} else
		    end = true;

	    } while (! end);


	} catch (IOException e) {

	    System.out.println ("PersistenceDialog.save(): "+e.getMessage());

	}

	return false;
    }

    static public boolean publishPloterModel (PloterModel _model) {

	try {

	    ColabFileSystem fileSystem = new ColabFileSystem (false);
	    _model.setAllInvisible();

	    fileSystem.publishModel ( (iToolModel) _model);
	    return true;

	} catch (IOException e) {

	    System.out.println ("PersistenceDialog.publishModel(): "+e.getMessage());

	}

	return false;
    }


    static public StyledDocument openStyledDocument (Component _parent) {

	try {

	    ColabFileSystem fileSystem = new ColabFileSystem (true);
	    JFileChooser fileChooser = new JFileChooser (fileSystem);
	    fileChooser.setApproveButtonText ("Abrir");
	    fileChooser.setApproveButtonToolTipText("Abre o arquivo selecionado.");
	    fileChooser.setDialogTitle("Abrir arquivo");
	    fileChooser.setFileFilter(new CustomFileFilter(".relat", "Relatorios"));
	    int returnVal = fileChooser.showOpenDialog(_parent);

	    if (returnVal == JFileChooser.APPROVE_OPTION) {

		File file = fileChooser.getSelectedFile();
		StyledDocument model = fileSystem.openStyledDocument (file);

		return model;

	    } else {

		//System.out.println("Open command cancelled by user.");

	    }

	} catch (IOException e) {

	    System.out.println ("PersistenceDialog.open(): "+e.getMessage());

	}

	return null;
    }


    static public boolean saveStyledDocument (Component _parent, StyledDocument _model) {

	try {

	    ColabFileSystem fileSystem = new ColabFileSystem (false);
	    JFileChooser fileChooser = new JFileChooser (fileSystem);
	    fileChooser.setApproveButtonText ("Salvar");
            fileChooser.setApproveButtonToolTipText("Salva o arquivo com o nome escolhido.");
	    fileChooser.setDialogTitle("Salvar arquivo");
	    fileChooser.setFileFilter(new CustomFileFilter(".relat", "Relatorios"));

	    boolean end = false;

	    do {
		int returnVal = fileChooser.showSaveDialog(_parent);

		if (returnVal == JFileChooser.APPROVE_OPTION) {

		    File file = fileChooser.getSelectedFile();
                    //System.out.println("Saving: " + file.getAbsolutePath() + ".");
                    //System.out.println("\tparent: " + file.getParent() + ".\n\tRoots:");
                    //File[] files = file.listRoots();
                    //for (int i=0; i < files.length; i++)
                    //    System.out.println("\t\t- " + files[i].getAbsolutePath());

		    if (! file.getParent().equals(File.separator)) {
			fileSystem.saveStyledDocument (file, _model);
			return true;

		    } else {

			StandardDialogs.errorDialog (null, "Erro", "Voc deve entrar no seu diretrio ou no diretrio do grupo para salvar o aquivo.", "file:\\c:\\java\\docs\\api\\index.html");

		    }

		} else
		    end = true;

	    } while (! end);


	} catch (IOException e) {

	    System.out.println ("PersistenceDialog.save(): "+e.getMessage());

	}

	return false;
    }
}
