package colab.client.gui.forum;

import colab.util.forum.*;
import colab.client.gui.*;
import java.util.*;
import java.rmi.*;
import colab.util.*;
import javax.swing.tree.*;
import javax.swing.SwingUtilities;

public class ForumTreeTableModel extends AbstractTreeTableModel implements TreeTableModel {
    // Names of the columns.
    static protected String[] cNames = {"Topico", "Assunto", "De", "Data"};

    // Types of the columns.
    static protected Class[]  cTypes = {TreeTableModel.class, String.class, String.class, Date.class};

    iForumServer forumServer;
    String       loginName;

    Vector       forumMsgs;
    HashMap      nodes;
    int          numRows = 0;
    String[]     readedMsgs;

    Date         last = new Date(0);

    DefaultMutableTreeNode2 root;


    public ForumTreeTableModel (iForumServer _forumServer, String _loginName) {

	super (new DefaultMutableTreeNode2 (new RootNode("raiz")));

	forumServer = _forumServer;
	loginName   = _loginName;
	nodes       = new HashMap();
	root        = (DefaultMutableTreeNode2) super.getRoot();
	forumMsgs   = new Vector();

	try {

	    readedMsgs = forumServer.getReaded(loginName);

	} catch (ServerIsGoingDownException e) {

	    System.out.println ("O servidor esta saindo do ar. Make Error log, ForumTreeTableModel.constructor(): "+e.getMessage());
            e.printStackTrace();

        } catch (RemoteException e) {

	    System.out.println ("Make Error log, forumTreeTableModel.ForumTableModel.constructor(): "+e.getMessage());
            e.printStackTrace();

	}

	this.verifyMsgs();
    }


    public void insertChild (DefaultMutableTreeNode2 _parent, DefaultMutableTreeNode2 _child) {

	Object[] path     = _parent.getPath();
	int[] indices     = {_parent.getChildCount()};
	Object[] children = {_child};

	_parent.add (_child);

	fireTreeStructureChanged (this, path, null, null);
	fireTreeNodesChanged (this, path, indices, children);
    }


    public boolean verifyMsgs () {

	boolean newMsg = false;

        try{
	    int size = forumServer.getSize();

	    if (size > numRows) {

		ForumMsg[] msgs = forumServer.getMsgs(last);

		for (int i = 0; i < msgs.length; i++) {

		    forumMsgs.add (msgs[i]);

		    // wrap messages with ForumMsgNode
		    ForumMsgNode node = new ForumMsgNode (false, msgs[i]);
		    DefaultMutableTreeNode2 treeNode = new DefaultMutableTreeNode2(node);

		    nodes.put (node.getId(), treeNode);

		    // set parents and childrens
		    String replyTo = node.getReplyTo();

		    if (replyTo == null) {

			this.insertChild (root, treeNode);

		    } else {

			DefaultMutableTreeNode2 parent = (DefaultMutableTreeNode2) nodes.get (replyTo);
			if (parent != null)
			    this.insertChild (parent, treeNode);

		    }

		} //for (int i = 0; i < forumMsgs.length; i++) {

		if (0 == last.getTime())
		    // only the first time
		    for (int i = 0; i < readedMsgs.length; i++) {

			//set readed messages
			DefaultMutableTreeNode2 node = (DefaultMutableTreeNode2) nodes.get(readedMsgs[i]);
			if (node != null)

			    ((iForumMsgNode)node.getUserObject()).setReaded (true);

		    } //for (int i = 0; i < readedMsgs.length; i++)

		newMsg = true;
		numRows = forumMsgs.size();
		last = msgs [msgs.length-1].getDate();
	    } //if (size != forumMsgs.length) {

        } catch (ServerIsGoingDownException e) {
            System.out.println ("O servidor esta saindo do ar. Make Error log, forumTreeTableModel.ForumForumModel.verifyMsgs(): "+e.getMessage());
            e.printStackTrace();
        } catch (RemoteException e) {
            System.out.println ("Make Error log, forumTreeTableModel.ForumForumModel.verifyMsgs(): "+e.getMessage());
            e.printStackTrace();
        }

	return newMsg;
    }

    public int getChildCount (Object _node) {
	return ((DefaultMutableTreeNode2)_node).getChildCount();
    }

    public Object getChild (Object _node, int _i){
	return ((DefaultMutableTreeNode2)_node).getChildAt (_i);
    }


    //public ForumMsg getMsg(int _i) {
	//if ( (_i >= 0) && (_i < forumMsgs.length) ) {
	//    iForumMsg node = root;
	//    return forumMsgs[_i];
	//}
    //  return null;
    //}

    public int getColumnCount() {
	return cNames.length;
    }

    public String getColumnName (int _col) {
	return cNames[_col];
    }

    public Class getColumnClass (int _col) {
	return cTypes[_col];
    }

    public Object getValueAt (Object _node, int _col) {
	iForumMsgNode node = (iForumMsgNode) ((DefaultMutableTreeNode2)_node).getUserObject();
	switch (_col) {
	case 0:

	    String str = node.getTopic();
	    return node.getTopic();
	case 1:
	    return node.getSubject();
	case 2:
	    if (node.getAnonymity())
		return "Anonimo";
	    else
		return node.getSender();
	case 3:
	    return node.getDate();
	}

	return null;
    }


}

class DefaultMutableTreeNode2 extends DefaultMutableTreeNode {

    public DefaultMutableTreeNode2 () {
	super();
    }

    public DefaultMutableTreeNode2 (Object _obj) {
	super (_obj);
    }

    public int getChildCount() {
	int i = super.getChildCount();

	return i;
    }

    public TreeNode getChildAt (int _i) {
	TreeNode tn = super.getChildAt (_i);

	return tn;
    }
}
