package colab.client.tool.ploter;

import javax.swing.table.*;
import javax.swing.*;
import java.awt.*;
import java.rmi.*;
import java.rmi.server.*;
import java.util.*;
import java.io.*;
import colab.util.tool.*;
import colab.util.tool.ploter.*;

public class FunctionTableModel extends AbstractTableModel implements iPloterListener, Remote {

    Vector functions;

    iPloterModel ploterModel;
    int mode;

    String columns[] = {"Nome", "Funcao", "Dominio", "Trava/Cor"};

    protected FunctionTableModel (iPloterModel _ploterModel, int _mode) throws RemoteException {

	//System.out.println ("\nCriando um novo FunctionTableModel\n");

	this.mode = _mode;
	try {

	    if (this.mode == PloterPanel.SHARED) {
		//System.out.println ("tornei a tabela visivel remotamente");
		UnicastRemoteObject.exportObject ( (iPloterListener) this);
	    }

	    ploterModel = _ploterModel;
	    ploterModel.addListener (this);

	    this.resetModel();
//  	    functions    = new Vector();

//  	    Function[] functionList = ploterModel.getFunctionList();

//  	    synchronized (functions) {

//  		for (int i = 0; i < functionList.length; i++)
//  		    if ( !functionList[i].getInvisible())
//  			functions.add (functionList[i]);

//  	    }

	} catch (RemoteException e) {

	    //e.printStackTrace();
	    throw new RemoteException("FunctionTableModel.contructor(), " +  e.getMessage());

	}
    }


    public void resetModel () throws RemoteException {

	try {

	    //System.out.println ("\tDownloading function to table model");
	    Function[] functionList = ploterModel.getFunctionList();

	    functions    = new Vector();
	    synchronized (functions) {

		for (int i = 0; i < functionList.length; i++)
		    if ( !functionList[i].getInvisible())
			functions.add (functionList[i]);
	    }

	} catch (RemoteException e) {

	    //e.printStackTrace();
	    throw new RemoteException("FunctionTableModel.resetModel(), " +  e.getMessage());

	}
    }

    public Function getFunction (int _i) {

	//System.out.println ("getFunction(" + _i + ")");

	try {

	    return (Function) functions.elementAt (_i);

	} catch (Exception e) {

	    //System.out.println ("nao tem essa linha ai");
	    return null;

	}

    }


    public void ploterChanged (PloterModelEvent _e) throws RemoteException {

	//System.out.println ("Evento: " + _e.getType());

	if (PloterModelEvent.FUNCTION_CHANGED == _e.getType()) {

	    Function function = _e.getFunction();

	    //System.out.println ("\n(FunctionTableModel)Function changed: " + function.getName());
	    //System.out.println ("\tcor: " + function.getColor());

//  	    FunctionLock fLock = function.getFunctionLock();
//  		if (fLock == null)
//  		    System.out.println ("\tno lock");
//  		else
//  		    System.out.println ("\twith lock");

	    synchronized (functions) {

		int i = this.getFunctionIndex (_e.getId());
		if (i != -1)
		    functions.setElementAt (function, i);

	    }

	    fireTableDataChanged();

	} else if (PloterModelEvent.FUNCTION_INSERTED == _e.getType()) {

	    //System.out.println ("Function inserted");

	    if ( ! _e.getFunction().getInvisible() ) {
		//if this is not invisible

		synchronized (functions) {

		    functions.add (_e.getFunction());

		}

		fireTableDataChanged();
	    }

	} else if (PloterModelEvent.FUNCTION_REMOVED == _e.getType()) {

	    //System.out.println ("Function removed: " + _e.getId());

	    synchronized (functions) {

		int i = this.getFunctionIndex (_e.getFunction().getId());
		functions.removeElementAt (i);

	    }

	    fireTableDataChanged();

	}
    }


    public void update (iEvent _e) throws RemoteException {

	PloterModelEvent e = (PloterModelEvent) _e;
	//System.out.println ("Evento: " + e.getType());

	if (PloterModelEvent.FUNCTION_CHANGED == e.getType()) {

	    Function function = e.getFunction();

	    //System.out.println ("\n(FunctionTableModel)Function changed: " + function.getName());
	    //System.out.println ("\tcor: " + function.getColor());

//  	    FunctionLock fLock = function.getFunctionLock();
//  		if (fLock == null)
//  		    System.out.println ("\tno lock");
//  		else
//  		    System.out.println ("\twith lock");

	    synchronized (functions) {

		int i = this.getFunctionIndex (e.getId());
		if (i != -1)
		    functions.setElementAt (function, i);

	    }

	    fireTableDataChanged();

	} else if (PloterModelEvent.FUNCTION_INSERTED == e.getType()) {

	    //System.out.println ("Function inserted");

	    if ( ! e.getFunction().getInvisible() ) {
		//if this is not invisible

		synchronized (functions) {

		    functions.add (e.getFunction());

		}

		fireTableDataChanged();
	    }

	} else if (PloterModelEvent.FUNCTION_REMOVED == e.getType()) {

	    //System.out.println ("Function removed: " + e.getId());

	    synchronized (functions) {

		int i = this.getFunctionIndex (e.getFunction().getId());
		functions.removeElementAt (i);

	    }

	    fireTableDataChanged();

	}
    }

    private int getFunctionIndex (String _id) {

	for (int i = 0; i < functions.size(); i++)
	    if ( ((Function)functions.elementAt(i)).getId().equals (_id))
		return i;

	return -1;

    }

    public int getRowCount() {

	return functions.size();

    }


    public int getColumnCount() {

	return columns.length;

    }

//      public Class getColumnClass (int _col) {

//  	return String.class;

//      }

    public Class getColumnClass (int _col) {

	if (_col == 0)
	    return String.class;
	else if  (_col == 1)
	    return String.class;
	else if (_col == 2)
	    return String.class;
	else if  (_col == 3)
	    if (this.mode == PloterPanel.SHARED)
		return  JLabel.class;
	    else // this.mode == PloterPanel.ALONE
		return Color.class;

	return String.class;
    }

    public Object getValueAt (int _row, int _col) {

	if (_col == 0)

	    return ( (Function)functions.elementAt(_row)).getName();

	else if (_col == 1)

	    return ( (Function)functions.elementAt(_row)).getFunctionString();

	else if (_col == 2)

	    return "["+ ((Function)functions.elementAt(_row)).getXMin()+","+((Function)functions.elementAt(_row)).getXMax()+"]";

	else if (_col == 3) {

	    Function function = (Function) functions.elementAt(_row);

	    if (this.mode == PloterPanel.SHARED) {

		JLabel owner = new JLabel("");
		owner.setForeground (function.getColor());

		FunctionLock fLock = function.getFunctionLock();

		if (null != fLock)
		    owner.setText (fLock.getOwnerName());

		return owner;

	    } else
		return function.getColor();
	}

	return null;
    }


    public String getColumnName (int columnIndex) {

	if (3 == columnIndex)
	    if (this.mode == PloterPanel.SHARED)
		return "Trava";
	    else
		return "Cor";

	return columns[columnIndex];

    }


    public boolean isCellEditable(int _row, int _col) {

	if ( (this.mode == PloterPanel.ALONE) && (_col == 3) )
	    return true;

	return false;

    }


    public void exit() {

	try {

	    ploterModel.removeListener(this);

	} catch (RemoteException e) {

	    System.out.println ("Problemas para removeListener. FunctionTableModel.exit()" + e.getMessage());
	    e.printStackTrace();

	}

    }
}
