package colab.client.tool.ploter;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import colab.util.tool.ploter.*;
import colab.util.tool.*;
import java.rmi.RemoteException;
import java.io.*;
import java.rmi.*;
import java.rmi.server.*;
import java.util.*;

public class Graph extends JPanel implements MouseMotionListener, iPloterListener, Runnable, Remote {

    private Object updating = new Object();
    private Bound  bound;

    private double pas;
    private int steps;
    private JTextArea infoArea = null;

    private boolean axes = true;
    private boolean axesLabel = true;
    private boolean grid = false;

    //Display box
    int DW = 16;
    int DH = 16;
    int LONG = 5;
    int LONGD = 2;
    int space = 2;

    // Mouse Position
    private boolean isListen = false;
    private double lastX, lastY;
    private boolean firstTime = true;

    private iPloterModel ploterModel;
    private Map functions;

    public Graph(iPloterModel _pModel, JTextArea info, int _steps) throws RemoteException {

	//System.out.println ("\nCriando um novo graph\n");
	try {

	    this.infoArea = info;
	    this.steps = _steps;

	    UnicastRemoteObject.exportObject ( (iPloterListener) this);

	    this.setModel (_pModel);

	} catch (RemoteException e) {

	    throw new RemoteException ("Graph.constructor(), " +  e.getMessage());

	}

    }

    public void setModel (iPloterModel _pModel) throws RemoteException {

	try {

	    if (null != ploterModel)
		ploterModel.removeListener(this);

	    ploterModel = _pModel;
	    ploterModel.addListener (this);

	    this.resetModel();
//  	    bound = ploterModel.getBound();

//  	    functions = Collections.synchronizedMap(new HashMap());

//  	    Function[] functionList = ploterModel.getFunctionList ();
//  	    for (int i = 0; i < functionList.length; i++) {
//  		GraphFunction gFunction = new GraphFunction (functionList[i]);
//  		this.setRootNode (gFunction);
//  		this.setPoints   (gFunction);
//  		functions.put (gFunction.getId(), gFunction);
//  	    }
//  	    //System.out.println ("Numeros de funcoes: " + functions.size());

//  	    this.updateUI();

	} catch (RemoteException e) {

	    throw new RemoteException ("colab.client.tool.ploter.Graph.setModel(), " + e.getMessage());
	    //e.printStackTrace();

	}
    }


    protected void resetModel() throws RemoteException {

	try {

	    //System.out.println ("\tDownloading function to graph.");

	    bound = ploterModel.getBound();

	    Function[] functionList = ploterModel.getFunctionList ();
	    functions = Collections.synchronizedMap(new HashMap());
	    synchronized (functions) {
		for (int i = 0; i < functionList.length; i++) {
		    GraphFunction gFunction = new GraphFunction (functionList[i]);
		    this.setRootNode (gFunction);
		    this.setPoints   (gFunction);
		    functions.put (gFunction.getId(), gFunction);
		}
	    }
	    //System.out.println ("Numeros de funcoes: " + functions.size());

	    this.updateUI();

	} catch (RemoteException e) {

	    throw new RemoteException ("colab.client.tool.ploter.Graph.resetModel(), " + e.getMessage());
	    //e.printStackTrace();
	}
    }

    private void setRootNode(GraphFunction _function) {

	Node root;

	if (null == _function.getFunctionString()) {

	    root = null;

	} else {
	    FionParserCustom parser;
	    FionPTMCustom lexer;

	    StringBufferInputStream is = new StringBufferInputStream(_function.getFunctionString());
	    lexer = new FionPTMCustom(new ASCII_CharStream(is,1,1));
	    parser = new FionParserCustom(lexer);

	    try {

		parser.Expression();
		_function.setValid (true);
		this.writeText (_function.getName() + ": sintaxe ok\n");

	    } catch (ParseError e) {

		_function.setValid (false);
		this.writeText (parser.ErrorMsg() + "Erro de sintaxe na funcao "+ _function.getName() + "\n");//Parser incorreto\n");

	    }

	    root = parser.getRootNode();
	}

	_function.setRootNode (root);
    }

    public void setSteps (int _steps) {
	if (steps != _steps) {
	    steps = _steps;
	    this.resetFunctions();
	    this.updateUI();
	}
    }

    public void updateUI() {
	new Thread(this).start();
    }

    public void run() {
	this.repaint();
    }

    private void setPoints (GraphFunction _function) {

	Node root = _function.getRootNode();

	if ( null != root ) {

	    double   pas = (bound.x2-bound.x1)/(steps);
	    double[] tabx = new double[steps];
	    double[] taby = new double[steps];
	    boolean[] def = new boolean[steps];

	    State s = new State();
	    double x = bound.x1;
	    double v;

	    int NbreInf,NbreUndef;
	    NbreInf = NbreUndef = 0;
	    boolean defined = true;

	    // evaluate function and calculate min/max
	    double minx = bound.x1;
	    double maxx = bound.x2;
	    double miny = Double.MAX_VALUE;
	    double maxy = -Double.MAX_VALUE;

	    for (int i = 0; i < steps; i++) {
		ASTIdentifier.setValue("x",x);
		root.Eval(s);
		v = s.pop();
		tabx[i] = x;
		taby[i] = v;

		if (Double.isNaN(v)) {

		    NbreUndef++;
		    defined = false;

		} else if (Double.isInfinite(v)) {

		    NbreInf++;
		    defined = false;

		} else
		    defined = true;

		if (defined) {

		    if (v > maxy)
			maxy = v;

		    if (v < miny)
			miny = v;

		} //else System.out.println("Undefined: "+x);

		def[i] = defined;
		x += pas;
	    }

	    _function.setXPoints (tabx);
	    _function.setYPoints (taby);
	    _function.setIsDefined (def);

	}
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        FontMetrics f = g.getFontMetrics();
        int w = getSize().width-2*DW;
        int h = getSize().height-2*DH;

        // clear
        g.setColor(Color.white);
        g.fillRect(0,0,getSize().width,getSize().height);
        g.setColor(Color.black);
        g.drawRect(DW/2,DH/2,w+DW,h+DH);

        if (0 == functions.size()) {                  // NO FUNCTION

            String s = "Nenhuma funcao";
            FontMetrics fm = g.getFontMetrics();
            g.drawString(s,(w-fm.stringWidth(s))/2, (h-fm.getHeight())/2);

	} else {

            double kx, lx, ky, ly;
	    double mix, miy, max, may;
            mix = bound.x1; max = bound.x2;
            miy = bound.y1; may = bound.y2;
            kx = w/(max-mix);
            lx = -mix*kx+DW;
            ky = -h/(may-miy);
            ly = -may*ky+DH;

            if (axes)
		this.paintAxes (g, w, h);

	    GraphFunction[] arrayFunction;
	    synchronized (functions) {
		//arrayFunction = new GraphFunction [functions.size()];
		Collection col = functions.values();
		arrayFunction = (GraphFunction[]) col.toArray(new GraphFunction[functions.size()]);

		// first paint the invisible functions
		for (int i = 0; i < arrayFunction.length; i++)
		    if (arrayFunction[i].getInvisible() && arrayFunction[i].getValid())
			    this.paintFunction (g, arrayFunction[i], w, h); //, kx, ky, lx, ly);

		// after paint the visible functions
		for (int i = 0; i < arrayFunction.length; i++)
		    if (! arrayFunction[i].getInvisible() && arrayFunction[i].getValid())
			this.paintFunction (g, arrayFunction[i], w, h); //, kx, ky, lx, ly);

	    }

        }
    }

    private void resetFunctions() {

	    GraphFunction[] arrayFunction;

	    synchronized (functions) {
		//arrayFunction = new GraphFunction [functions.size()];
		Collection col = functions.values();
		arrayFunction = (GraphFunction[]) col.toArray(new GraphFunction[functions.size()]);
	    }

	    for (int i = 0; i < arrayFunction.length; i++)
		this.setPoints (arrayFunction[i]);
    }


    private void paintFunction (Graphics _g, GraphFunction _gFunction, int _w, int _h) {

	if (_gFunction == null)
	    System.out.println (" => null function ");

	double kx, lx, ky, ly;
	double mix, miy, max, may;

	mix = bound.x1; max = bound.x2;
	miy = bound.y1;  //_gFunction.getYMin();
	may = bound.y2; //_gFunction.getYMax();
	kx = _w/(max-mix);
	lx = -mix*kx+DW;
	ky = -_h/(may-miy);
	ly = -may*ky+DH;

	double[] tabx = _gFunction.getXPoints();
	double[] taby = _gFunction.getYPoints();
	boolean[] def = _gFunction.getIsDefined();
	String xMin   = _gFunction.getXMin();
	String xMax   = _gFunction.getXMax();

	//System.out.println ("xMin = " + xMin + " xMax = " + xMax);
	int iInit  = this.getIndex (xMin, tabx, true);
	int iFinal = this.getIndex (xMax, tabx, false);

	//System.out.println ("init = " + iInit + " final = " + iFinal);
	//System.out.println ("init = " + tabx[0] + " final = " + tabx[steps-1]);

	int cw, ch, cwo = 0, cho = 0;
	boolean prev = false;

	if (_gFunction.getInvisible())
	    _g.setColor (new Color(200, 200, 200));
	else
	    _g.setColor (_gFunction.getColor());

	for (int i = iInit; i < iFinal; i++) {

	    if (def[i]) {

		cw = (int)(kx*tabx[i]+lx);
		ch = (int)(ky*taby[i]+ly);

		if (prev) {
		    _g.drawLine(cwo,cho,cw,ch);
		    //if (_gFunction.getInvisible()) {
			//_g.drawLine(cwo,cho-2,cw,ch-2);
			//_g.drawLine(cwo,cho-1,cw,ch-1);
			//_g.drawLine(cwo,cho+1,cw,ch+1);
			//_g.drawLine(cwo,cho+2,cw,ch+2);
		    //}
		}
		cwo = cw;
		cho = ch;
		prev = true;
	    } else prev = false;
	}

    }


    private int getIndex (String _dominion, double[] _tabx, boolean _initial) {

	double dominion;

	try {

	    dominion = Double.parseDouble (_dominion);

	} catch (Exception e) {

	    if (_initial)
		return 0;
	    else
		return steps;
	}

	if (dominion <= _tabx[0])
	    return 0;

	if (dominion >= _tabx[steps-1])
	    return steps;

	return this.binarySearch (_tabx, 0, steps-1, dominion);

    }


    private int binarySearch (double[] _tabx, int _init, int _final, double _dom) {

	if ( (_final-_init) < 2 )
	    return _final;

	int middle = (_final+_init) / 2;

	if ( _dom > _tabx [middle])
	    return this.binarySearch (_tabx, middle, _final, _dom);

	else if ( _dom < _tabx [middle])
	    return this.binarySearch (_tabx, _init, middle, _dom);

	else
	    return middle;

    }


    private void paintAxes (Graphics g, int w, int h) {

	double kx,lx,ky,ly,ux,uy,sx,sy,x,y,uxd,uyd,mix,miy,max,may;
	mix = bound.x1; max = bound.x2;
	miy = bound.y1; may = bound.y2;
	kx = w/(max-mix);
	lx = -mix*kx+DW;
	ky = -h/(may-miy);
	ly = -may*ky+DH;

	ux = Math.pow(10.0,Math.ceil(Math.log(max-mix)/Math.log(10.0)-1));
	uy = Math.pow(10.0,Math.ceil(Math.log(may-miy)/Math.log(10.0)-1));

	//this.writeText("X: "+mix+" -> "+max+" Y: "+miy+" -> "+may+"\n");// + "Unidades:"+ux+" (x) and "+uy+" (y)\n");

	uxd = ux/2.0;
	uyd = uy/2.0;

	sx = Math.ceil(mix/ux)*ux;
	sy = Math.ceil(miy/uy)*uy;

	int aux1,aux2, hAxis;
	g.setColor(Color.black);

	if (! isListen) {
	    isListen = true;
	    addMouseMotionListener(this);
	}

	if ((may > 0) && (miy) < 0)
	    hAxis = (int)ly;
	else
	    hAxis = h+DH;

	g.drawLine(DW,hAxis,DW+w,hAxis);

	x = sx-uxd;                   // start with half unit

	if (x>=mix) {
	    aux2 = (int)(x*kx+lx);
	    g.drawLine(aux2,hAxis,aux2,hAxis+LONGD);
	}

	x = sx;                       // units and half units
	while (x < max) {
	    aux2 = (int)(x*kx+lx);
	    g.drawLine(aux2,hAxis,aux2,hAxis+LONG);
	    x += uxd;
	    aux2 = (int)(x*kx+lx);
	    g.drawLine(aux2,hAxis,aux2,hAxis+LONGD);
	    x += uxd;
	}

	if ((max > 0) && (mix) < 0)  // VERTICAL AXIS
	    aux1 = (int)lx;
	else
	    aux1 = DH;

	g.drawLine(aux1,DH,aux1,DH+h);

	y = sy-uyd;                   // start with half unit ?
	if (y >= miy) {
	    aux2 = (int)(y*ky+ly);
	    g.drawLine(aux1,aux2,aux1+LONGD,aux2);
	}

        y = sy;                       // units and half units
	while (y < may) {
	    aux2 = (int)(y*ky+ly);
	    g.drawLine(aux1,aux2,aux1+LONG,aux2);
	    y += uyd;
	    aux2 = (int)(y*ky+ly);
	    g.drawLine(aux1,aux2,aux1+LONGD,aux2);
	    y += uyd;
	}

        if (axesLabel) {
	    g.setColor(new Color(60, 150, 60));
	    FontMetrics fm = g.getFontMetrics();
	    g.drawString(String.valueOf(may), DW+space, DH+fm.getAscent());
	    g.drawString(String.valueOf(miy), DW+space, h+DH);
	    g.drawString(String.valueOf(mix), DW+space, hAxis+fm.getAscent());
	    g.drawString(String.valueOf(max), w+DW-fm.stringWidth(String.valueOf(max)), hAxis+fm.getAscent());
	}
    }

    public void ploterChanged (PloterModelEvent _e) throws RemoteException {

//  	try {
	if (PloterModelEvent.BOUND_CHANGED == _e.getType()) {

	    try {
		bound = ploterModel.getBound();

		this.resetFunctions();
		this.updateUI();

	    } catch (RemoteException e) {
		System.out.println (e.getMessage());
	    }

	} else if (PloterModelEvent.FUNCTION_CHANGED == _e.getType()) {

	    GraphFunction graphFunction = new GraphFunction (_e.getFunction());

	    this.setRootNode (graphFunction);
	    this.setPoints   (graphFunction);

	    if ( ! _e.getId().equals (graphFunction.getId()) )
		functions.remove (_e.getId());

	    functions.put (graphFunction.getId(), graphFunction);

	    this.updateUI();

	} else if (PloterModelEvent.FUNCTION_INSERTED == _e.getType()) {

	    GraphFunction graphFunction = new GraphFunction (_e.getFunction());
	    this.setRootNode (graphFunction);
	    this.setPoints   (graphFunction);

	    functions.put (graphFunction.getId(), graphFunction);
	    this.updateUI();

	} else if (PloterModelEvent.FUNCTION_REMOVED == _e.getType()) {

	    Function function = _e.getFunction();
	    functions.remove (function.getId());

	    this.updateUI();

	}

//  	} catch (RemoteException e) {
//  		//never throwed, ploterModel is local;
//  	}

    }

    public void update (iEvent _e) throws RemoteException {

        PloterModelEvent e = (PloterModelEvent) _e;
//  	try {
	if (PloterModelEvent.BOUND_CHANGED == e.getType()) {

	    try {
		bound = ploterModel.getBound();

		this.resetFunctions();
		this.updateUI();

	    } catch (RemoteException ex) {
		System.out.println (ex.getMessage());
	    }

	} else if (PloterModelEvent.FUNCTION_CHANGED == e.getType()) {

	    GraphFunction graphFunction = new GraphFunction (e.getFunction());

	    this.setRootNode (graphFunction);
	    this.setPoints   (graphFunction);

	    if ( ! e.getId().equals (graphFunction.getId()) )
		functions.remove (e.getId());

	    functions.put (graphFunction.getId(), graphFunction);

	    this.updateUI();

	} else if (PloterModelEvent.FUNCTION_INSERTED == e.getType()) {

	    GraphFunction graphFunction = new GraphFunction (e.getFunction());
	    this.setRootNode (graphFunction);
	    this.setPoints   (graphFunction);

	    functions.put (graphFunction.getId(), graphFunction);
	    this.updateUI();

	} else if (PloterModelEvent.FUNCTION_REMOVED == e.getType()) {

	    Function function = e.getFunction();
	    functions.remove (function.getId());

	    this.updateUI();

	}

//  	} catch (RemoteException e) {
//  		//never throwed, ploterModel is local;
//  	}

    }

    public Dimension getPreferredSize() {
    return new Dimension(250,250);  }

    public void mouseMoved(MouseEvent e) {
        // Show relative mouse position
        double x = e.getX()-DW, y = e.getY()-DH;
        Graphics g = getGraphics();
        g.setXORMode(new Color(255, 0, 255));
        if (! firstTime) {
            printMousePosition (lastX, lastY, g);
        }
        firstTime = false;
        int w = getSize().width - 2*DW;
        double kx = (bound.x2 - bound.x1)/w;
        x = (int) ((bound.x1 + x*kx) * 10);
        x = ((double)x) /10;
        int h = getSize().height - 2*DH;
        double ky = (bound.y1 - bound.y2)/h;
        y = (int) ((bound.y2 + y*ky) * 10);
        y = y / 10;
        printMousePosition (x, y, g);
        g.setPaintMode();
        lastX = x; lastY = y;
    }

    private void printMousePosition (double x, double y, Graphics g) {
        String pos = "("+String.valueOf(x)+","+String.valueOf(y)+")";
        int tam =  g.getFontMetrics().stringWidth(pos);
        //g.drawString (pos, getSize().width-(DW+tam), DH+g.getFontMetrics().getAscent());
    }

    public void mouseDragged(MouseEvent e) {
    }


    private void writeText (String _txt) {

	if (null != this.infoArea) {
	    this.infoArea.append (_txt);
	    JScrollPane infoScroll = (JScrollPane) this.infoArea.getParent().getParent();
	    infoScroll.getVerticalScrollBar().setValue(infoScroll.getVerticalScrollBar().getMaximum());
	}

    }


    public void exit() {

	try {

	    ploterModel.removeListener(this);

	} catch (RemoteException e) {

	    System.out.println ("Problemas para removeListener. Graph.exit()" + e.getMessage());

	}

    }

}
