package colab.client.tool.ploter;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.io.*;
import java.util.*;
import java.rmi.*;
import java.rmi.server.*;
import colab.util.tool.*;
import colab.util.tool.ploter.*;
import colab.client.gui.*;
import colab.client.*;
import colab.client.tool.*;

public class PloterPanel extends ToolPanel implements ActionListener, iPloterListener, FocusListener {

    public static final int SHARED = 1; // Remote
    public static final int ALONE  = 2; // Local
    protected int mode;
    protected String ploterTypeName;

    public static final int STEPS_DEFAULT = 300;

    protected static final Border outBorder = BorderFactory.createEmptyBorder (4, 6, 4, 6);
    protected static final Border inBorder = BorderFactory.createEmptyBorder (4, 4, 4, 4);

    protected JPanel functionPanel;
    protected JPanel tablePanel;
    protected JPanel graphPanel;
    protected JPanel editingPanel;
    protected JPanel infoPanel;

    protected boolean   functionListVisible = true;
    protected JMenuItem showHideItem;

    protected Border defaultBorder = BorderFactory.createCompoundBorder(
                                               outBorder,
                                               BorderFactory.createCompoundBorder (
                                                            BorderFactory.createEtchedBorder(),
                                                            inBorder));
    protected Border lineBorder = BorderFactory.createTitledBorder(
                                        LineBorder.createGrayLineBorder(),
                                        "Sala: ",
                                        TitledBorder.RIGHT,
                                        TitledBorder.TOP);



    protected String loginName;

    protected Thread update = null;

    protected Graph graph;

    protected JTextField  functionJText, functionNameJText, functionXMinJText, xMinJText, xMaxJText;
    protected JTextField  x1JText, y1JText, x2JText, y2JText, stepTxt;
    protected JTextArea   infoJText;
    protected JButton     unlock;

    protected Bound        bound;
    protected int          steps = STEPS_DEFAULT;
    protected iPloterModel ploterModel;

    protected Function     editingFunction;
    protected FunctionLock myFunctionLock;
    protected boolean      isEditing = false;

    protected FunctionTableModel tableModel;
    protected JTable             functionTable;

    protected Color myColor;

    protected boolean saved = true;

    public PloterPanel() {
    }

    public PloterPanel (iPloterModel _ploterModel) {

        this.mode = PloterPanel.ALONE;
	this.ploterTypeName = "Individual";

        ColabProperties properties = PropertiesOwner.getProperties();
        this.loginName  = properties.getLoginName();

        this.myColor        = Color.black;
        this.myFunctionLock = new FunctionLock (loginName, myColor, this);

	try {

	    if (null == _ploterModel) {

		bound = new Bound ();
		ploterModel = (iPloterModel) new PloterModel (bound);

	    } else
		ploterModel = _ploterModel;

	    ploterModel.addListener(this);

	    this.makeLayout();

	} catch (RemoteException e) {
	    //Never willl be throwed
	}

    }

    protected void makeLayout() throws RemoteException {

        this.setLayout(new BorderLayout());

        //MenuBar
        JMenuBar menuBar = new JMenuBar();
        this.createMenuBar(menuBar);

        //South Panel
        JPanel southPanel = new JPanel (new BorderLayout());
        southPanel.add (this.getEditingPanel(),       BorderLayout.NORTH);

        if (functionListVisible)
            southPanel.add (this.getFunctionTablePanel(), BorderLayout.SOUTH);

        this.add (menuBar,               BorderLayout.NORTH);
        this.add (this.getGraphPanel(),  BorderLayout.CENTER);
        this.add (southPanel,            BorderLayout.SOUTH);

    }


    public void createMenuBar (JMenuBar _menuBar) {

        _menuBar.setAlignmentX(JComponent.LEFT_ALIGNMENT);
        _menuBar.setAlignmentY(JComponent.TOP_ALIGNMENT);

        //------------
        // File
        JMenu menu = (JMenu) _menuBar.add(new JMenu("Arquivo"));
        menu.setMnemonic('a');

        //JMenuItem item = (JMenuItem) menu.add(new JMenuItem("Novo"));
        //item.setHorizontalTextPosition(JButton.RIGHT);
        //item.addActionListener (this);
        //item.setMnemonic('n');

//          JMenuItem item = (JMenuItem) menu.add(new JMenuItem("Novo")); //, new ImageIcon("images/open.gif")));
//          item.setActionCommand ("newModel");
//          item.setHorizontalTextPosition(JButton.RIGHT);
//          item.addActionListener (this);
//          item.setMnemonic('n');

        JMenuItem item = (JMenuItem) menu.add(new JMenuItem("Abrir", new ImageIcon("images/open.gif")));
        item.setIcon (new ImageIcon("images/open.gif"));
        item.setActionCommand ("open");
        item.setHorizontalTextPosition(JButton.RIGHT);
        item.addActionListener (this);
        item.setMnemonic('a');

        item = (JMenuItem) menu.add(new JMenuItem("Salvar", new ImageIcon("images/save.gif")));
        item.setActionCommand ("save");
        item.setHorizontalTextPosition(JButton.RIGHT);
        item.addActionListener (this);
        item.setMnemonic('s');

        item = (JMenuItem) menu.add(new JMenuItem("Salvar como imagem"));
        item.setActionCommand ("save as");
        item.setHorizontalTextPosition(JButton.RIGHT);
        item.addActionListener (this);
        item.setMnemonic('i');

        menu.addSeparator();

        item = (JMenuItem) menu.add(new JMenuItem("Publicar"));
        item.setActionCommand ("publish");
        item.setHorizontalTextPosition(JButton.RIGHT);
        item.addActionListener (this);
        item.setMnemonic('p');

        item = (JMenuItem) menu.add(new JMenuItem("Gerar pre-relatorio"));
        item.setActionCommand ("report");
        item.setHorizontalTextPosition(JButton.RIGHT);
        item.addActionListener (this);
        item.setMnemonic('e');

//      menu.addSeparator();

//      item = (JMenuItem) menu.add(new JMenuItem("Sair"));
//      item.setHorizontalTextPosition(JButton.RIGHT);
//      item.addActionListener (this);
//      item.setMnemonic('r');

        this.addMenuItem(_menuBar);

        // panels
        menu = (JMenu) _menuBar.add(new JMenu("Paineis"));
        menu.setMnemonic('p');

        String str = "Esconde o painel Lista de Funcoes";;
        if (! functionListVisible)
            str = "Mostra o painel Lista de Funcoes";
        this.showHideItem = (JMenuItem) menu.add(new JMenuItem(str));
        this.showHideItem.setActionCommand ("show_hide_function_linst");
        this.showHideItem.setHorizontalTextPosition(JButton.RIGHT);
        this.showHideItem.addActionListener (this);
        this.showHideItem.setMnemonic('L');

        // Help
        menu = (JMenu) _menuBar.add(new JMenu("Ajuda"));
        menu.setMnemonic('j');

        item = (JMenuItem) menu.add(new JHelpMenuItem("Tutorial", "http://www.dca.fee.unicamp.br/~fisch/colab/ploter.html"));
        item.setHorizontalTextPosition(JButton.RIGHT);
        item.addActionListener (this);
        item.setMnemonic('T');

	menu.addSeparator();

        item = (JMenuItem) menu.add(new JHelpMenuItem("Atividade 1", "http://www.dca.fee.unicamp.br/~fisch/colab/projeto/node1.html"));
        item.setHorizontalTextPosition(JButton.RIGHT);
        item.addActionListener (this);
        item.setMnemonic('1');

        item = (JMenuItem) menu.add(new JHelpMenuItem("Atividade 2", "http://www.dca.fee.unicamp.br/~fisch/colab/projeto/node2.html"));
        item.setHorizontalTextPosition(JButton.RIGHT);
        item.addActionListener (this);
        item.setMnemonic('2');

        item = (JMenuItem) menu.add(new JHelpMenuItem("Atividade 3", "http://www.dca.fee.unicamp.br/~fisch/colab/projeto/node3.html"));
        item.setHorizontalTextPosition(JButton.RIGHT);
        item.addActionListener (this);
        item.setMnemonic('3');

  	menu.addSeparator();

	item = (JMenuItem) menu.add(new JHelpMenuItem("Funcoes Validas", "http://www.dca.fee.unicamp.br/~fisch/colab/funcoes_validas.html"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('F');

	item = (JMenuItem) menu.add(new JHelpMenuItem("Publicando uma Funcao", "http://www.dca.fee.unicamp.br/~fisch/colab/ploter_parte4.html"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('P');

	item = (JMenuItem) menu.add(new JHelpMenuItem("Gerando o Pre-Relatorio", "http://www.dca.fee.unicamp.br/~fisch/colab/ploter_parte5.html"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('G');

  	menu.addSeparator();

	item = (JMenuItem) menu.add(new JHelpMenuItem("Definicao", "http://www.dca.fee.unicamp.br/~fisch/colab/func_def.html"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('D');

	item = (JMenuItem) menu.add(new JHelpMenuItem("Operacoes com funcoes", "http://www.dca.fee.unicamp.br/~fisch/colab/func_oper.html#operacoes"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('O');

	item = (JMenuItem) menu.add(new JHelpMenuItem("Funcoes Fundamentais", "http://www.dca.fee.unicamp.br/~fisch/colab/func_fund.html#fundamentais"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('u');

	item = (JMenuItem) menu.add(new JHelpMenuItem("Funcoes Pares e Impares", "http://www.dca.fee.unicamp.br/~fisch/colab/func_par.html#par"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('p');

	item = (JMenuItem) menu.add(new JHelpMenuItem("Maximos", "http://www.dca.fee.unicamp.br/~fisch/colab/func_maximo.html#maximo"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('x');

	item = (JMenuItem) menu.add(new JHelpMenuItem("Minimos", "http://www.dca.fee.unicamp.br/~fisch/colab/func_minimo.html#minimo"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('n');

	item = (JMenuItem) menu.add(new JHelpMenuItem("Periodo", "http://www.dca.fee.unicamp.br/~fisch/colab/func_period.html#periodo"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('e');

	item = (JMenuItem) menu.add(new JHelpMenuItem("Amplitude", "http://www.dca.fee.unicamp.br/~fisch/colab/func_period.html#amplitude"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('A');

	item = (JMenuItem) menu.add(new JHelpMenuItem("Simetria", "http://www.dca.fee.unicamp.br/~fisch/colab/func_simetria.html#simetria"));
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('S');

    }


    protected void addMenuItem (JMenuBar _menuBar) {
    }


    protected JPanel getGraphPanel() throws RemoteException {
	if (graphPanel == null) {
	    this.graphPanel = new JPanel (new BorderLayout());

	    graphPanel.setBorder (BorderFactory.createTitledBorder(
                                        BorderFactory.createEtchedBorder(),
                                        "Janela de Visualizacao",
                                        TitledBorder.CENTER,
                                        TitledBorder.TOP));


	    // graph

	    this.graph = new Graph(ploterModel, infoJText, steps);
	    graphPanel.add (graph,            BorderLayout.CENTER);

	    graphPanel.add (getConfigPanel(), BorderLayout.NORTH);
	}

        return graphPanel;

    }


    protected JPanel getFunctionTablePanel() throws RemoteException {

        if (this.tablePanel == null) {
            JToolBar toolBar = new JToolBar();
            toolBar.setFloatable(false);
            toolBar.setBorder(BorderFactory.createEmptyBorder (6, 6, 6, 6));

            JButton button = new JButton ("Nova Funcao");
            button.setActionCommand ("new");
            button.addActionListener(this);
            button.setToolTipText("Acrescenta uma nova funcao na lista de funcoes.");
            toolBar.add(button);

            button = new JButton ("Editar Funcao");
            button.setActionCommand ("edit");
            button.addActionListener(this);
            button.setToolTipText("Permite editar a funcao selecionada.");
            toolBar.add(button);

            button = new JButton ("Remover Funcao");
            button.setActionCommand ("remove");
            button.addActionListener(this);
            button.setToolTipText("Remove a funcao selecionada.");
            toolBar.add(button);

            button = this.getToOtherButton();
            button.setActionCommand ("toOtherModel");
            button.addActionListener(this);
            toolBar.add(button);

            tableModel = new FunctionTableModel (this.ploterModel, this.mode);
            functionTable = new JTable (tableModel);
            functionTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            functionTable.setPreferredScrollableViewportSize (new Dimension (66, 66));
            this.setTableRenderers (functionTable);

            this.tablePanel = new JPanel (new BorderLayout());
            //this.tablePanel.setBorder (defaultBorder);
            this.tablePanel.setBorder (BorderFactory.createTitledBorder(
                                            BorderFactory.createEtchedBorder(),
                                            "Lista de Funcoes",
                                            TitledBorder.CENTER,
                                            TitledBorder.TOP));

            this.tablePanel.add (toolBar,                         BorderLayout.NORTH);
            this.tablePanel.add (new JScrollPane (functionTable), BorderLayout.SOUTH);
        }

        return this.tablePanel;

    }

    protected JButton getToOtherButton () {

        JButton button = new JButton ("-> Chat");
        button.setToolTipText ("Envia a funcao selecionada para o ploter do Chat.");
        return button;

    };

    protected void setTableRenderers (JTable _table) {

        _table.setDefaultRenderer (Color.class, new ColorRenderer(true));
        setUpColorEditor (_table);

    }

    protected JPanel getEditingPanel () {

	if (this.editingPanel == null) {

	    this.editingPanel = new JPanel();

	    this.editingPanel.setBorder (BorderFactory.createTitledBorder(
                                        BorderFactory.createEtchedBorder(),
                                        "Edicao de Funcao",
                                        TitledBorder.CENTER,
                                        TitledBorder.TOP));

	    this.editingPanel.setLayout(new BorderLayout());
	    this.editingPanel.add (this.getFunctionPanel(), BorderLayout.NORTH);
	    this.editingPanel.add (this.getInfoPanel(),     BorderLayout.SOUTH);
	}

        return editingPanel;

    }

    protected JPanel getConfigPanel() {

        // Config Buton
        JPanel configPanel = new JPanel(new GridLayout(-1,1,0,0));
        JButton config = new JButton ("Configurar Janela de Visualizacao");
        config.setActionCommand ("config");
        config.addActionListener(this);
        config.setToolTipText ("Permite configurar os intervalos de x e y que aparecerao na janela do ploter.");
        configPanel.add(config);

        return configPanel;

    }

    protected JPanel getFunctionPanel() {

        if (null == functionPanel) {
            this.functionPanel = new JPanel (new BorderLayout(10,10));
            functionPanel.setBorder (BorderFactory.createEmptyBorder(6, 0, 6, 0));

            // Text fields
            JPanel columnPanel = new JPanel (new GridLayout (0,1));

            functionJText = new JTextField(20);
            functionJText.setActionCommand ("functionJText");
            functionJText.setEditable (false);
            functionJText.addActionListener(this);
            functionJText.addFocusListener(this);
            columnPanel.add (functionJText);

            functionNameJText = new JTextField(20);
            functionNameJText.setActionCommand ("functionNameJText");
            functionNameJText.setEditable (false);
            functionNameJText.addActionListener(this);
            functionNameJText.addFocusListener(this);
            columnPanel.add (functionNameJText);

            xMinJText = new JTextField();
            xMinJText.setActionCommand ("xMin");
            xMinJText.setEditable (false);
            xMinJText.addActionListener (this);
            xMinJText.addFocusListener(this);
            xMaxJText = new JTextField();
            xMaxJText.setActionCommand ("xMin");
            xMaxJText.setEditable (false);
            xMaxJText.addActionListener (this);
            xMaxJText.addFocusListener(this);

            // dominio Layout
            JPanel dominion = new JPanel();
            GridBagLayout gridbag = new GridBagLayout();
            dominion.setLayout (gridbag);
            JLabel label = new JLabel("[");
            GridBagConstraints cLabel = new GridBagConstraints();
            cLabel.gridx = 0;
            cLabel.gridy = 0;
            gridbag.setConstraints (label, cLabel);
            dominion.add (label);
            GridBagConstraints cText = new GridBagConstraints();
            cText.gridx   = 1;
            cText.gridy   = 0;
            cText.fill    = GridBagConstraints.HORIZONTAL;
            cText.weightx = 0.5;
            cText.ipadx   = 2;
            gridbag.setConstraints (xMinJText, cText);
            dominion.add (xMinJText);
            label = new JLabel(",");
            cLabel.gridx = 2;
            gridbag.setConstraints (label, cLabel);
            dominion.add (label);
            cText.gridx = 3;
            gridbag.setConstraints (xMaxJText, cText);
            dominion.add (xMaxJText);
            label = new JLabel("]");
            cLabel.gridx = 4;
            gridbag.setConstraints (label, cLabel);
            dominion.add (label);

            columnPanel.add (dominion);

            functionPanel.add (columnPanel, BorderLayout.CENTER);

            // Label Fields
            columnPanel = new JPanel (new GridLayout (0,1));
            columnPanel.add(new JLabel("Funcao: "));
            columnPanel.add(new JLabel("Nome: "));
            columnPanel.add(new JLabel("Dominio: "));
            functionPanel.add (columnPanel, BorderLayout.WEST);

            // Buttons
            functionPanel.add(this.getFunctionButtonsPanel(), BorderLayout.EAST);
        }

        return this.functionPanel;

    }

    protected JPanel getFunctionButtonsPanel() {
        JPanel columnPanel = new JPanel (new GridLayout (0,1));
        JButton go = new JButton("Redesenha");
        go.setActionCommand ("go");
        go.addActionListener(this);
        go.setToolTipText ("Redesenha as funcoes.");
        columnPanel.add (go);

        return columnPanel;
    }

    protected JPanel getInfoPanel() {

	if (infoPanel == null) {

	    // Info Panel
	    this.infoPanel = new JPanel();
	    this.infoPanel.setLayout(new BorderLayout());
	    this.infoPanel.add(new JLabel("Info:"), BorderLayout.WEST);
	    this.infoJText = new JTextArea();
	    this.infoJText.setRows(3);
	    this.infoJText.setEditable(false);
	    this.infoJText.setLineWrap(true);
	    this.infoJText.setWrapStyleWord(true);
	    this.infoPanel.add(new JScrollPane (infoJText), BorderLayout.CENTER);

	}

        return infoPanel;

    }

    protected void performEditFunction() {

        this.unlockActualFunction();

        int selection = functionTable.getSelectedRow();

        if (selection != -1) {

            editingFunction = tableModel.getFunction (selection);

            this.editFunction (editingFunction);

        } else
	    StandardDialogs.informDialog (null, "Aviso", "Voce deve selecionar a funcao que deseja editar (alterar).\nPara isso clique na linha correspodente a ela na lista de funcoes e depois clique no botao [Editar Funcao].");

    }


    protected boolean isFunctionEdited() {

	//System.out.println ("IsFunctionEdited ??");
	if (null == editingFunction)
	    return false;

	if (! isEditing)
	    return false;

        if ( editingFunction.getFunctionString().equals (functionJText.getText()) &&
             editingFunction.getName().equals (functionNameJText.getText()) &&
             editingFunction.getXMin().equals (xMinJText.getText()) &&
             editingFunction.getXMax().equals (xMaxJText.getText()) )
            return false;

        return true;

    }


    protected void publishChangeInFunction () {

	//System.out.println ("publishChangeInFunction");
        try {

            this.editingFunction.setName(functionNameJText.getText());
            this.editingFunction.setFunctionString (functionJText.getText());
            this.editingFunction.setXMin (xMinJText.getText());
            this.editingFunction.setXMax (xMaxJText.getText());

            ploterModel.changeFunction (editingFunction.getId(), editingFunction, myFunctionLock);

        } catch (RemoteException e){

	    StandardDialogs.errorDialog (null, "ERRO", "Problemas na comunicacao com o servidor.\nTente novamente, se o problema persistir saia e entre de novo na sala de chat.");
	    e.printStackTrace();

        } catch (FunctionLockException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel modificar a funcao: funcao travado por um usuario!");

        } catch (FunctionNotFoundException e) {

	    this.isEditing = false;

	    functionJText.setText ("");
	    functionJText.setEditable (false);
	    functionNameJText.setText ("");
	    functionNameJText.setEditable (false);
	    xMinJText.setEditable (false);
	    xMinJText.setText ("");
	    xMaxJText.setEditable (false);
	    xMaxJText.setText ("");

	    StandardDialogs.errorDialog (null, "ERRO", "Ha um problema de inconsistencia, esta funcao nao aparece no modelo de funcoes.\nClique no botao [Redesenha] para corrigir esta falha.");

	}
    }


    public void actionPerformed (ActionEvent evt) {

        if (evt.getActionCommand().equals("go")) {

	    try {

		graph.resetModel();
		tableModel.resetModel();

	    } catch (RemoteException e) {

		StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel refazer os grafico, problemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente no Chat.");

	    }

        } else if (evt.getActionCommand().equals("functionJText") || evt.getActionCommand().equals("functionNameJText") ||
                   evt.getActionCommand().equals("xMin")          || evt.getActionCommand().equals("xMax")) {

            //System.out.println ("alterei a funcao: " + editingFunction.getName());
            this.publishChangeInFunction();

        } else if (evt.getActionCommand().equals("show_hide_function_linst")) {

            this.removeAll();
            functionListVisible = ! functionListVisible;

	    try {

		this.makeLayout();

	    } catch (RemoteException e) {

		StandardDialogs.errorDialog (null, "ERRO", "Problemas na comunicacao com o servidor. Tente novamente, se o problema persistir saia e entre de novo na sala de chat.");

	    }

            this.validate();
            this.repaint();

        } else if (evt.getActionCommand().equals("open")) {

            //System.out.println ("open");

	    if (this.verifySaved()) {

		PloterModel newModel = PersistenceDialog.openPloterModel (this);
		if (null != newModel) {

		    //System.out.println ("Vou setar o modelo");

		    try {

			//if (ploterModel.verifyLocks()) {
			//StandardDialogs.questionYesNoCancelDialog (null, "O Modelo nao esta salvo", "Voce ira perder as ultimas modificacoes feitas. Voce quer salvar o seu modelo de funcoes antes de abrir um outro ?");
			this.unlockActualFunction();
			ploterModel.setModel(newModel);
			this.saved = true;
			//}

		    } catch (RemoteException e){

			StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel refazer os grafico, problemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente no Chat.");

		    } catch (FunctionLockException e) {

			StandardDialogs.errorDialog (null, "ERRO", "Nao posso abrir outro arquivo. O modelo de funcoes que esta aberto agora tem funcoes travadas por algum usuario!");

		    }
		}

            } //else
                //System.out.println ("Modelo == null");

        } else if (evt.getActionCommand().equals("newModel")) {

            //System.out.println ("new");

        } else if (evt.getActionCommand().equals("save")) {

            //System.out.println ("save");

	    this.saveModel();

        } else if (evt.getActionCommand().equals("save as")) {

            //System.out.println ("save as");

	    this.savePicture();

        } else if (evt.getActionCommand().equals("publish")) {

            //System.out.println ("publish");

            try {

                PloterModel clone = ploterModel.cloneModel();
                boolean saved = PersistenceDialog.publishPloterModel (clone);

		if (saved)
		    StandardDialogs.informDialog (null, "Funcao Publicada", "A sua funcao foi publicada com exito.\nCaso queira verificar, ela se encontra na pasta \"Publicados\", com o nome do seu grupo");

            } catch (RemoteException e){

		StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel refazer os grafico, problemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente no Chat.");

            }

        } else if (evt.getActionCommand().equals("config")) {

            // open a JOptionPane asking for x and y range and steps
            JPanel line1 = new JPanel(new FlowLayout(FlowLayout.LEFT));

            try {

                int resp = JOptionPane.showConfirmDialog (this, getConfigLines(), "Configuracao", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE);

                if (resp == JOptionPane.OK_OPTION) {
                    double tx1 = new Double(x1JText.getText()).doubleValue();
                    double tx2 = new Double(x2JText.getText()).doubleValue();
                    double ty1 = new Double(y1JText.getText()).doubleValue();
                    double ty2 = new Double(y2JText.getText()).doubleValue();

                    if (tx1 != bound.x1 || ty1 != bound.y1 || tx2 != bound.x2 || ty2 != bound.y2)
                        ploterModel.setBound (new Bound (tx1, ty1, tx2, ty2));

                    int tsteps = new  Integer(stepTxt.getText()).intValue();

                    if (steps != tsteps) {
                        steps = tsteps;
                        graph.setSteps (steps);
                    }

                    steps = new Integer(stepTxt.getText()).intValue();

                }

            } catch (RemoteException e) {

		StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel refazer os grafico, problemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente no Chat.");

            }

        } else if (evt.getActionCommand().equals ("remove")) {

            int selection = functionTable.getSelectedRow();

            if (selection != -1) {

                Function function = tableModel.getFunction (selection);

                try {

                    if (isEditing)
                        if (editingFunction.getId() == function.getId())
                            this.unlockActualFunction();

                    ploterModel.removeFunction (function.getId());


                } catch (RemoteException e) {

		    StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel refazer os grafico, problemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente no Chat.");

                } catch (FunctionLockException e) {

		    StandardDialogs.errorDialog (null, "Erro para remover funcao", "Nao foi possivel remover a funcao pois ela esta travada por algum usuario!");

                }
            } else
		StandardDialogs.informDialog (null, "Aviso", "Voce deve selecionar a funcao que deseja remover.\nPara isso clique na linha correspodente a ela na lista de funcoes e depois clique no botao [Editar Funcao].");

                this.unselectFunction();

        } else if (evt.getActionCommand().equals ("toOtherModel")) {

            int selection = functionTable.getSelectedRow();
            //System.out.println ("Movendo: " + selection);

            if (selection != -1) {

                Function function = tableModel.getFunction (selection);

                if (null != function) {
		    Function f = new Function (function.getName(), function.getFunctionString());
		    f.setColor (function.getColor());
		    f.setXMin (function.getXMin());
		    f.setXMax (function.getXMax());

                    this.moveFunctionToOtherPloter(f);
		    this.unselectFunction();
		}

            } else {
		String button = "Chat";
		if (this.mode == PloterPanel.SHARED)
		    button = "Individual";

		StandardDialogs.informDialog (null, "Aviso", "Voce deve selecionar a funcao que deseja enviar para o outro ploter.\nPara isso clique na linha correspodente a ela na lista de funcoes e depois clique no botao [-> " + button + "].");
	    }

        } else if (evt.getActionCommand().equals ("edit")) {

            this.performEditFunction();
            this.unselectFunction();

        } else if (evt.getActionCommand().equals ("report")) {

            this.makePreReport();

        } else if (evt.getActionCommand().equals ("new")) {

            this.unlockActualFunction();

            //System.out.println ("Nova: ");

            try {
                String name = ploterModel.getValidName();

                editingFunction = new Function (name, "");
                editingFunction.setFunctionLock (myFunctionLock);

                editingFunction.setId ( ploterModel.addFunction (editingFunction) );

                this.editFunction (editingFunction);

            } catch (RemoteException e) {

		StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel refazer os grafico, problemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente no Chat.");

            }

        } else if (evt.getActionCommand().equals ("unlock")) {

            //System.out.println ("Liberando a funcao");
            this.unlockActualFunction();

        }

    }


    public void focusGained(FocusEvent e) {
        // Invoked when a component gains the keyboard focus.
    }

    public void focusLost(FocusEvent e){
        // Invoked when a component loses the keyboard focus.

        if (isFunctionEdited())
            this.publishChangeInFunction();

    }

    private JLabel Component2Label(JComponent tSource, int w, int h) {

	tSource.setSize (w,h);
	tSource.setPreferredSize (new Dimension(w,h));
	BufferedImage bImage = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
	//Image tImage = graph.createImage(w, h);//Create a !!BLANK!! Image
	//System.out.println ("aqui 1");
	Graphics tGraphic = bImage.getGraphics();//Get the Graphics associated with the new Image
	//System.out.println ("aqui 2");
	tGraphic.setClip(new Rectangle(tSource.getSize()));//This is the important line!! I don't know why this isn't already set by CreateImage, maybe its a bug??
	tSource.paint(tGraphic);//Paint the Component to the Image.

	return new JLabel (new ImageIcon(bImage));//Done!
    }

    private ImageIcon Component2Icon(JComponent tSource, int w, int h) {

	tSource.setSize (w,h);
	tSource.setPreferredSize (new Dimension(w,h));
	Image tImage = graph.createImage(w, h);//Create a !!BLANK!! Image
	//System.out.println ("aqui a");
	Graphics tGraphic = tImage.getGraphics();//Get the Graphics associated with the new Image
	//System.out.println ("aqui b");
	tGraphic.setClip(new Rectangle(tSource.getSize()));//This is the important line!! I don't know why this isn't already set by CreateImage, maybe its a bug??
	tSource.paint(tGraphic);//Paint the Component to the Image.

	return new ImageIcon(tImage);//Done!
    }

    private void makePreReport() {

	int width = 300, height = 300;
        //System.out.println ("Gerando as figuras");

        try {


	    //System.out.println ("\nCriando a primeira figura com todas as funcoes");
            PloterModel cloneModel = ploterModel.cloneModel();

            Function[] functions = cloneModel.getFunctionList();

            Vector doc = new Vector();
            Object[] objects = new Object [(functions.length+1) * 2 + 1];

            try {

                doc.add("Grafico com todas as funcoes: \n");
                doc.add (this.Component2Icon(new Graph (cloneModel, null, width), width, height));

            } catch (IOException e) {

		StandardDialogs.errorDialog (null, "ERRO", "Problemas para gerar o pre-relatorio: " +  e.getMessage());

            }

            for (int i = 0; i < functions.length; i++) {

		//System.out.println ("Criando figura " + (i+2)+": "+ functions[i].getId() + ", " + functions[i].getFunctionString());

                PloterModel newModel = new PloterModel(cloneModel.getBound());
                newModel.addFunction (functions[i]);
                doc.add("\n\n\nFuncao " + functions[i].getName() + ": " + functions[i].getFunctionString() + "\n");

                try {

                    doc.add (this.Component2Icon(new Graph (newModel, null, width), width, height));
                    //System.out.println ("\t...ok");

                } catch (IOException e) {

		    StandardDialogs.errorDialog (null, "ERRO", "Problemas para gerar o pre-relatorio: " +  e.getMessage());

                }

            }

            doc.add("\n");
            ModulesOwner.getNotepad().insertComponents (doc.toArray(), width, height);
            ModulesOwner.getNotepadFrame().setVisible(true);

        } catch (RemoteException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel refazer os grafico, problemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente no Chat.");

        }
    }


    protected void moveFunctionToOtherPloter (Function _function) {

        // 1- Get the ChatPanel
	try {
	    ChatPanel chat = ModulesOwner.getChatPanel();

	    // 2- Verify if the user is inside a room
	    if (chat.isInARoom()) {

		// 3- Get the other ploter panel
		PloterPanel otherPloter = (PloterPanel) chat.getToolPanel();

		// 4- Send the function
		otherPloter.insertFunction (_function);

		StandardDialogs.informDialog (null, "Sucesso", "A funcao selecionada foi enviada com sucesso para o ploter colaborativo no chat.");

	    }

        } catch (IOException e){

	    StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel enviar a funcao para o Chat, problemas com o servidor de chat!");

	}
    }


    protected void insertFunction (Function _function) {

        try {

            _function.setFunctionLock (null);
            _function.setId ( ploterModel.addFunction (_function) );

        } catch (RemoteException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel refazer os grafico, problemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente no Chat.");

        }
    }


    protected void editFunction (Function _function) {

        functionJText.setText (editingFunction.getFunctionString());
        functionNameJText.setText (editingFunction.getName());
        xMinJText.setText (editingFunction.getXMin());
        xMaxJText.setText (editingFunction.getXMax());

        functionJText.setEditable (true);
        functionNameJText.setEditable (true);
        xMinJText.setEditable (true);
        xMaxJText.setEditable (true);

        if (this.mode == PloterPanel.SHARED)
            unlock.setEnabled (true);

        isEditing = true;
    }

    // return false if user cancel last action
    protected boolean verifySaved() {

	if (this.saved == false) {
	    int resp = StandardDialogs.questionYesNoCancelDialog (null, "O modelo de funcoes nao esta salvo", "Voce nao salvou o seu atual modelo de funcoes do ploter "+this.ploterTypeName+" e ira perder as ultimas modificacoes.\nGostaria de salvar antes de continuar?");

	    if (resp == StandardDialogs.CANCEL)
		return false;

	    if (resp == StandardDialogs.YES)
		this.saveModel();

	}

	return true;
    }

    // return false if user cancel last action
    protected boolean verifySaved(String _question) {

	if (this.saved == false) {
	    int resp = StandardDialogs.questionYesNoCancelDialog (null, "O modelo de funcoes nao esta salvo", "Voce nao salvou o seu atual modelo de funcoes do ploter "+this.ploterTypeName+" e ira perder as ultimas modificacoes.\n"+_question);

	    if (resp == StandardDialogs.CANCEL)
		return false;

	    if (resp == StandardDialogs.YES)
		this.saveModel();

	}

	return true;
    }

    protected void saveModel() {

	try {

	    PloterModel clone = ploterModel.cloneModel();
	    this.saved = PersistenceDialog.savePloterModel (this, clone);

	} catch (RemoteException e){

	    StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel salvar o modelo de funcoes.\nProblemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente na sala do Chat.");

	}
    }

    protected void savePicture() {



	int width = 300, height = 300;
        //System.out.println ("Gerando as figuras");

        try {

	    //System.out.println ("\nCriando a figura com todas as funcoes");
	    PloterModel cloneModel = ploterModel.cloneModel();
	    Function[] functions = cloneModel.getFunctionList();

	    ImageIcon icon = this.Component2Icon(new Graph (cloneModel, null, width), width, height);

	    //  this.saved = PersistenceDialog.savePloterModel (this, clone);

	} catch (RemoteException e){

	    StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel salvar uma figura das funcoes.\nProblemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente na sala do Chat.");

	}

    }


    public void exit() {

        this.unlockActualFunction();

	if (this.saved == false) {
	    int resp = StandardDialogs.questionDialog (null, "O modelo de funcoes nao esta salvo", "Voce nao salvou o seu atual modelo de funcoes e ira perder as ultimas modificacoes.\nGostaria de salvar em um arquivo ?");

	    if (resp == StandardDialogs.YES)
		this.saveModel();
	}

	graph.exit();
        tableModel.exit();

        try {

            ploterModel.removeListener(this);

        } catch (RemoteException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel refazer os grafico, problemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente no Chat.");

        }

    }

    protected void unlockActualFunction () {

        if (isEditing) {

            try {

                ploterModel.unlockFunction (myFunctionLock, editingFunction.getId());

                this.isEditing = false;

                functionJText.setText ("");
                functionJText.setEditable (false);
                functionNameJText.setText ("");
                functionNameJText.setEditable (false);
                xMinJText.setEditable (false);
                xMinJText.setText ("");
                xMaxJText.setEditable (false);
                xMaxJText.setText ("");

                if (this.mode == PloterPanel.SHARED)
                    unlock.setEnabled (false);


            } catch (RemoteException e) {

		StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel refazer os grafico, problemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente no Chat.");

            } catch (FunctionLockException e) {

		StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel destravar a funcao, tente o botao redesenha ou sair e entrar na sala. \n"+e.getMessage());

            }
        }
    }

//    protected int getSelected() {
//
//          int selection = functionTable.getSelectedRow();
//          return -1;
//
//    }

    protected void unselectFunction() {
        //System.out.println ("unselecting function");
        int sel = functionTable.getSelectedRow();
        if (sel != -1) {
            functionTable.getSelectionModel().removeSelectionInterval (sel, sel);
        }
    }


    protected Object[] getConfigLines() {
        try {

            if (null == x1JText) {
                x1JText = new JTextField(5);
                x2JText = new JTextField(5);
                y1JText = new JTextField(5);
                y2JText = new JTextField(5);
                stepTxt = new JTextField(5);
            }

            bound = ploterModel.getBound();
            x1JText.setText(String.valueOf(bound.x1));
            y1JText.setText(String.valueOf(bound.y1));
            x2JText.setText(String.valueOf(bound.x2));
            y2JText.setText(String.valueOf(bound.y2));

            JPanel line1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
            stepTxt.setText(String.valueOf(steps));
            line1.add(new JLabel("X:")); line1.add(x1JText);
            line1.add(new JLabel("->")); line1.add(x2JText);
            JPanel line2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
            line2.add(new JLabel("Y:")); line2.add(y1JText);
            line2.add(new JLabel("->")); line2.add(y2JText);
            line2.add(Box.createHorizontalStrut(10));
            line2.add(new JLabel("Passos:"));
            line2.add(stepTxt);

            String str = new String("Configuracao do Ploter");
            Object[] lines = {str, line1, line2};

            return lines;

        } catch (RemoteException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel refazer os grafico, problemas na comunicacao com o servidor do ploter.\nTente de novo, se o problema persistir saia e entre novamente no Chat.");

        }

        return null;
    }

    public void setMyColor (Color _myColor) {
        myColor = _myColor;
    }

    public void ploterChanged (PloterModelEvent _e) throws RemoteException {

        //System.out.println ("PloterPanel.ploterChanged");

        if (PloterModelEvent.BOUND_CHANGED == _e.getType()) {

	    this.saved = false;
            bound = _e.getBound();

        } else if (PloterModelEvent.FUNCTION_CHANGED == _e.getType()) {

            //System.out.println ("Estou aqui");

            //if (editingFunction == null)
            //System.out.println ("edicao nula");

	    this.saved = false;

            if (editingFunction != null)

                if (_e.getId().equals (editingFunction.getId()))

                    if ( ! _e.getFunction().getColor().equals (editingFunction.getColor()))

                        editingFunction.setColor (_e.getFunction().getColor());

        }
    }

    public void update (iEvent _event) throws RemoteException {

        //System.out.println ("PloterPanel.ploterChanged");

        PloterModelEvent e = (PloterModelEvent) _event;

	if (PloterModelEvent.BOUND_CHANGED == e.getType()) {

	    this.saved = false;
            bound = e.getBound();

        } else if (PloterModelEvent.FUNCTION_CHANGED == e.getType()) {

            //System.out.println ("Estou aqui");

            //if (editingFunction == null)
            //System.out.println ("edicao nula");

	    this.saved = false;

            if (editingFunction != null)

                if (e.getId().equals (editingFunction.getId()))

                    if ( ! e.getFunction().getColor().equals (editingFunction.getColor()))

                        editingFunction.setColor (e.getFunction().getColor());

        }
    }

    protected void changeColor (Color _color) {

        int selection = functionTable.getSelectedRow();

        if (selection != -1) {

            Function function = tableModel.getFunction (selection);
            function.setColor (_color);
            //System.out.println ("Vou mudar a cor da funcao " + function.getName() + " para " + _color.toString());

            try {

                ploterModel.changeColor (function.getId(), _color);
                //System.out.println ("\t ...ok");

            } catch (RemoteException e){
                //Never willl be throwed
            }
        }

    }


    protected void setUpColorEditor(JTable table) {
        //First, set up the button that brings up the dialog.
        final JButton button = new JButton("") {
            public void setText(String s) {
                //Button never shows text -- only color.
            }
        };
        button.setBackground(Color.white);
        button.setBorderPainted(false);
        button.setToolTipText ("Permite redefinir a cor da funcao.");
        button.setMargin(new Insets(0,0,0,0));

        //Now create an editor to encapsulate the button, and set it up as the editor for all Color cells.
        final ColorEditor colorEditor = new ColorEditor(button);
        table.setDefaultEditor(Color.class, colorEditor);

        //Set up the dialog that the button brings up.
        final JColorChooser colorChooser = new JColorChooser();
        //XXX: PENDING: add the following when setPreviewPanel
        //XXX: starts working.
        //JComponent preview = new ColorRenderer(false);
        //preview.setPreferredSize(new Dimension(50, 10));
        //colorChooser.setPreviewPanel(preview);
        ActionListener okListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                colorEditor.currentColor = colorChooser.getColor();
            }
        };
        final JDialog dialog = JColorChooser.createDialog(button,
                                        "Escolha a cor das suas funcoes:",
                                        true,
                                        colorChooser,
                                        okListener,
                                        null); //XXXDoublecheck this is OK

        //Here's the code that brings up the dialog.
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                button.setBackground(colorEditor.currentColor);
                colorChooser.setColor(colorEditor.currentColor);
                //Without the following line, the dialog comes up
                //in the middle of the screen.
                //dialog.setLocationRelativeTo(button);
                dialog.show();

                changeColor (colorEditor.currentColor);
            }
        });
    }


    class ColorRenderer extends JLabel
                        implements TableCellRenderer {
        Border unselectedBorder = null;
        Border selectedBorder = null;
        boolean isBordered = true;

        public ColorRenderer(boolean isBordered) {
            super();
            this.isBordered = isBordered;
            setOpaque(true); //MUST do this for background to show up.
        }

        public Component getTableCellRendererComponent(
                                JTable table, Object color,
                                boolean isSelected, boolean hasFocus,
                                int row, int column) {
            setBackground((Color)color);
            if (isBordered) {
                if (isSelected) {
                    if (selectedBorder == null) {
                        selectedBorder = BorderFactory.createMatteBorder(2,5,2,5,
                                                  table.getSelectionBackground());
                    }
                    setBorder(selectedBorder);
                } else {
                    if (unselectedBorder == null) {
                        unselectedBorder = BorderFactory.createMatteBorder(2,5,2,5,
                                                  table.getBackground());
                    }
                    setBorder(unselectedBorder);
                }
            }
            return this;
        }
    }

    class ColorEditor extends DefaultCellEditor {
        Color currentColor = null;

        public ColorEditor(JButton b) {
            super(new JCheckBox()); //Unfortunately, the constructor
                                    //expects a check box, combo box,
                                    //or text field.

            editorComponent = b;
            setClickCountToStart(1); //This is usually 1 or 2.

            //Must do this so that editing stops when appropriate.
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    fireEditingStopped();
                }
            });
        }

        protected void fireEditingStopped() {
            super.fireEditingStopped();
        }

        public Object getCellEditorValue() {
            return currentColor;
        }

        public Component getTableCellEditorComponent(JTable table,
                                                     Object value,
                                                     boolean isSelected,
                                                     int row,
                                                     int column) {
            //((JButton)editorComponent).setText(value.toString());

            currentColor = (Color)value;

            return editorComponent;
        }
    }

}
