package colab.client.tool.ploter;

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.io.*;
import java.util.*;
import java.rmi.*;
import java.rmi.server.*;
import colab.util.tool.*;
import colab.util.tool.ploter.*;
import colab.client.gui.*;
import colab.client.*;

public class SharedPloterPanel extends PloterPanel implements iPloterListener, Remote, ListSelectionListener  {

    int lastSelection = -1;

    public SharedPloterPanel (iPloterModel _model, Color _myColor) {

	this.mode           = PloterPanel.SHARED;
	this.ploterTypeName = "Colaborativo";

	ColabProperties properties = PropertiesOwner.getProperties();
	this.loginName  = properties.getLoginName();

	this.myColor        = _myColor;
	this.myFunctionLock = new FunctionLock (loginName, myColor, this);

	try {

	    //System.out.println ("SharedPloterPanel");

	    UnicastRemoteObject.exportObject ( (iPloterListener) this);

	    ploterModel = _model;
	    ploterModel.addListener (this);

	    this.makeLayout();


	} catch (RemoteException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "Problemas na comunicacao com o servidor. Tente novamente, se o problema persistir saia e entre de novo na sala de chat.");

	}
    }

    public SharedPloterPanel (iPloterModel _model) {

	this.mode           = PloterPanel.SHARED;
	this.ploterTypeName = "Colaborativo";

	ColabProperties properties = PropertiesOwner.getProperties();
	this.loginName  = properties.getLoginName();

	this.myColor = Color.black;
	this.chooseColor();

	this.myFunctionLock = new FunctionLock (loginName, myColor, this);

	try {

	    //System.out.println ("SharedPloterPanel");

	    UnicastRemoteObject.exportObject ( (iPloterListener) this);

	    ploterModel = _model;
	    ploterModel.addListener (this);

	    this.makeLayout();


	} catch (RemoteException e) {

	    StandardDialogs.errorDialog (null, "ERRO", "Problemas na comunicacao com o servidor. Tente novamente, se o problema persistir saia e entre de novo na sala de chat.");

	}
    }


    private void chooseColor() {

	boolean notOk;

	do {
            myColor = JColorChooser.showDialog(null, "Escolha a cor em que aparecero suas funes", Color.black);
	    notOk = myColor.getRed() > 200 && myColor.getBlue() < 103 && myColor.getGreen() < 103;
	} while (notOk);

	if (myFunctionLock != null) {
	    myFunctionLock.setColor (myColor);

	    if (isEditing)
		this.publishChangeInFunction();
	}
    }


    protected void setTableRenderers (JTable _table) {

	_table.setDefaultRenderer (JLabel.class, new LabelRenderer());

    }


    protected JButton getToOtherButton () {

	JButton button = new JButton ("-> Individual");
	button.setToolTipText ("Envia a funcao selecionada para o ploter individual.");
	return button;

    }


    protected void moveFunctionToOtherPloter (Function _function) {

	// 1- Get the PloterPanel (individual)
	PloterPanel otherPloter = (PloterPanel) ModulesOwner.getToolPanel(null);

	// 2- Send the function
	otherPloter.insertFunction (_function);

	StandardDialogs.informDialog (null, "Sucesso", "A funcao selecionada foi enviada com sucesso para o seu ploter individual.");

    }


    protected JPanel getFunctionButtonsPanel() {
	JPanel columnPanel = new JPanel (new GridLayout (0,1));
        JButton go = new JButton("Redesenha");
        go.setActionCommand ("go");
        go.addActionListener(this);
	go.setToolTipText ("Redesenha as funcoes.");
	columnPanel.add (go);
	unlock = new JButton("Destravar Funcao");
        unlock.setActionCommand ("unlock");
	unlock.setEnabled (false);
        unlock.addActionListener(this);
	unlock.setToolTipText ("Destrava esta funcao permitindo que outra pessoa a edite.");
	columnPanel.add (unlock);

	return columnPanel;
    }


    protected void performEditFunction() {

	//System.out.println ("edit");

	this.unlockActualFunction();

	int selection = functionTable.getSelectedRow();
	//System.out.println ("Editando: " + selection);

	if (selection != -1) {
	    try {
		editingFunction = tableModel.getFunction (selection);

		if (ploterModel.lockFunction (editingFunction.getId(), myFunctionLock)) {
		    this.editFunction (editingFunction);
		    editingFunction.setFunctionLock (myFunctionLock);
		}

	    } catch (RemoteException e) {

		StandardDialogs.errorDialog (null, "ERRO", "Problemas na comunicacao com o servidor. Tente novamente, se o problema persistir saia e entre de novo na sala de chat.");


	    } catch (FunctionLockException e) {

		StandardDialogs.errorDialog (null, "ERRO", "Nao foi possivel conseguir a funcao para edicao: outro usuario ja esta editando-a!");

	    } catch (FunctionNotFoundException e) {

		StandardDialogs.errorDialog (null, "ERRO", "Esta funcao nao existe no servidor, ocorreu um erro de consistencia.\nClique no botao [Redesenha] para atualizar a sua lista de funcoes!");

	    }
	}  else
	    StandardDialogs.informDialog (null, "Aviso", "Voce deve selecionar a funcao que deseja editar (alterar).\nPara isso clique na linha correspodente a ela na lista de funcoes e depois clique no botao [Editar Funcao].");

    }


    public void actionPerformed (ActionEvent evt) {

	if (evt.getActionCommand().equals("change color")) {
	    this.chooseColor();
	} else
	    super.actionPerformed (evt);
    }


    protected void addMenuItem (JMenuBar _menuBar){

	JMenu menu = (JMenu) _menuBar.add(new JMenu("Cor"));
	menu.setMnemonic('c');

	JMenuItem item = (JMenuItem) menu.add(new JMenuItem("Trocar de Cor"));
	item.setActionCommand ("change color");
	item.setHorizontalTextPosition(JButton.RIGHT);
	item.addActionListener (this);
	item.setMnemonic('t');

    }


    public void ploterChanged (PloterModelEvent _e) throws RemoteException {
	boolean update = false;

	if (PloterModelEvent.BOUND_CHANGED == _e.getType()) {

	    this.saved = false;
	    bound = _e.getBound();

	} else if (PloterModelEvent.FUNCTION_CHANGED == _e.getType()) {

	    this.saved = false;
	    //
	}
    }


    public void update (iEvent _e) throws RemoteException {

        PloterModelEvent e = (PloterModelEvent) _e;
	boolean update = false;

	if (PloterModelEvent.BOUND_CHANGED == e.getType()) {

	    this.saved = false;
	    bound = e.getBound();

	} else if (PloterModelEvent.FUNCTION_CHANGED == e.getType()) {

	    this.saved = false;
	    //
	}
    }


    public void valueChanged (ListSelectionEvent evt){

	int selection = functionTable.getSelectedRow();

	if ( (-1 == selection) && (lastSelection != -1))
	    System.out.println ("Retira destaque");
	else if ( (-1 != selection) && (lastSelection != selection) )
	    System.out.println ("Muda Selecao");

    }


    class LabelRenderer extends JTextField implements TableCellRenderer {

        public LabelRenderer() {
            super();

            setOpaque(true); //MUST do this for background to show up.
        }


        public Component getTableCellRendererComponent( JTable table, Object _label,boolean isSelected, boolean hasFocus,int row, int column) {
	    JLabel label = (JLabel) _label;

	    if (isSelected)
		setBackground (table.getSelectionBackground());
            return label;
        }
    }

}
