/*---------------------------------------------------
|
|  br.unicamp.fee.dca.colab.server.chat.Room
|  by Paulo Henrique Fisch de Brito
|  Implements a chat room that waits for subscribers
|  and messages and forwards the messages to others
|  users.
|
----------------------------------------------------*/

package colab.server.chat;

import java.rmi.*;
import java.rmi.server.*;
import java.util.*;
import colab.util.chat.*;
import colab.util.*;
import colab.server.*;
import colab.server.tool.*;
import colab.util.tool.*;

public class Room extends UnicastRemoteObject
                  implements iRoom, Runnable
{
    private String      name;
    private ChatServer  chatServer;
    private Vector      allMsgs;
    private boolean     ServerGoingDown = false;
    private Map         allUsers; //and groups

    //***********************************
    private iToolModel toolModel;
    private ServerToolFactory toolFactory = ToolFactoryOwner.getFactory();
    //***********************************

    public Room(String _name, ChatServer _chat) throws RemoteException {
        super();
        name       = new String (_name);
        chatServer = _chat;
        allMsgs    = new Vector();

        // create user list and insert the group allGroups
        allUsers = Collections.synchronizedMap(new HashMap());
	everybody = new ChatGroup (ChatServer.ALL_USERS_STRING);
        allUsers.put(ChatServer.ALL_USERS_STRING, everybody);
        //allGroups = new ChatGroup (ChatServer.ALL_USERS_STRING);
        //allUsers.put(ChatServer.ALL_USERS_STRING, allGroups);
    }


    protected void exit() {
        //Verify if there are users

        ChatMsg msg = new ChatMsg ("SYSTEM", ChatServer.ALL_USERS_STRING, "O servidor esta saindo do ar", false);
        try {
            this.send (msg);
        } catch (Exception e) {
        }

        ServerGoingDown = true;
    } //exit


    private void putUser (String _name, absUser _usr) {
        // accept the user

        //System.out.println ("putUser (" + _name+")");

        this.event (ChatServer.USER_IN, _name);

        allUsers.put (_name, _usr);
	everybody.insertUser (_usr);
    }


    private absUser removeUser (String _name) {
	// retira o usuario da sala
        //System.out.println ("removeUser (" + _name+")");

	everybody.removeUser (_name);
	absUser us = (absUser) allUsers.remove (_name);
	if (us != null)
	    this.event (ChatServer.USER_OUT, us.getName());

	return us;
    }


    public void in (String _login, iChatClient _chatClient) throws RemoteException, ServerIsGoingDownException {

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

	//System.out.println ("in (" + _login + ")");

	//Primeiro verifica se ja ha este cliente na lista de clientes
	absUser cUser = (absUser) allUsers.get (_login);
	if (cUser != null) {
	    System.out.println ("Ja existe este usuario, vou retira-lo antes de inseri-lo novamente.");
	    this.removeUser (_login);
	}

        //Inclui este cliente na lista de usuarios
        // 1 pede informacoes do usuario para o servidor do chat
        ChatUser cu = chatServer.getUser(_login);
        cu.setAddress (_chatClient);

	if (cu != null) {
	    //System.out.println (cu.getName() + " entrou em " + name);

	    // insere o usuario na lista de usuarios
	    this.putUser (_login, cu);

	    //System.out.println ("Inscrevi cliente: " + _login);
	} else
	    {}//System.out.println ("Usuario desconhecido pelo ChatServer: "+_login);
    }


//      private ChatGroup getGroup (String _group) {
//          // retorna o grupo pedido, se nao houver cria um com este nome
//          //System.out.println ("getGroup(" + _group+")");

//          ChatGroup cg;
//          if ( allUsers.containsKey(_group) ) {
//              // retorna o grupo pedido
//              cg = (ChatGroup) allUsers.get (_group);
//          } else {
//              //cria o grupo pedido e insere no grupo todos
//              cg = new ChatGroup (_group);
//  	    allUsers.put (_group, cg);
//              allGroups.insertUser (cg);
//          }
//          return cg;
//      }


    public void out (String _login) throws RemoteException, ServerIsGoingDownException {
        //Exclui este cliente da lista de clientes

//          if (ServerGoingDown)
//              throw new ServerIsGoingDownException();

	//System.out.println ("out ("+_login+")");

	// 1 remove este usuario da lista de usuarios
	ChatUser cu = (ChatUser) this.removeUser (_login);
	//allGroups.removeUser (_login);

	//System.out.println ("\t1");
	// remove este usuario do grupo ao qual ele pertence
//  	ChatGroup cg = (ChatGroup) allUsers.get (cu.getGroup());
//  	System.out.println ("\t2");
//  	if (null != cg)
//  	    cg.removeUser(_login);

//  	System.out.println ("\t2");
//  	// verifica se o grupo ficou vazio e o retira tambem
//  	if (cg.isEmpty()) {
//              this.removeUser (cu.getGroup());
//              allGroups.removeUser (cu.getGroup());
//  	}

	//System.out.println ("Removi cliente: " + _login);
    }


    public void send (ChatMsg _msg) throws RemoteException, ServerIsGoingDownException {
        // Armazena a mensagem na fila
	//System.out.println ("send()");

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        allMsgs.add (_msg);
    }

    public void run () {
        // Look for messages and send
        while (true) {
            if (allMsgs.isEmpty())
                try {
                    Thread.sleep(1500);
                } catch (InterruptedException e){
                }
            else {
                this.realsend ( (ChatMsg) allMsgs.remove(0));
            }
        }
    }


    private void realsend (ChatMsg _msg) {
        // send message to destiny

	//System.out.println ("realSend()");

        absUser dest;

	// pega o usuario destino
	if (_msg.getPrivate())
	    dest = (absUser) allUsers.get (_msg.getDestiny());
	else
	    dest = everybody;

        if (dest != null) {
            // pega a lista de enderecos
            Vector v = new Vector();
	    int size = dest.getUserList(v);

	    if  (_msg.getPrivate()) {
		//need send the message to the sender of message
		dest = (absUser) allUsers.get (_msg.getSender());
		size = size + dest.getUserList (v);
	    }

            for (int i = 0; i<size; i++) {

		ChatUser user  = (ChatUser) v.elementAt(i);
                iChatClient cl = user.getAddress();

                if ( cl != null) {

		    try {

			cl.receive (_msg);
			user.resetRefuses();

		    } catch (RemoteException re) {

			user.incRefuses();
			if (user.getRefuses() >= 3) {
			    // there is no response from user, i will log out (s)he
			    try {
				this.out (user.getName());
			    } catch (Exception e) {
			    }
			}

			ErrorLog.logError ("colab.server.chat.Room.realSend()", "Nao foi possivel enviar mensagem de chat para o cliente", (Exception) re);
			System.out.println ("server.chat.Room.realSend: "+re.getMessage());
		    }

                } //else //System.out.println ("cl nulo");
            }

        } else {
            //System.out.println ("Destiny not found:" + _msg.getDestiny());
        }
    }


    private void event (int _event, String _name) {
	// envia um evento a todos os usuarios

	//System.out.println ("event ("+_event+", "+_name+")");

	// pega a lista de enderecos
	Vector v = new Vector();
	int size = everybody.getUserList(v);

	for (int i = 0; i<size; i++) {

	    ChatUser user  = (ChatUser) v.elementAt(i);
	    iChatClient cl = user.getAddress();

	    if ( cl != null)

		try {

		    cl.receiveEvent (_event, _name);
		    user.resetRefuses();

		} catch (RemoteException re) {

		    user.incRefuses();
		    if (user.getRefuses() >= 3) {
			// there is no response from user, i will log out (s)he
			try {
			    this.out (user.getName());
			} catch (Exception e) {
			}
		    }

		    ErrorLog.logError ("colab.server.chat.Room.event()", "Nao foi possivel enviar evento de chat para o cliente", (Exception) re);
		    System.out.println ("server.chat.Room.send: "+re);

		}
	}
    }

    public String[] getList() throws RemoteException, ServerIsGoingDownException {
        // send user list from this room
	//System.out.println ("getList ()");

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        Vector names = new Vector();
        String s[] = null;
        try{
            everybody.getNames(names);

            s = new String [names.size()];
            s = (String[]) names.toArray(s);

        } catch (Exception e) {
            ErrorLog.logError ("colab.server.chat.Room.getList()", "", e);
            System.out.println ("server.chat.Room.getList: " + e);
        }

        return s;
    }

    //***************************************************************************************
    public iToolModel getRemoteToolModel() throws RemoteException, ServerIsGoingDownException {

	if (null == toolModel)
	    toolModel = toolFactory.createRemoteToolModel ();

	return toolModel;

    }

}
