package colab.server.file;

import java.rmi.*;
import java.rmi.server.*;
import java.io.IOException;
import colab.util.*;
import colab.server.*;
import colab.util.group.*;
import colab.util.file.*;

public class FileServer extends UnicastRemoteObject implements iFileServer {

    String       host;
    boolean      ServerGoingDown = false;
    iGroupServer groupServer;


    public FileServer (String _host, iGroupServer _groupServer) throws RemoteException, IOException {

        host = _host;
        groupServer = _groupServer;

        String name = "//" + host + ":" + RegistryPort.getPort() + "/" + iFileServer.NAME;
        Naming.rebind(name, (iFileServer) this);

    }


    public void exit() throws RemoteException {

	ServerGoingDown = true;

    }


    public iRemoteFileSystem getFileSystem (String _loginName, boolean _load) throws RemoteException, ServerIsGoingDownException {

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        try {
            User user = groupServer.getUserInformation (_loginName);

	    if (user == null)
		return null;

	    String group = user.getGroup().getName();

	    iRemoteFileSystem remoteFileSystem = new RemoteFileSystem (_loginName, group, _load);

	    return remoteFileSystem;

        } catch (RemoteException e) {

            //Will never be throws: local object
            ErrorLog.logError ("colab.server.file.FileServer.getFileSystem()", "", e);

	} catch (ServerIsGoingDownException e) {

	    // Never will be throwed: FileServer fall before GroupServer
            ErrorLog.logError ("colab.server.file.FileServer.getFileSystem()", "", e);

	}

	return null;
    }
}
