package colab.server.file;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.text.StyledDocument;
import java.rmi.*;
import java.rmi.server.*;
import colab.util.*;
import colab.server.*;
import colab.util.file.*;
import colab.util.tool.iToolModel;

public class RemoteFileSystem implements iRemoteFileSystem{

    private String userName, groupName;
    private File[] dirs;
    private File mainRoot = new File ( File.separator );
    private String baseDir;

    private boolean serverGoingDown = false;

    public RemoteFileSystem(String _loginName, String _groupName, boolean _load) throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("Criando RemoteFileSystem");

        try {

            UnicastRemoteObject.exportObject ( (iRemoteFileSystem) this);
	    this.userName  = _loginName;
	    this.groupName = _groupName;

	    this.baseDir  = Persistence.getDirectory (Persistence.HOME_DIR);
	    this.baseDir  = this.baseDir.substring (0, this.baseDir.length()-1);

	    this.getDirs (_loginName, _groupName, _load);

        } catch (Exception e) {

	    ErrorLog.logError ("colab.server.file.RemoteFileSystem.constructor", "", e);
            System.out.println ("Need to make error log. colab.server.file.RemoteFileSystem():" + e.getMessage());
	    e.printStackTrace();

        }

    }

    public File createFileObject(File dir, String filename) throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("createFileObject(file, filename)");

	File f;

	try {

	    f = new File (this.getRealFile(dir), filename);

	} catch (NullPointerException e) {

	    return null;

	}

	return this.getFakeFile (f);
    }

    public File createFileObject(String path) throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("createFileObject (" +path+")");

	File f;

	try {

	    f = new File (baseDir+path);

	} catch (NullPointerException e) {

	    return null;

	}

	return this.getFakeFile (f);
    }

    public File createNewFolder(File containingDir) throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("createNewFolder");

	String baseName = "Nova_Pasta";

	for (int i = 0; i < Integer.MAX_VALUE; i++ ) {

	    String name = baseName;

	    if ( i != 0 )
		name = baseName + i;

	    File dir = new File (this.getRealFile(containingDir), name);
	    if ( !dir.exists()){

		dir.mkdir();
		return this.getFakeFile(dir);

	    }

	}

	return null;
    }

    public File[] getFiles(File dir, boolean useFileHiding) throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("getFiles (" + dir.getAbsolutePath() + ", boolean)");

	File[] files;

	if (dir.equals (mainRoot))
	    files = dirs;
	else
	    files = this.getRealFile(dir).listFiles();

	for (int i = 0; i < files.length; i++)
	    files[i] = this.getFakeFile(files[i]);

	return files;

    }

    public File getHomeDirectory() throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("getHomeDirectory");

	return mainRoot;

    }

    public File getParentDirectory(File dir) throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("getParentDirectory (" + dir.getAbsolutePath() +") (r)");

	if (this.isRoot(dir)) {
	    //System.out.println ("\tE' a raiz ja.");
	    return dir;
	}

	return this.getFakeFile (this.getRealFile(dir).getParentFile());

    }

    public File[] getRoots() throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("getRoots (r)");

	File[] r = {mainRoot};

	return r;
    }

    private void getDirs(String _userDir, String _groupDir, boolean _load) {

	//System.out.println ("getDirs (r)");

	if (_load)
	    dirs = new File [3];
	else
	    dirs = new File [2];

	dirs[0] = new File (baseDir + File.separator + _userDir);
	dirs[1] = new File (baseDir + File.separator + _groupDir);

	if (_load)
	    dirs[2] = new File (Persistence.getDirectory (Persistence.PUBLISH_DIR));

	if ( !dirs[0].exists() )
	    dirs[0].mkdir();
	if ( !dirs[1].exists() )
	    dirs[1].mkdir();

	//System.out.println (dirs[0].getName());
	//System.out.println (dirs[0].getAbsolutePath());
	//System.out.println (dirs[1].getName());
	//System.out.println (dirs[1].getAbsolutePath());
	//System.out.println (mainRoot.getName());
	//System.out.println (mainRoot.getAbsolutePath());

    }

    public boolean isHiddenFile(File f) throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("isHiddenFiles (r)");

	return this.getRealFile(f).isHidden();
    }

    public boolean isRoot(File f) throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("isRoot (" + f.getAbsolutePath()+") (r)");

	if ( mainRoot.equals (f) ) {
	    //System.out.println ("\ttrue");
	    return true;
	}

	//System.out.println ("\tfalse");
	return false;
    }

    public iToolModel openModel (File _file) throws RemoteException, ServerIsGoingDownException, IOException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

	return FilePersistence.readModel (this.getRealFile(_file));

    }

    public void saveModel (File _file, iToolModel _model) throws RemoteException, ServerIsGoingDownException, IOException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

	File file = this.getRealFile(_file);

	String name = _file.getName();

	if ( ! name.endsWith(".func"))
	    file = new File(file.getParent(), name+".func");

	FilePersistence.writeModel (file, _model);

    }

    public void publishModel (iToolModel _model) throws RemoteException, ServerIsGoingDownException, IOException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

	String fileName = Persistence.getDirectory (Persistence.PUBLISH_DIR) + this.groupName;

	if ( ! fileName.endsWith(".func"))
	    fileName = fileName+".func";

	FilePersistence.writeModel (new File (fileName), (iToolModel) _model);

    }

    private File getRealFile (File _file) {

	//System.out.println ("getRealFile ("+_file.getAbsolutePath() + ")");

	String name = baseDir + _file.getAbsolutePath();
	//System.out.println (name);

	File realFile = new File (name);

	return realFile;

    }

    private File getFakeFile (File _file) {

	//System.out.println ("getFakeFile (" + _file.getAbsolutePath() + ")");

	String name = _file.getAbsolutePath();

	if ( name.startsWith (baseDir) ) {

	    name = name.substring (baseDir.length());
	    if (name.equals (""))
		name = File.separator;

	    //System.out.println (_file.getAbsolutePath() + " -> " + name);

	    if (_file.isDirectory()) {
		//System.out.println ("\t is Directory");
		return new RemoteDirectory (name);
	    }

	    return new RemoteFile (name);

	} //else
	    //System.out.println ("\tNao comeca com " +  baseDir);

	return _file;

    }

    public StyledDocument openStyledDocument (File _file) throws RemoteException, ServerIsGoingDownException, IOException {

	//System.out.println ("openStyledDocument");

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

	//System.out.println ("Vou ler: " + this.getRealFile(_file).getAbsolutePath());
	return FilePersistence.readStyledDocument (this.getRealFile(_file));

    }

    public void saveStyledDocument (File _file, StyledDocument _model) throws RemoteException, ServerIsGoingDownException, IOException {

	//System.out.println ("RemoteFileSystem.save(): " + _file.getAbsolutePath());

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

	File file = this.getRealFile(_file);

	//System.out.println ("Nome do arquivo: " + _file.getName());

	String name = _file.getName();

	if ( ! name.endsWith(".relat"))
	    file = new File(file.getParent(), name+".relat");

	FilePersistence.writeStyledDocument (file, _model);

    }
}
