package colab.server.forum;

import java.rmi.*;
import java.rmi.server.*;
import colab.server.*;
import colab.util.forum.*;
import colab.util.*;
import colab.util.group.*;
import java.io.*;
import java.util.*;
import java.net.*;

public class ForumServer extends UnicastRemoteObject implements iForumServer, Runnable {

    ForumModel forumModel;
    ForumTopics forumTopics;
    Map usersMap;
    String host;
    Vector receivedMsgs;
    Thread thread;
    iGroupServer groupServer;

    boolean serverGoingDown = false;

    public ForumServer (String _host, iGroupServer _groupServer) throws RemoteException, IOException {
        super();

        host = _host;
        groupServer = _groupServer;
        String name = "//" + host + ":" + RegistryPort.getPort() + "/" + iForumServer.NAME;
        Naming.rebind(name, (iForumServer) this);

        receivedMsgs = new Vector();
	usersMap = Collections.synchronizedMap(new HashMap());

	forumModel    = this.getForumModel ("forum");
	forumTopics = this.getForumTopics ();

        thread = new Thread (this);
        thread.start();
    }

    private ForumModel getForumModel(String _room) {

	if (forumModel != null)
	    return forumModel;

	forumModel = ForumPersistence.readForumModel (_room);
	if (forumModel != null)
	    return forumModel;

	forumModel = new ForumModel (_room);
	ForumPersistence.writeForumModel (forumModel);

	return forumModel;
    }

    private ForumTopics getForumTopics() {

	if (forumTopics != null)
	    return forumTopics;

	ForumTopics fTopics = ForumPersistence.readForumTopics ();
	if (fTopics != null)
	    return fTopics;

	fTopics = new ForumTopics ();
	ForumPersistence.writeForumTopics (fTopics);

	return fTopics;
    }


    public void send (ForumMsg _forum) throws RemoteException, ServerIsGoingDownException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

        _forum.setDate (new Date());
        receivedMsgs.add (_forum);
    }

    public void exit() throws RemoteException {
        serverGoingDown = true;

        if ( !receivedMsgs.isEmpty() )
            this.realSend();

	ForumPersistence.writeForumModel (forumModel);

	ForumUserModel[] arrayUsers;

	synchronized (usersMap) {
	    arrayUsers = new ForumUserModel[usersMap.size()];
	    Collection col = usersMap.values();
            arrayUsers = (ForumUserModel[]) col.toArray(arrayUsers);

            for (int i = 0; i < arrayUsers.length; i++)
                ForumPersistence.writeUserModel (arrayUsers[i]);
        }
    }

    public void run () {
        //look for messages and send
        while (!serverGoingDown)
	    if (receivedMsgs.isEmpty())
                try {
                    Thread.sleep (5000);
                } catch (InterruptedException e) {
                    System.out.println("Need make error log. ForumServer.run(): "+e.getMessage());
                    ErrorLog.logError ("colab.server.forum.ForumServer.run()", "Problemas para chamar sleep", e);
                }
            else
                this.realSend ();
    }

    private void realSend() {

	while ( ! receivedMsgs.isEmpty()) {
	    ForumMsg forumMsg = (ForumMsg) receivedMsgs.remove(0);

	    try {
		forumModel.receive(forumMsg);
	    } catch (Exception e) {
		//Will never be throws: local object
		System.out.println ("Log error");
		ErrorLog.logError ("colab.server.forum.ForumServer.realSend()", "", e);
	    }
	}
    }

    public int getSize() throws RemoteException, ServerIsGoingDownException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

       	return forumModel.getSize();
    }

    public void login (String _loginName) throws RemoteException, ServerIsGoingDownException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

        boolean exist = true;
        ForumUserModel userModel = (ForumUserModel) usersMap.get (_loginName);

        if (userModel == null) {
	    // New User or UserModel only in persistent memory
	    System.out.println ("Vou ler um");
	    userModel = ForumPersistence.readUserModel (_loginName);

	    if (userModel != null) {

		// forumBox was only in persistent memory
		System.out.println ("Li, tava gravado");
		usersMap.put (_loginName, userModel);

	    } else {

		// Maybe new user
		System.out.println ("nao li, vou ver se o usuarios existe");
		User user = groupServer.getUserInformation (_loginName);

		if (user != null) {
		    //new User
		    System.out.println ("Existe, vou criar forumUserModel");
		    userModel = new ForumUserModel (_loginName);
		    ForumPersistence.writeUserModel (userModel);
		    usersMap.put (user.getLogin(), userModel);
		} else {
		    //user don't exist
		    System.out.println ("Nao existe");
		    exist = false;
		}
	    } //
	} //if (userModel == null)

	if (!exist) {
	    System.out.println ("Mudar a excecao levantada e fazer log de erro");
	    throw new RemoteException ("Usuario inexistente");
	}

    }

    public String[] getTopics() throws  RemoteException, ServerIsGoingDownException {

	if (serverGoingDown)
            throw new ServerIsGoingDownException();

	return forumTopics.getTopics();
    } // public String[] getTopics()


    public String[] getReaded (String _loginName) throws RemoteException, ServerIsGoingDownException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

	ForumUserModel userModel = (ForumUserModel) usersMap.get (_loginName);

        if (userModel == null) {

            System.out.println ("Nao era para acontecer isto. fazer log do erro");
	    return null;
        }  //if (userModel == null)

        return userModel.getReaded();
    } // public String[] getReaded (String _loginName)

    public ForumMsg[] getMsgs() throws RemoteException, ServerIsGoingDownException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

	return forumModel.getMsgs();
    }

    public ForumMsg[] getMsgs(Date _date) throws RemoteException, ServerIsGoingDownException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

	return forumModel.getMsgs(_date);
    }

}
