package colab.server.group;

import java.rmi.*;
import java.rmi.server.*;
import java.util.*;
import colab.util.*;
import colab.util.group.*;
import colab.server.*;
import java.io.*;

public class GroupServer extends UnicastRemoteObject implements iGroupServer, Runnable {


    Map     users;
    Map     groups;
    boolean serverGoingDown = false;
    Map     loggedUsers;
    Vector  events;
    Thread  thread;


    public GroupServer(String host) throws RemoteException, IOException {
        super();

        users  = Collections.synchronizedMap(new HashMap());
        groups = Collections.synchronizedMap(new HashMap());
	events = new Vector();

        String name = "//"+ host+ ":"+ RegistryPort.getPort()+ "/GroupServer";
        Naming.rebind(name, (iGroupServer) this);

        UsersReader.readAll (this, Persistence.getDirectory(Persistence.USERS));

	thread = new Thread (this);
	thread.start();
    }


    public void exit() throws RemoteException {
        serverGoingDown = true;

        new Thread(new Thread () {
            public void run() {
                try {
                    Thread.sleep (10000);
                    System.exit(0);
                } catch (Exception e) {
                }
            }
        }).start();
    }


    protected void insertGroup(Group _group) throws NameAlreadyUsedException {
        Group g = (Group) groups.get (_group.getLogin());

        if (g == null)
            groups.put (_group.getLogin(), _group);
        else
            throw new NameAlreadyUsedException("Ja ha um grupo com este nome.");

        groups.put (_group.getLogin(), _group);
    }


    protected void insertUser (UserPaswd _user) throws NameAlreadyUsedException {
        UserPaswd g = (UserPaswd) users.get (_user.getLogin());

        if (g == null)
            users.put (_user.getLogin(), _user);
        else
            throw new NameAlreadyUsedException("Ja ha um usuario com este nome.");
    }


    public User login (String _login, String _password) throws RemoteException, LoginFailException, ServerIsGoingDownException {

        //System.out.println ("Entrando: "+_login);
        if (serverGoingDown)
            throw new ServerIsGoingDownException ();

        //________ TEMP __________
        //User u = new User (_login, User.STUDENT, _login, _password);
        //users.put(_login, u);
        //________ TEMP __________

        UserPaswd userPsw = (UserPaswd) users.get (_login);
        if (userPsw == null) {
            //System.out.println ("deu pau: nao tem este usuario");
            throw new LoginFailException ("Usuario e/ou senha nao confere");
        }

	if ( !userPsw.isPassword (_password))
	    throw new LoginFailException ("Usuario e/ou senha nao confere");

	if (null == loggedUsers)
	    loggedUsers = Collections.synchronizedMap(new HashMap());

	User user = new User (userPsw);
	LoggedUser lUser = new LoggedUser (user, null, null);

	loggedUsers.put (_login, lUser);


        return user;
    }


    public void logout(String _login) throws RemoteException, LoginFailException, ServerIsGoingDownException{

        //System.out.println ("Saindo: "+_login);

	if (loggedUsers.containsKey (_login)) {

	    LoggedUser lUser = (LoggedUser) loggedUsers.remove (_login);

	    fireEvent (new GroupEvent(USEROUT, new UserInformation(lUser)));
	}

    }


    private void fireEvent (GroupEvent _event) {

	events.add (_event);
    }


    public UserInformation[] subscribe(String _login, iGroupClient _client) throws RemoteException, ServerIsGoingDownException {

	if (null == loggedUsers)
	    return null;

	LoggedUser lUser = (LoggedUser) loggedUsers.get (_login);

	if (null == lUser)
	    return null;

	this.fireEvent (new GroupEvent(USERIN, new UserInformation(lUser)));
	lUser.setGroupClient (_client);

        LoggedUser[] lUsers;

        synchronized (loggedUsers) {
	    lUsers = new LoggedUser[loggedUsers.size()];
	    Collection col = loggedUsers.values();
	    lUsers = (LoggedUser[]) col.toArray(lUsers);
	}

	UserInformation[] users = new UserInformation[lUsers.length];
	for (int i = 0; i < lUsers.length; i++)
	    users[i] = new UserInformation (lUsers[i]);

	return users;
    }


    public String[] getLoginNames () throws RemoteException, ServerIsGoingDownException{
        //return all user names, including group names

        if (serverGoingDown)
            throw new ServerIsGoingDownException ();

        String usersLogin[] = new String[users.size()];
        usersLogin = (String[]) users.keySet().toArray(usersLogin);

        return usersLogin;
    }


    public User getUserInformation(String _login) throws RemoteException, ServerIsGoingDownException {
        //return null if there is no user _login

        if (serverGoingDown)
            throw new ServerIsGoingDownException ();

        UserPaswd userPaswd = (UserPaswd) users.get(_login);

        if (userPaswd == null)
            return null;

        return new User( userPaswd );
    }


    public void userGetInARoom (String _login, String _room) throws RemoteException, ServerIsGoingDownException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException ();

	LoggedUser lUser= (LoggedUser) loggedUsers.get (_login);

	if (lUser != null)
	    lUser.setRoomName (_room);

	this.fireEvent (new GroupEvent(USERMOVE, new UserInformation(lUser)));
    }


    public void userGetOutARoom (String _login, String _room)throws RemoteException, ServerIsGoingDownException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException ();

	LoggedUser lUser= (LoggedUser) loggedUsers.get (_login);

	if (lUser != null)
	    lUser.setRoomName ("");

	this.fireEvent (new GroupEvent(USERMOVE, new UserInformation(lUser)));
    }


    public void run() {

	while (! serverGoingDown) {

	    if (events.isEmpty())
		try {

		    Thread.sleep (5000);

                } catch (InterruptedException e) {
                    System.out.println("Need make error log. ForumServer.run(): "+e.getMessage());
                    ErrorLog.logError ("colab.server.forum.ForumServer.run()", "Problemas para chamar sleep", e);
                }
            else
		this.realFireEvent();
	}
    }


    private void realFireEvent() {

	LoggedUser[] lUsers;

        synchronized (loggedUsers) {

	    lUsers = new LoggedUser[loggedUsers.size()];
	    Collection col = loggedUsers.values();
	    lUsers = (LoggedUser[]) col.toArray(lUsers);
	}

	while ( ! events.isEmpty() ) {

	    GroupEvent event = (GroupEvent) events.remove(0);

	    //System.out.println ("Evento: " + event.getType());

	    for (int i = 0; i < lUsers.length; i++)

		try {

		    iGroupClient client = lUsers[i].getGroupClient();

		    if (null != client)
			client.receiveEvent(event);

		    lUsers[i].resetRefuses();

		} catch (java.rmi.ConnectException ce) {

		    String ref = "";
		    lUsers[i].incRefuses();
		    if (3 == lUsers[i].getRefuses()) {
			try {
			    this.logout (lUsers[i].getLogin());
			} catch (Exception e) {
			}
			//ref = " [LOGOUT DE " + lUsers[i].getLogin() + "] ";
		    }

		    ErrorLog.logError ("colab.server.group.GroupServer.fireEvent()", ref + "Nao foi possivel enviar evento "+eventNames[event.getType()]+" para o cliente", (Exception) ce);
		    System.out.println ("server.group.GroupServer.fireEvent(): "+ce.getMessage());
		    ce.printStackTrace();

		} catch (RemoteException re) {

		    ErrorLog.logError ("colab.server.group.GroupServer.fireEvent()", "Nao foi possivel enviar evento "+eventNames[event.getType()]+" para o cliente", (Exception) re);
		    System.out.println ("server.group.GroupServer.fireEvent(): "+re.getMessage());
		    re.printStackTrace();

		}
	}
    }
}
