package colab.server.mail;

import java.rmi.*;
import java.rmi.server.*;
import colab.server.*;
import colab.util.*;
import colab.util.mail.*;
import colab.util.group.User;
import java.io.*;
import java.util.*;

public class MailBox implements iMailBox, Serializable, iPersistent {

    private User   user;
    private long   uniqueID;
    private Map    mailMsgs;

    private transient boolean login;

    private transient boolean ServerGoingDown = false;

    public MailBox(User _user) throws RemoteException {
        user = _user;
        uniqueID = 0;
        mailMsgs = Collections.synchronizedMap(new HashMap());

        this.makeExportable();
    }

    public void exit() {

        ServerGoingDown = true;
    }

    public void beforeSave() {
        //
    }

    public void afterSave(){
        //
        ServerGoingDown = false;
        this.makeExportable();
    }

    protected void setLogged (boolean _login) {
        login = _login;
    }

    protected boolean getLogged () {
        return login;
    }

    protected void makeExportable() {
        try {
            UnicastRemoteObject.exportObject ( (iMailBox) this);
        } catch (Exception e) {
            ErrorLog.logError ("colab.server.mail.MailBox.makeExportable()", "Erro ao tentar exportar objeto", e);
            System.out.println ("Need to make error log. MailBox.makeExportable():" + e.getMessage());
        }
    }

    protected String getLogin() {
        return user.getLogin();
    }

    public User getUser() throws RemoteException, ServerIsGoingDownException {
        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        return user;
    }

    protected String getNextID () {
        String id = user.getLogin();
        uniqueID++;
        return id+String.valueOf(uniqueID);
    }

    protected void receive (PersonalizedMailMsg _mail) {
        mailMsgs.put (_mail.getId(), _mail);
    }

    public void delete (String id) throws RemoteException, ServerIsGoingDownException{

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        mailMsgs.remove (id);
    }

    public iMailMsg[] getList() throws RemoteException, ServerIsGoingDownException{

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        iMailMsg[] arrayMail = null;

        try {

            synchronized(mailMsgs) {
                int size = mailMsgs.size();
                arrayMail = new PersonalizedMailMsg[mailMsgs.size()];
                Collection col = mailMsgs.values();
                arrayMail = (iMailMsg[]) col.toArray(arrayMail);
            }
        } catch (Exception e) {
            ErrorLog.logError ("colab.server.mail.MailBox.getList()", "", e);
            System.out.println ("Need to make error log. MailBox.getList():" + e.getMessage());
        }

        return arrayMail;
    }

    public int getSize() throws RemoteException, ServerIsGoingDownException {

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        //System.out.println ("getSize() = " +mailMsgs.size());
        return mailMsgs.size();
    }

    public String getBody (String id) throws RemoteException, ServerIsGoingDownException{

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        MailMsg mail = (MailMsg) mailMsgs.get (id);

        if (mail == null)
            return null;

        return mail.getBody();
    }

    public void setReaded(String id) throws RemoteException, ServerIsGoingDownException{

        if (ServerGoingDown)
            throw new ServerIsGoingDownException();

        PersonalizedMailMsg mail = (PersonalizedMailMsg) mailMsgs.get (id);

        if (mail != null)
            mail.setReaded (true);
    }

}
