package colab.server.mail;

import java.rmi.*;
import java.rmi.server.*;
import colab.server.*;
import colab.util.*;
import colab.util.mail.*;
import colab.util.group.*;
import java.io.*;
import java.util.*;
import java.net.*;

public class MailServer extends UnicastRemoteObject implements iMailServer, Runnable {

    Map mailBoxes;
    String host;
    Vector receivedMsgs;
    Thread thread;
    iGroupServer groupServer;

    boolean serverGoingDown = false;

    public MailServer(String _host, iGroupServer _groupServer) throws RemoteException, IOException {
        super();

        host = _host;
        groupServer = _groupServer;
        String name = "//" + host + ":" + RegistryPort.getPort() + "/" + iMailServer.NAME;
        Naming.rebind(name, (iMailServer) this);

        mailBoxes = Collections.synchronizedMap(new HashMap());
        receivedMsgs = new Vector();
        thread = new Thread (this);
        thread.start();
    }

    public void send (MailMsg _mail) throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("Enviando mensagem de " + _mail.getSender() + " para " + _mail.getDestiny());
        if (serverGoingDown)
            throw new ServerIsGoingDownException();

        _mail.setDate (new Date());
        receivedMsgs.add (_mail);
    }

    public void exit() throws RemoteException {
        serverGoingDown = true;

        if ( ! receivedMsgs.isEmpty() )
            this.realSend();

        MailBox[] arrayBoxes;

        synchronized (mailBoxes) {
	    arrayBoxes = new MailBox[mailBoxes.size()];
	    Collection col = mailBoxes.values();
            arrayBoxes = (MailBox[]) col.toArray(arrayBoxes);

            for (int i = 0; i < arrayBoxes.length; i++)
                arrayBoxes[i].exit();

            for (int i = 0; i < arrayBoxes.length; i++)
                MailPersistence.writeMailBox (arrayBoxes[i], arrayBoxes[i].getLogin());
        }

    }

    public void run () {
        //look for messages and send
        while (! serverGoingDown)
           if (receivedMsgs.isEmpty())
                try {
                    Thread.sleep (5000);
                } catch (InterruptedException e) {
                    System.out.println("Need make error log. MailServer.run(): "+e.getMessage());
                    ErrorLog.logError ("colab.server.mail.MailServer.run()", "Problemas para chamar sleep", e);
                }
	   else {
	       System.out.println (".run: tem mensagens vou mandar realmente");
	       this.realSend ();
	   }
    }

    private void realSend() {
        MailMsg mailMsg = (MailMsg) receivedMsgs.remove(0);

        // First, get the destiny (if it exists)
        String destinyName = mailMsg.getDestiny();

        try {
            User user = groupServer.getUserInformation (destinyName);
            if (user == null) {

		System.out.println ("Send reply to this mail: unknown destiny");

	    } else {

		//System.out.println ("enviando para " +  mailMsg.getDestiny());

		// Generate mail ID
                String senderName = mailMsg.getSender();
		MailBox senderBox = this.getFullAccessMailBox(senderName);
                String newId = senderBox.getNextID();
                mailMsg.setId ( newId );

                Vector mailTo = new Vector();
                int size = user.getLogins (mailTo);

		for (int i = 0; i < size; i++) {

		    String destName = (String) mailTo.elementAt(i);
		    MailBox destBox = this.getFullAccessMailBox (destName);

                    if (destBox != null) {
                        destBox.receive (new PersonalizedMailMsg (mailMsg));
                    } else {
                        //Make error log
                    }
                }

                //Save (persistent) this mail message
                MailPersistence.writeMailMsg (mailMsg);
            }
        } catch (RemoteException e) {

            //Will never be throws: local object
            System.out.println ("Log error");
            ErrorLog.logError ("colab.server.mail.MailServer.realSend()", "", e);

        } catch (ServerIsGoingDownException e) {

            // Never will be throwed: MailServer fall before GroupServer
            ErrorLog.logError ("colab.server.mail.MailServer.realSend()", "", e);

        }
    }

    private MailBox getFullAccessMailBox (String _loginName) {

	MailBox mb = (MailBox) mailBoxes.get (_loginName);

        if (mb == null) {

            if ( mailBoxes.containsKey(_loginName) ) {

                //restore mail box from file
                mb = MailPersistence.readMailBox (_loginName);
                mailBoxes.put (_loginName, mb);

            } else {

                //There is no MailBox for that user, verify if (s)he exists
                try {

                    User user = groupServer.getUserInformation (_loginName);

                    if (user != null) {
                        //(s)he exists, create MailBox
                        mb = new MailBox (user);
                        MailPersistence.writeMailBox (mb, user.getLogin());
                        mailBoxes.put (user.getLogin(), mb);
                    } else {
                        //user doesn't exist
                        return null; // throws some exception ?
                    }

                } catch (Exception e) {
                    ErrorLog.logError ("colab.server.mail.MailServer.getFullAccessMailBox()", "", e);
                    System.out.println ("1Need to make error log. MailServer.getFullAccessMailBox(): "+ e.getMessage());
                    return null;
                } //catch (Exception e) {
            } // else {
        } //if (mb == null)

        return mb;
    } // private MailBox getFullAccessMailBox (String _loginName)


    public void login (String _loginName) throws RemoteException, ServerIsGoingDownException {

        if (serverGoingDown)
            throw new ServerIsGoingDownException();

        boolean exist = true;
        MailBox mb = (MailBox) mailBoxes.get (_loginName);

        if (mb == null) {

            // New User or MailBox only in persistent memory
            mb = MailPersistence.readMailBox (_loginName);

            if (mb != null) {

                // mailBox was only in persistent memory
                mailBoxes.put (_loginName, mb);
		mb.makeExportable();

            } else {

                // Maybe new user
                User user = groupServer.getUserInformation (_loginName);

                if (user != null) {
                    //new User
                    mb = new MailBox (user);
                    MailPersistence.writeMailBox (mb, user.getLogin());
                    mailBoxes.put (user.getLogin(), mb);
                } else {
                    //user don't exist
                    exist = false;
                }
            } //
        } //if (mb == null)

        if (!exist) {
            System.out.println ("Mudar a excecao levantada e fazer log de erro");
            throw new RemoteException ("Usuario inexistente");
        }
    }

    public iMailBox getMailBox(String _loginName) throws RemoteException, ServerIsGoingDownException {

	//System.out.println ("Buscando MailBox para " + _loginName);
        if (serverGoingDown)
            throw new ServerIsGoingDownException();

        MailBox mb = (MailBox) mailBoxes.get (_loginName);

        if (mb == null) {

            System.out.println (" Nao era para acontecer isto");

	    if ( mailBoxes.containsKey(_loginName) ) {

                //restore mail box from file
                mb = MailPersistence.readMailBox (_loginName);
                mailBoxes.put (_loginName, mb);

            } else {

                //There is no MailBox for that user, verify if (s)he exists
                try {

                    User user = groupServer.getUserInformation (_loginName);

                    if (user != null) {
                        //(s)he exists, create MailBox
                        mb = new MailBox (user);
                        MailPersistence.writeMailBox (mb, user.getLogin());
                        mailBoxes.put (user.getLogin(), mb);
                    } else {
                        //user doesn't exist
                        return null; // throws some exception ?
                    }

                } catch (Exception e) {
                    ErrorLog.logError ("colab.server.mail.MailServer.getMailBox()", "", e);
                    System.out.println ("1Need to make error log. MailServer.getMailBox(): "+ e.getMessage());
                    e.printStackTrace();
                    return null;
                } //catch (Exception e) {
            } // else {
        } //if (mb == null)

        return (iMailBox) mb;
    } // public iMailBox getMailBox(String _loginName)

}
