package colab.util.tool.ploter;

import colab.util.tool.*;
import java.io.*;
import java.awt.geom.*;
import java.awt.*;
import java.util.*;
import java.rmi.*;

public class PloterModel implements iPloterModel, Serializable, Runnable {

    protected Bound  bound;
    protected Map    functions;
    protected Vector notes;

    protected transient Vector listeners;
    protected transient Vector events;
    protected transient Thread thread;

    protected Long   uniqueNumberName;

    protected int sleepTime = 500;

    public PloterModel () throws RemoteException {
	this.bound     = new Bound ();
	this.functions = Collections.synchronizedMap(new HashMap());
	events         = new Vector();
	this.uniqueNumberName  = new Long(1);

	new Thread(this).start();
    }


    public PloterModel (Bound _bound) throws RemoteException {
	this.bound     = _bound;
	this.functions = Collections.synchronizedMap(new HashMap());
	events         = new Vector();
	this.uniqueNumberName  = new Long(1);

	new Thread(this).start();
    }


    public Function[] getFunctionList() throws RemoteException {
	//System.out.println ("\tgetFunctionList ()");

	Function[] arrayFunction;

	synchronized (this.functions) {
	    //arrayFunction = new Function [this.functions.size()];
	    Collection col = this.functions.values();
	    arrayFunction = (Function[]) col.toArray(new Function[this.functions.size()]);
	}

	//for (int i = 0; i < arrayFunction.length; i++)
	    //System.out.println ("\t\t funcao " + i + ": " + arrayFunction[i].getName() + " \t " + arrayFunction[i].getFunctionString() + " \t " + arrayFunction[i].getId());
	return arrayFunction;
    }

    public PloterModel cloneModel () throws RemoteException {

	PloterModel cloneModel = new PloterModel (bound);

	Function[] arrayFunction;
	synchronized (this.functions) {
	    Collection col = functions.values();
	    arrayFunction = (Function[]) col.toArray(new Function[functions.size()]);
	}

	for (int i = 0; i < arrayFunction.length ; i++) {
	    Function func = (Function) arrayFunction[i].clone();
	    func.setFunctionLock (null);
	    //System.out.println ("Colocando a funcao { " + func.getName() + " = " + func.getFunctionString() + " } no clone");
	    cloneModel.addFunction (func);
	}

	return cloneModel;
    }


    public void setModel (PloterModel _model) throws RemoteException, FunctionLockException {

	//System.out.println ("Vou tentar alterar o Modelo");

	synchronized (this.functions) {
	    if (this.verifyLocks())
		// if there is lock
		throw new FunctionLockException ("Nao posso alterar o modelo pois ha funes travadas.");

	    this.removeAllFunctions();
	    this.bound    = _model.bound;
	    this.uniqueNumberName = _model.uniqueNumberName;
	    this.notes    = _model.notes;

	    Collection col = _model.functions.values();
	    Function[] arrayFunction = (Function[]) col.toArray(new Function[_model.functions.size()]);

	    for (int i = 0; i < arrayFunction.length; i++)
		this.addFunction (arrayFunction[i]);
	}

    }


    // return true if there is at least one function locked
    public boolean verifyLocks() throws RemoteException {

	return false;

    }

    public String addFunction(Function _func)  throws RemoteException{

	//System.out.println ("Verificar se o nome ja existe, porque senao vai sobrepor");

	String id = this.getValidId();
	//System.out.print ("inserindo funcao com ID = " + _func.getId());
	_func.setId (id);

	//System.out.println (" => ID = "+ _func.getId());
	this.functions.put (_func.getId(), _func);

	this.fireEvent (new PloterModelEvent (PloterModelEvent.FUNCTION_INSERTED, _func.getId(), _func, null));

	return id;
    }


    public void changeFunction (String _id, Function _func, FunctionLock _functionLock)
	throws RemoteException, FunctionLockException, FunctionNotFoundException {

	//System.out.println ("changeFunction (" + _id + ", " + _func.getName()+ ", " + _func.getFunctionString()+")");

	if (this.functions.containsKey (_id)) {

	    this.functions.put (_id, _func);

	    this.fireEvent (new PloterModelEvent (PloterModelEvent.FUNCTION_CHANGED, _id, _func, null));
	}

    }


    public void changeColor (String _id, Color _color) throws RemoteException {

	Function f = (Function) this.functions.get (_id);

	if (null != f)
	    f.setColor (_color);

	this.fireEvent (new PloterModelEvent (PloterModelEvent.FUNCTION_CHANGED, _id, f, null));
    }


    public Function getFunction (String _id) throws RemoteException {

	return (Function) this.functions.get (_id);

    }


    public boolean removeFunction (String _id)  throws RemoteException, FunctionLockException {

	//System.out.print ("removendo funcao " + _id);

	Function f = (Function) this.functions.get (_id);

	f = (Function) this.functions.remove (_id);

	if (null == f) {
	    //System.out.println (" ... nao existe");
	    return false;
	}

	this.fireEvent (new PloterModelEvent (PloterModelEvent.FUNCTION_REMOVED, _id, f, null));

	return true;
    }


    public void removeAllFunctions() throws RemoteException, FunctionLockException {

	//System.out.print ("removendo TODAS as funcoes");

	synchronized (this.functions) {
	    Collection col = this.functions.values();
	    Function[] arrayFunction = (Function[]) col.toArray(new Function[this.functions.size()]);

	    for (int i = 0; i <  arrayFunction.length; i++)
		this.removeFunction (arrayFunction[i].getId());
	}
    }


    public Note[] getNoteList()  throws RemoteException{
	return null;
    }


    public void addNote(Note _note) throws RemoteException{
	this.notes.add(_note);
    }


    public boolean removeNote (Note _note)  throws RemoteException{
	return this.notes.remove (_note);
    }


    public String getValidName()  throws RemoteException {

	String name = "funcao ";

	synchronized (this.uniqueNumberName) {
	    name = name + this.uniqueNumberName;
	    uniqueNumberName = new Long (1 + uniqueNumberName.longValue());
	}

	return name;
    }


    public String getValidId()  throws RemoteException {

	//System.out.println ("Retornando um Id valido (unico)");
	this.getFunctionList();

	String id = (new Date()).toString().substring(11,19);
	String validId = id;

	int idNumber = 1;

	while (functions.containsKey (validId))
	    validId = id + (idNumber++);

	//System.out.println ("\tid = " + validId);

	return validId;
    }


    public Bound getBound () throws RemoteException {
	return this.bound;
    }


    public void setBound (Bound _bound) throws RemoteException {
	this.bound.x1 = _bound.x1;
	this.bound.y1 = _bound.y1;
	this.bound.x2 = _bound.x2;
	this.bound.y2 = _bound.y2;

	this.fireEvent (new PloterModelEvent (PloterModelEvent.BOUND_CHANGED, null, null, this.bound));
    }


//      public void addPloterListener(iPloterListener _pListener) throws RemoteException{

//  	//System.out.println ("Inserindo listener");

//  	if (null == this.listeners)
//  	    this.listeners = new Vector();

//  	PloterListenerData listener = new PloterListenerData (_pListener);
//  	this.listeners.add (listener);
//      }


    public void addListener(iListener _listener) throws RemoteException{

	//System.out.println ("Inserindo listener");

	if (null == this.listeners)
	    this.listeners = new Vector();

	PloterListenerData listener = new PloterListenerData ((iPloterListener)_listener);
	this.listeners.add (listener);
    }


//      public void removePloterListener(iPloterListener _pListener) throws RemoteException{

//  	//System.out.print ("Removendo listener");

//  	if (null != this.listeners)
//  	    if (this.listeners.remove (new PloterListenerData (_pListener)))
//  		{}//System.out.println (" ...ok");
//  	    else
//  		{}//System.out.println (" ...not ok");

//      }


    public void removeListener(iListener _listener) throws RemoteException{

	//System.out.print ("Removendo listener");

	if (null != this.listeners)
	    if (this.listeners.remove (new PloterListenerData ((iPloterListener) _listener)))
		{}//System.out.println (" ...ok");
	    else
		{}//System.out.println (" ...not ok");

    }


    public void run () {
	// who will real fire events
	while (true ) {
	    if (events.isEmpty())
		try {

		    Thread.sleep (sleepTime);

                } catch (InterruptedException e) {
                }

	    else {
		Vector invalidListeners = new Vector();

		while (! events.isEmpty()) {
		    PloterModelEvent event = (PloterModelEvent) events.remove (0);
		    //System.out.println ("\nVou mandar evento:");

		    if (listeners != null) {
			PloterListenerData[] pListeners;
			synchronized (this.listeners) {

			    pListeners = (PloterListenerData[]) listeners.toArray(new PloterListenerData[listeners.size()]);

			}

			for (int i = 0; i < pListeners.length; i++) {

			    //System.out.println("\tlistener " + i);

			    try {

				pListeners[i].getPloterListener().ploterChanged (event);
				pListeners[i].resetRefuses();
				//System.out.println ("\t\tok");

			    } catch (RemoteException e) {

				pListeners[i].incRefuses();
				if (pListeners[i].getRefuses() >= 3) {

				    invalidListeners.add (pListeners[i]);
				this.listeners.remove (pListeners[i]);
				}

				//System.out.println ("\t\tProblems with fireEvent: " + e.getMessage());

			    }
			} // for
		    } // if (listeners != null)

		} // while (! events.isEmpty()) {

		if ( ! invalidListeners.isEmpty() )
		    this.removeInvalidListeners2 (invalidListeners);

	    } //else

	}

    }

    private void removeInvalidListener (PloterListenerData _pListener){

	//System.out.println ("removeInvalidListener");
	// Need see if exist a function locked by this listener
	Function[] arrayFunction;
	synchronized (this.functions) {

	    Collection col = functions.values();
	    arrayFunction = (Function[]) col.toArray(new Function[functions.size()]);

	}

	for (int i = 0; i < arrayFunction.length; i++) {

	    FunctionLock fL = arrayFunction[i].getFunctionLock();
	    //System.out.println ("\nTrying function " + arrayFunction[i].getName());

	    if (fL != null)
		//System.out.println ("\tthere is a lock:");
		//System.out.println ("\n\t\tfunction:" + _pListener.getPloterListener());
		//System.out.println ("\n\t\tinvalid: " + fL.getOwnerReference());
		if (_pListener.getPloterListener().equals (fL.getOwnerReference()))
		    try {

			//System.out.println ("\n\t\tREMOVENDO lock da funcao: " + arrayFunction[i].getName());
			this.unlockFunction (fL, arrayFunction[i].getId());
			i = arrayFunction.length;

		    } catch (FunctionLockException e) {

			System.out.println ("Nao removi o lock: FunctionLockException");

		    } catch (RemoteException e) {

			System.out.println ("Nao removi o lock: RemoteException");

		    }

	}

	this.listeners.remove (_pListener);
    }

    private void removeInvalidListeners2 (Vector _listeners){

	//System.out.println ("removeInvalidListener");

	// Need see if exist a function locked by this listener
	Function[] arrayFunction;
	synchronized (this.functions) {

	    Collection col = functions.values();
	    arrayFunction = (Function[]) col.toArray(new Function[functions.size()]);

	}

	for (int i = 0; i < arrayFunction.length; i++) {

	    FunctionLock fL = arrayFunction[i].getFunctionLock();
	    //System.out.println ("Trying function " + arrayFunction[i].getName());

	    if (fL != null)
		//System.out.println ("\n\tthere is a lock:");
		//System.out.println ("\n\t\tfunction:" + fL.getOwnerReference());

		for (int j = 0; j < _listeners.size(); j++) {

		    PloterListenerData pListener = (PloterListenerData) _listeners.elementAt (j);

		    if (pListener.getPloterListener().equals (fL.getOwnerReference()))
			try {

			    //System.out.println ("\n\t\tREMOVENDO lock da funcao: " + arrayFunction[i].getName());
			    this.unlockFunction (fL, arrayFunction[i].getId());
			    j = _listeners.size();

		    } catch (FunctionLockException e) {

			System.out.println ("Nao removi o lock: FunctionLockException");

		    } catch (RemoteException e) {

			System.out.println ("Nao removi o lock: RemoteException");

		    }
		}

	}

    }

    public void fireEvent (iEvent _event) throws RemoteException{

      events.add ((PloterModelEvent) _event);

    }

//      protected void fireEvent (PloterModelEvent _e) {

//  	events.add (_e);

//      }


    private void realFire() {

	if (null == thread) {
	    thread = new Thread (this);
	}

	//if ( ! thread.isAlive )

    }

    public void setAllInvisible () throws RemoteException {

	synchronized (this.functions) {

	    Collection col = functions.values();
	    Function[] arrayFunction = (Function[]) col.toArray(new Function[functions.size()]);

	    for (int i = 0; i < arrayFunction.length; i++)
		arrayFunction[i].setInvisible(true);
	}
    }


    public boolean lockFunction (String _functionId, FunctionLock _lock) throws RemoteException, FunctionLockException , FunctionNotFoundException {

	return false;

    }


    public void unlockFunction (FunctionLock _functionLock, String _functionId) throws RemoteException, FunctionLockException {

    }

}
